<?php

namespace Glopal\Connector\Block\Adminhtml\Order\View;

use Glopal\Connector\Api\Data\OrderAttributeInterface;
use Glopal\Connector\Api\Data\OrderTotalCostInterface;
use Magento\Framework\Exception\LocalizedException;
use Magento\Sales\Api\Data\OrderInterface;
use Magento\Sales\Block\Adminhtml\Order\AbstractOrder;

class ExtraShippingChargesInfo extends AbstractOrder
{
    /** @var OrderAttributeInterface */
    private $glAttributes;

    /** @var OrderTotalCostInterface */
    private $glTotalCost;

    /**
     * Is available?
     *
     * @return bool
     * @throws LocalizedException
     */
    public function isAvailable(): bool
    {
        $order = $this->getOrder();

        return null !== $this->getGlopalAttributes($order) && !empty($this->getShippingCharges());
    }

    /**
     * Retrieve shipping charges.
     *
     * @return array
     * @throws LocalizedException
     */
    public function getShippingCharges(): array
    {
        $shippingCharges = [];
        $order = $this->getOrder();
        $totalCost = $this->getGlopalTotalCost($order);

        if (null === $totalCost) {
            return $shippingCharges;
        }

        $shippingCharges['amount'] = [
            'label' => 'Amount',
            'value' => $this->getPriceHtml(
                $order,
                $order->getBaseShippingAmount(),
                $order->getShippingAmount()
            )
        ];

        if ($totalCost->getShippingDutyAmount()) {
            $shippingCharges['duty'] = [
                'label' => 'Duty',
                'value' => $this->getPriceHtml(
                    $order,
                    $totalCost->getBaseShippingDutyAmount(),
                    $totalCost->getShippingDutyAmount()
                )
            ];
        }

        $shippingCharges['tax'] = [
            'label' => 'Tax',
            'value' => $this->getPriceHtml(
                $order,
                $order->getBaseShippingTaxAmount() + $totalCost->getShippingTaxOnDutyAmount(),
                $order->getShippingTaxAmount() + $totalCost->getShippingTaxOnDutyAmount()
            )
        ];

        $totalValue = $order->getShippingAmount()
            + $totalCost->getShippingDutyAmount()
            + $order->getShippingTaxAmount()
            + $totalCost->getShippingTaxOnDutyAmount();
        $totalBaseValue = $order->getBaseShippingAmount()
            + $totalCost->getBaseShippingDutyAmount()
            + $order->getBaseShippingTaxAmount()
            + $totalCost->getBaseShippingTaxOnDutyAmount();
        $shippingCharges['total'] = [
            'label' => 'Total',
            'value' => $this->getPriceHtml(
                $order,
                $totalBaseValue,
                $totalValue
            )
        ];

        return $shippingCharges;
    }

    /**
     * Retrieve extra charges.
     *
     * @return array
     * @throws LocalizedException
     */
    public function getExtraCharges(): array
    {
        $extraCharges = [];
        $order = $this->getOrder();

        $orderAttribute = $this->getGlopalAttributes($order);
        if (null === $orderAttribute) {
            return $extraCharges;
        }

        $extraAmount = $orderAttribute->getExtraAmount();
        if (empty($extraAmount)) {
            return $extraCharges;
        }

        $extraCharges['amount'] = [
            'label' => 'Amount',
            'value' => $this->getPriceHtml(
                $order,
                $orderAttribute->getBaseExtraAmount(),
                $orderAttribute->getExtraAmount()
            )
        ];

        $totalCost = $this->getGlopalTotalCost($order);
        if (null !== $totalCost) {
            $extraCharges['duty'] = [
                'label' => 'Duty',
                'value' => $this->getPriceHtml(
                    $order,
                    $totalCost->getBaseExtraDutyAmount(),
                    $totalCost->getExtraDutyAmount()
                )
            ];
        }

        $extraTaxAmount = $orderAttribute->getExtraTaxOnAmount();
        $baseExtraTaxAmount = $orderAttribute->getBaseExtraTaxOnAmount();
        $taxOnDuty = (null === $totalCost) ? 0.0 : $totalCost->getExtraTaxOnDutyAmount();
        $baseTaxOnDuty = (null === $totalCost) ? 0.0 : $totalCost->getBaseExtraTaxOnDutyAmount();

        $extraCharges['tax'] = [
            'label' => 'Tax',
            'value' => $this->getPriceHtml(
                $order,
                $baseExtraTaxAmount + $baseTaxOnDuty,
                $extraTaxAmount + $taxOnDuty
            )
        ];

        $extraDutyAmount = (null === $totalCost) ? 0.0 : $totalCost->getExtraDutyAmount();
        $extraTotal = $orderAttribute->getExtraAmount()
            + $orderAttribute->getExtraTaxOnAmount()
            + $extraDutyAmount
            + $taxOnDuty;
        $baseExtraDutyAmount = (null === $totalCost) ? 0.0 : $totalCost->getBaseExtraDutyAmount();
        $baseExtraTotal = $orderAttribute->getBaseExtraAmount()
            + $orderAttribute->getBaseExtraTaxOnAmount()
            + $baseExtraDutyAmount
            + $baseTaxOnDuty;

        $extraCharges['total'] = [
            'label' => 'Total',
            'value' => $this->getPriceHtml(
                $order,
                $baseExtraTotal,
                $extraTotal
            )
        ];

        return $extraCharges;
    }

    /**
     * Retrieve Glopal total cost object.
     *
     * @param OrderInterface $order
     * @return OrderTotalCostInterface|null
     */
    private function getGlopalTotalCost(OrderInterface $order): ?OrderTotalCostInterface
    {
        if ($this->glTotalCost) {
            return $this->glTotalCost;
        }

        $orderExtensionAttributes = $order->getExtensionAttributes();
        if (null === $orderExtensionAttributes) {
            return null;
        }

        if (!$orderExtensionAttributes->getGlTcc()) {
            return null;
        }

        $this->glTotalCost = $orderExtensionAttributes->getGlTcc();

        return $this->glTotalCost;
    }

    /**
     * Retrieve Glopal attributes object.
     *
     * @param OrderInterface $order
     * @return OrderAttributeInterface|null
     */
    private function getGlopalAttributes(OrderInterface $order): ?OrderAttributeInterface
    {
        if ($this->glAttributes) {
            return $this->glAttributes;
        }

        $orderExtensionAttributes = $order->getExtensionAttributes();
        if (null === $orderExtensionAttributes) {
            return null;
        }

        if (!$orderExtensionAttributes->getGlAttribute()) {
            return null;
        }

        $this->glAttributes = $orderExtensionAttributes->getGlAttribute();

        return $this->glAttributes;
    }

    /**
     * Retrieve price html with correct format.
     *
     * @param OrderInterface $order
     * @param float $basePrice
     * @param float $price
     * @param string $separator
     * @return string
     */
    private function getPriceHtml(
        OrderInterface $order,
        float $basePrice,
        float $price,
        string $separator = ' '
    ): string {
        $res = '';

        if ($order->isCurrencyDifferent()) {
            $res .= $order->formatBasePrice($basePrice);
            $res .= $separator;
            $res .= '[' . $order->formatPrice($price) . ']';
        } elseif ($order) {
            $res = $order->formatPrice($price);
        }

        return $res;
    }
}
