<?php

namespace Glopal\Connector\Block\Adminhtml\System\Config;

use Exception;
use Glopal\Connector\Helper\Configuration;
use Glopal\Connector\Helper\CurlRequest;
use Glopal\Connector\Helper\Installation;
use Glopal\Connector\Helper\ModuleData;
use Magento\Backend\Block\Template\Context;
use Magento\Backend\Model\Auth\Session;
use Magento\Config\Block\System\Config\Form\Field;
use Magento\Framework\App\Area;
use Magento\Framework\Data\Form\Element\AbstractElement;
use Magento\Framework\DataObjectFactory;
use Magento\Framework\Json\DecoderInterface;
use Magento\Framework\Json\EncoderInterface;
use Magento\Store\Model\App\Emulation;
use Magento\Store\Model\ScopeInterface;
use Magento\Theme\Block\Html\Header\Logo;

/**
 * @SuppressWarnings(PHPMD.CouplingBetweenObjects)
 */
class Settings extends Field
{
    /** @var CurlRequest */
    private $curlRequest;

    /** @var Configuration */
    private $configuration;

    /** @var ModuleData */
    private $helper;

    /** @var Installation */
    private $installationHelper;

    /** @var DataObjectFactory */
    private $objectFactory;

    /** @var Session */
    private $authSession;

    /** @var Emulation */
    private $emulation;

    /** @var Logo */
    private $logo;

    /** @var EncoderInterface */
    private $jsonEncoder;

    /** @var DecoderInterface */
    private $jsonDecoder;

    /**
     * @param Context $context
     * @param Installation $installationHelper
     * @param CurlRequest $curlRequest
     * @param Configuration $configuration
     * @param ModuleData $helper
     * @param DataObjectFactory $objectFactory
     * @param EncoderInterface $jsonEncoder
     * @param DecoderInterface $jsonDecoder
     * @param Session $authSession
     * @param Emulation $emulation
     * @param Logo $logo
     * @param array $data
     *
     * @SuppressWarnings(PHPMD.ExcessiveParameterList)
     */
    public function __construct(
        Context $context,
        Installation $installationHelper,
        CurlRequest $curlRequest,
        Configuration $configuration,
        ModuleData $helper,
        DataObjectFactory $objectFactory,
        EncoderInterface $jsonEncoder,
        DecoderInterface $jsonDecoder,
        Session $authSession,
        Emulation $emulation,
        Logo $logo,
        array $data = []
    ) {
        parent::__construct($context, $data);

        $this->helper = $helper;
        $this->configuration = $configuration;
        $this->curlRequest = $curlRequest;
        $this->installationHelper = $installationHelper;
        $this->objectFactory = $objectFactory;
        $this->jsonEncoder = $jsonEncoder;
        $this->jsonDecoder = $jsonDecoder;
        $this->authSession = $authSession;
        $this->emulation = $emulation;
        $this->logo = $logo;
    }

    /**
     * Gets element html.
     *
     * @param AbstractElement $element
     * @return string
     *
     * @SuppressWarnings(PHPMD.UnusedFormalParameter)
     */
    protected function _getElementHtml(AbstractElement $element): string
    {
        $storeId = $this->getStoreId();

        try {
            $notified = $storeId && $this->installationHelper->isStoreInstallationNotified($storeId);

            if ($storeId && !$notified) {
                $this->installationHelper->notifyInstallation(true);
                $notified = $this->installationHelper->isStoreInstallationNotified($storeId);
            }

            return $this->_renderExternalUi($storeId, $notified);
        } catch (Exception $e) {
            $this->configuration->saveGeneralValue(
                'last_request_error_load',
                $e->getMessage() . "\n" . $e->getTraceAsString(),
                ScopeInterface::SCOPE_STORES,
                $storeId
            );
            $this->configuration->reinitialiseConfig();

            return $this->renderError($e->getCode());
        }
    }

    /**
     * Render external UI.
     *
     * @param int|string $storeId
     * @param bool $notified
     * @return string
     * @throws Exception
     */
    private function _renderExternalUi($storeId, bool $notified): string
    {
        $data = $this->getExternalUiData($storeId, $notified);
        $config = $this->buildExternalUiConfig($data, $storeId);
        $src = $data['script_src'];

        return $this->renderHtml($src, $config);
    }

    /**
     * Create configuration for external js
     *
     * @param array $data
     * @param int|string $storeId
     * @return array
     */
    private function buildExternalUiConfig(array $data, $storeId): array
    {
        $config = $data['config'];
        $config['magento_adminview_api_urls'] = [
            'config' => $this->getUrl('glopal/connector/config'),
            'stores' => $this->getUrl('glopal/connector/stores'),
        ];
        $config['magento_config'] = [];

        if (isset($data['prefetch']) && is_array($data['prefetch'])) {
            foreach ($data['prefetch'] as $key) {
                if ($key[0] === ':') {
                    $value = $this->prefetchCustomKey($key);
                } elseif ($key === 'design/header/logo_src') {
                    try {
                        $this->emulation->startEnvironmentEmulation(
                            $storeId,
                            Area::AREA_FRONTEND,
                            true
                        );
                        $value = $this->logo->getLogoSrc();
                        $this->emulation->stopEnvironmentEmulation();
                    } catch (Exception $e) {
                        $value = '';
                    }
                } else {
                    $value = $this->configuration->getValue($key, $storeId);
                }

                $config['magento_config'][$key] = $value;
            }
        }

        return $config;
    }

    /**
     * Get WebInterpret configuration
     *
     * @param int|string $storeId
     * @param bool $notified
     * @return array
     * @throws Exception
     */
    private function getExternalUiData($storeId, bool $notified): array
    {
        $request = $this->objectFactory->create();
        $request->setStoreViewId($storeId ? $storeId : null);
        $request->setMagentoUserEmail($this->authSession->getUser()->getEmail());
        $request->setMagentoVersion($this->helper->getMagentoVersion());
        $request->setMagentoPluginVersion($this->helper->getExtensionVersion());
        $request->setIsInstallationNotified($notified);
        $request->setStoreKey($this->helper->getGlopalKey());
        $request->setShortName($this->configuration->getGeneralValue('short_name', $storeId));

        $data = $this->jsonEncoder->encode($request);
        $url = $this->configuration->getGeneralValue('url_load_magento');

        $response = $this->curlRequest->sendPostRequest($url, $data);

        if (($json = $this->jsonDecoder->decode($response)) === null) {
            // phpcs:ignore
            throw new Exception("Invalid JSON response: {$response}");
        }

        if (! isset($json['script_src']) || ! isset($json['config'])) {
            // phpcs:ignore
            throw new Exception("Invalid response schema: {$response}");
        }

        return $json;
    }

    /**
     * Prefetch custom keys.
     *
     * @param string $key
     * @return mixed|string|null
     */
    private function prefetchCustomKey(string $key)
    {
        switch ($key) {
            case ':user:firstname':
                $value = $this->authSession->getUser()->getFirstname();
                break;
            case ':user:lastname':
                $value = $this->authSession->getUser()->getLastname();
                break;
            case ':user:email':
                $value = $this->authSession->getUser()->getEmail();
                break;
            default:
                $value = '';
        }

        return $value;
    }

    /**
     * Get the current or default store ID
     *
     * @return int
     */
    private function getStoreId(): int
    {
        $storeId = 0;

        if (in_array($this->getForm()->getScope(), [ 'websites', 'default' ])) {
            // Load the default store if any
            $stores = $this->_storeManager->getStores();

            if (count($stores) == 1) {
                $store = reset($stores);
                $storeId = $store->getStoreId();
            }
        } else {
            $storeId = (int) $this->getRequest()->getParam('store', $storeId);
        }

        return $storeId;
    }

    /**
     * Render error HTML
     *
     * @param int $code
     * @return string
     */
    private function renderError(int $code = 0): string
    {
        $html = '<div class="messages"><div class="message message-error error"><div>';
        $html.= __($code == 401
            ? 'Failed to authenticate when loading Glopal extension'
            : 'Failed to load Glopal extension');
        $html.= '</div></div></div>';

        return $html;
    }

    /**
     * Output HTML for config API
     *
     * @param string $src
     * @param mixed $config
     * @return string
     */
    private function renderHtml(string $src, $config): string
    {
        $html = '<div id="webinterpret-placeholder" class="webinterpret-placeholder">' . __('Loading...') . '</div>';
        $html.= '<script>';
        $html.= 'window.webinterpretReady = function() {';
        $html.= '    webinterpret.render("webinterpret-placeholder", ' . $this->jsonEncoder->encode($config) . ')';
        $html.= '}';
        $html.= '</script>';
        $html.= '<script crossorigin="anonymous" src="' . $src . '" async></script>';
        return $html;
    }
}
