<?php

namespace Glopal\Connector\Controller\Adminhtml\Connector;

use Exception;
use Glopal\Connector\Helper\Configuration;
use Magento\Framework\App\Action\HttpGetActionInterface;
use Magento\Framework\App\Action\HttpPostActionInterface;
use Magento\Framework\App\RequestInterface;
use Magento\Framework\Controller\Result\Json;
use Magento\Framework\Controller\Result\JsonFactory;
use Magento\Framework\Serialize\SerializerInterface;

class Config implements HttpGetActionInterface, HttpPostActionInterface
{
    /** @var RequestInterface */
    private $request;

    /** @var Configuration */
    private $configuration;

    /** @var JsonFactory */
    private $resultJsonFactory;

    /** @var SerializerInterface */
    private $jsonSerializer;

    /**
     * @param RequestInterface $request
     * @param Configuration $configuration
     * @param SerializerInterface $jsonSerializer
     * @param JsonFactory $resultJsonFactory
     */
    public function __construct(
        RequestInterface $request,
        Configuration $configuration,
        SerializerInterface $jsonSerializer,
        JsonFactory $resultJsonFactory
    ) {
        $this->request = $request;
        $this->configuration = $configuration;
        $this->resultJsonFactory = $resultJsonFactory;
        $this->jsonSerializer = $jsonSerializer;
    }

    /**
     * Execute index action
     */
    public function execute()
    {
        try {
            if ($this->request->isPost()) {
                return $this->saveConfig();
            }

            return $this->viewConfig();
        } catch (Exception $e) {
            /** @var Json $result */
            $result = $this->resultJsonFactory->create();
            /** You may introduce your own constants for this custom REST API */
            $result->setHttpResponseCode(\Magento\Framework\Webapi\Exception::HTTP_FORBIDDEN);
            $result->setData([
                'error' => true,
                'error_message' => $e->getMessage()
            ]);

            return $result;
        }
    }

    /**
     * View configuration.
     *
     * @return Json
     * @throws Exception
     */
    private function viewConfig(): Json
    {
        $data = [];
        $fields = $this->request->getParam('fields');

        if (!empty($fields) && is_array($fields)) {
            $this->validateFields($fields);

            foreach ($fields as $field) {
                $path = $field['path'];
                $scope = $field['scope'];
                $scopeId = $field['scope_id'];

                $field['value'] = $this->configuration->getScopedValue($path, $scope, $scopeId);

                $data[] = $field;
            }
        }

        $resultJson = $this->resultJsonFactory->create();
        $resultJson->setData($data);

        return $resultJson;
    }

    /**
     * Save configuration.
     *
     * @return Json
     * @throws Exception
     */
    private function saveConfig(): Json
    {
        $body = $this->request->getContent();
        $fields = $this->jsonSerializer->unserialize($body);

        if (!empty($fields) && is_array($fields)) {
            $this->validateFields($fields, false);

            foreach ($fields as $field) {
                $path = $field['path'];
                $scope = $field['scope'];
                $scopeId = $field['scope_id'];
                $value = $field['value'];

                if ($value === null) {
                    $this->configuration->deleteValue($path, $scope, $scopeId);
                } else {
                    $this->configuration->saveValue($path, $value, $scope, $scopeId);
                }
            }

            $this->configuration->reinitialiseConfig();
        }

        $resultJson = $this->resultJsonFactory->create();
        $resultJson->setData(['status' => 'ok']);

        return $resultJson;
    }

    /**
     * Validate allowed field keys.
     *
     * @param array $fields
     * @param bool $read
     * @return void
     * @throws Exception
     */
    private function validateFields(array $fields, bool $read = true)
    {
        $pathsRegEx = ($read)
            ? $this->configuration->getGeneralValue('adminview_readable_paths')
            : $this->configuration->getGeneralValue('adminview_writable_paths');

        foreach ($fields as $field) {
            if (empty($field['path']) || ! preg_match('/' . $pathsRegEx . '/', $field['path'])) {
                // phpcs:ignore
                throw new Exception(sprintf(
                    'Unauthorized to %s %s',
                    $read ? 'read' : 'write',
                    $field['path']
                ));
            }
        }
    }
}
