<?php

namespace Glopal\Connector\Controller\Checkout;

use Exception;
use Glopal\Connector\Helper\Configuration;
use Glopal\Connector\Helper\Utils;
use Glopal\Connector\Provider\CartTransfer\CallBackUrlsProvider;
use Magento\Framework\App\Action\HttpPostActionInterface;
use Magento\Framework\App\RequestInterface;
use Magento\Framework\Controller\Result\Json;
use Magento\Framework\Controller\ResultFactory;
use Magento\Framework\Controller\ResultInterface;
use Magento\Framework\Exception\NoSuchEntityException;
use Magento\Framework\Serialize\SerializerInterface;
use Magento\Quote\Api\CartRepositoryInterface;
use Psr\Log\LoggerInterface;

class CallbackPurchase implements HttpPostActionInterface
{
    /** @var CartRepositoryInterface */
    private $cartRepository;

    /** @var ResultFactory */
    private $resultFactory;

    /** @var RequestInterface */
    private $request;

    /** @var LoggerInterface  */
    private $logger;

    /** @var Configuration */
    private $configuration;

    /** @var SerializerInterface */
    private $jsonSerializer;

    /** @var Utils */
    private $utils;

    /** @var CallBackUrlsProvider */
    private $callBackUrlsProvider;

    /**
     * @param CartRepositoryInterface $cartRepository
     * @param ResultFactory $resultFactory
     * @param RequestInterface $request
     * @param LoggerInterface $logger
     * @param Configuration $configuration
     * @param SerializerInterface $jsonSerializer
     * @param Utils $utils
     * @param CallBackUrlsProvider $callBackUrlsProvider
     */
    public function __construct(
        CartRepositoryInterface $cartRepository,
        ResultFactory $resultFactory,
        RequestInterface $request,
        LoggerInterface $logger,
        Configuration $configuration,
        SerializerInterface $jsonSerializer,
        Utils $utils,
        CallBackUrlsProvider $callBackUrlsProvider
    ) {
        $this->cartRepository = $cartRepository;
        $this->resultFactory = $resultFactory;
        $this->request = $request;
        $this->logger = $logger;
        $this->configuration = $configuration;
        $this->jsonSerializer = $jsonSerializer;
        $this->utils = $utils;
        $this->callBackUrlsProvider = $callBackUrlsProvider;
    }

    /**
     * Execute Callback Purchase action.
     *
     * @return Json|ResultInterface
     * @throws NoSuchEntityException
     */
    public function execute()
    {
        $result = [];
        $response = $this->resultFactory->create(ResultFactory::TYPE_JSON);

        try {
            $jsonBody = $this->request->getContent();
            $requestBody = $this->jsonSerializer->unserialize($jsonBody);
            $requestCartId = $requestBody['cart']['external_id'] ?? null;

            if (null === $requestCartId || !$this->validateHmacKey($requestCartId)) {
                $errorMessage = 'Invalid or missing hmac key';
                $this->logger->error($errorMessage);

                return $response->setData(['error' => $errorMessage])->setHttpResponseCode(400);
            }

            $checkoutSuccessUrl = $requestBody['checkout']['success_url'] ?? null;
            if ($this->isSuccessRedirectAvailable($checkoutSuccessUrl)) {
                $result = [
                    'checkout' => [
                        'success_url' => $this->callBackUrlsProvider->getCustomSuccessPageUrl($checkoutSuccessUrl),
                        'success_url_autoredirect' => true,
                        'success_url_enable_hmac' => true
                    ]
                ];
            }

            $cart = $this->cartRepository->get($requestCartId);
            $cart->setIsActive(false);
            $this->cartRepository->save($cart);
        } catch (Exception $e) {
            $errorMessage = $e->getMessage();
            $this->logger->error($errorMessage);

            return $response->setData(['error' => $errorMessage])->setHttpResponseCode(500);
        }

        return $response->setData($result);
    }

    /**
     * Validate requested hmac key.
     *
     * @param string $cartId
     * @return bool
     */
    private function validateHmacKey(string $cartId): bool
    {
        $requestedHmacKey = $this->request->getParam('hmac');
        if (null === $requestedHmacKey) {
            return false;
        }

        $secretKey = $this->configuration->getGeneralValue('key');
        $expectedHmacKey = $this->utils->buildHmacKey($cartId, $secretKey);

        return hash_equals($expectedHmacKey, $requestedHmacKey);
    }

    /**
     * Is Success redirect enabled? (for clearing the local storage after Glopal checkout)
     *
     * @param $checkoutSuccessUrl
     *
     * @return bool
     */
    private function isSuccessRedirectAvailable($checkoutSuccessUrl): bool
    {
        return !empty($checkoutSuccessUrl)
            && $this->configuration->getCheckoutFlag('general/is_clear_local_storage');
    }
}
