<?php

namespace Glopal\Connector\Controller\Checkout;

use Exception;
use Glopal\Connector\Helper\Configuration;
use Magento\Checkout\Model\Session;
use Magento\Framework\App\ActionInterface;
use Magento\Framework\App\RequestInterface;
use Magento\Framework\App\ResponseInterface;
use Magento\Framework\Controller\Result\Json;
use Magento\Framework\Controller\Result\Redirect as ResultRedirect;
use Magento\Framework\Controller\ResultFactory;
use Magento\Framework\Controller\ResultInterface;
use Magento\Framework\UrlInterface;
use Magento\Quote\Api\CartRepositoryInterface;
use Psr\Log\LoggerInterface;

/**
 * @SuppressWarnings(PHPMD.AllPurposeAction)
 */
class Redirect implements ActionInterface
{
    /** @var string */
    private const CIPHER_ALGORITHM = 'AES-256-CBC';

    /** @var Configuration */
    private $configuration;

    /** @var Session */
    private $checkoutSession;

    /** @var CartRepositoryInterface */
    private $cartRepository;

    /** @var ResultFactory */
    private $resultFactory;

    /** @var UrlInterface */
    private $url;

    /** @var RequestInterface */
    private $request;

    /** @var LoggerInterface  */
    private $logger;

    /**
     * @param Configuration $configuration
     * @param Session $checkoutSession
     * @param CartRepositoryInterface $cartRepository
     * @param ResultFactory $resultFactory
     * @param UrlInterface $url
     * @param RequestInterface $request
     * @param LoggerInterface $logger
     */
    public function __construct(
        Configuration $configuration,
        Session $checkoutSession,
        CartRepositoryInterface $cartRepository,
        ResultFactory $resultFactory,
        UrlInterface $url,
        RequestInterface $request,
        LoggerInterface $logger
    ) {
        $this->configuration = $configuration;
        $this->checkoutSession = $checkoutSession;
        $this->cartRepository = $cartRepository;
        $this->resultFactory = $resultFactory;
        $this->url = $url;
        $this->request = $request;
        $this->logger = $logger;
    }

    /**
     * Execute redirect action.
     *
     * @return ResponseInterface|Json|ResultInterface
     */
    public function execute()
    {
        if (!function_exists('openssl_cipher_iv_length')) {
            return $this->jsonResponse(['error' => 'Newer version of OpenSSL is required.'], 500);
        }

        try {
            if ($this->request->isPost()) {
                return $this->createRedirect();
            } elseif ($this->request->isGet()) {
                return $this->followRedirect();
            } else {
                return $this->jsonResponse(['error' => 'Bad Request'], 400);
            }
        } catch (Exception $e) {
            return $this->jsonResponse(['error' => 'Unexpected error: ' . $e->getMessage()], 500);
        }
    }

    /**
     * Prepare json response.
     *
     * @param array $data
     * @param int $statusCode
     * @return Json|ResultInterface
     */
    private function jsonResponse(array $data, int $statusCode = 200)
    {
        return $this->resultFactory->create(ResultFactory::TYPE_JSON)
            ->setHttpResponseCode($statusCode)
            ->setData($data);
    }

    /**
     * Creates redirect.
     *
     * @return Json|ResultInterface
     */
    private function createRedirect()
    {
        $quoteId = $this->checkoutSession->getQuoteId();
        $redirectUrl = $this->url->getUrl('glopal_connector/checkout/redirect', [
            'token' => $this->createRedirectToken($quoteId)
        ]);

        return $this->jsonResponse([
            'redirect_url' => $redirectUrl
        ]);
    }

    /**
     * Build redirect token by encrypting the quote id
     *
     * @param string|int $quoteId
     * @return string
     */
    private function createRedirectToken($quoteId): string
    {
        // phpcs:disable
        $key = base64_decode($this->getSecretKey());
        $iv = openssl_random_pseudo_bytes(openssl_cipher_iv_length(self::CIPHER_ALGORITHM));

        return base64_encode(json_encode([
            0 => $this->encryptData($quoteId, $key, $iv),
            1 => base64_encode($iv),
        ]));
        // phpcs:enable
    }

    /**
     * Parse redirect token and return quote id
     *
     * @param string $token
     * @return float|int|string
     * @throws Exception
     */
    private function parseRedirectToken(string $token)
    {
        // phpcs:disable
        $token = json_decode(base64_decode($token));
        if (json_last_error() !== JSON_ERROR_NONE) {
            throw new Exception('Invalid token (json error).');
        }

        if (!is_array($token) || count($token) < 2) {
            throw new Exception('Invalid token (bad data).');
        }

        $encryptedData = $token[0];
        $iv = base64_decode($token[1]);
        $key = base64_decode($this->getSecretKey());
        $quoteId = $this->decryptData($encryptedData, $key, $iv);

        if (!is_numeric($quoteId)) {
            throw new Exception('Invalid quote id.');
        }
        // phpcs:enable

        return $quoteId;
    }

    /**
     * Encrypt data using AES-256-CBC
     *
     * @param string $data
     * @param string $key
     * @param string $iv
     * @return false|string
     */
    private function encryptData(string $data, string $key, string $iv)
    {
        return openssl_encrypt($data, self::CIPHER_ALGORITHM, $key, 0, $iv);
    }

    /**
     * Decrypt data using AES-256-CBC
     *
     * @param string $data
     * @param string $key
     * @param string $iv
     * @return false|string
     */
    private function decryptData(string $data, string $key, string $iv)
    {
        return openssl_decrypt($data, self::CIPHER_ALGORITHM, $key, 0, $iv);
    }

    /**
     * Copy items to the cart from an existing quote and then redirect the user to the checkout.
     *
     * @return ResultRedirect|ResultInterface
     */
    private function followRedirect()
    {
        try {
            // Get token from URL parameter and parse it
            $token = $this->request->getParam('token');
            if (empty($token)) {
                // phpcs:ignore
                throw new Exception('Missing token.');
            }

            $quoteId = $this->parseRedirectToken($token);

            // Copy items from quote to current cart
            $quote = $this->cartRepository->get($quoteId);

            $this->checkoutSession->resetCheckout();
            $cart = $this->checkoutSession->getQuote();

            foreach ($cart->getItemsCollection() as $item) {
                $cart->removeItem($item->getId());
            }

            $cart->merge($quote)->collectTotals();
            $this->cartRepository->save($cart);

            // Redirect to checkout
            $url = $this->getCheckoutUrl();

            return $this->resultFactory->create(ResultFactory::TYPE_REDIRECT)->setUrl($url);
            // phpcs:ignore
        } catch (Exception $e) {
            $this->logger->error($e->getMessage(), ['exception' => $e]);

            // Something went wrong. We don't want to display the error
            // to the end user so let's redirect to the cart and place
            // the error message in an header to assist with troubleshooting.
            // phpcs:ignore
            header('X-Glopal-Error: ' . $e->getMessage());

            return $this->resultFactory->create(ResultFactory::TYPE_REDIRECT)->setUrl('checkout/cart');
        }
    }

    /**
     * Gets checkout url.
     *
     * @return string
     */
    private function getCheckoutUrl(): string
    {
        $url = $this->configuration->getGeneralValue('checkout_url');

        if (empty($url)) {
            $url = $this->url->getUrl('checkout');
        }

        return $url;
    }

    /**
     * Gets secret key.
     *
     * @return string
     */
    public function getSecretKey(): string
    {
        $key = $this->configuration->getGeneralValue('secret_key');
        if (empty($key)) {
            $key = base64_encode(
                openssl_random_pseudo_bytes(openssl_cipher_iv_length(self::CIPHER_ALGORITHM))
            );
            $this->configuration->saveGeneralValue('secret_key', $key);
        }

        return $key;
    }
}
