<?php

namespace Glopal\Connector\Controller\Diagnostics;

use Exception;
use Glopal\Connector\Helper\Configuration;
use Glopal\Connector\Helper\Installation;
use Glopal\Connector\Helper\ModuleData;
use Magento\Framework\App\ActionInterface;
use Magento\Framework\App\State;
use Magento\Framework\Controller\Result\JsonFactory;
use Magento\Framework\Module\FullModuleList;
use PDO;
use Psr\Log\LoggerInterface;

/**
 * @SuppressWarnings(PHPMD.AllPurposeAction)
 */
class Index implements ActionInterface
{
    /** @var Configuration */
    private $configuration;

    /** @var ModuleData */
    private $helper;

    /** @var Installation */
    private $installation;

    /** @var JsonFactory */
    private $jsonResultFactory;

    /** @var FullModuleList */
    private $fullModuleList;

    /** @var State */
    private $appState;

    /** @var LoggerInterface */
    private $logger;

    /**
     * @param Configuration $configuration
     * @param ModuleData $data
     * @param Installation $installation
     * @param JsonFactory $jsonResultFactory
     * @param FullModuleList $fullModuleList
     * @param State $appState
     * @param LoggerInterface $logger
     */
    public function __construct(
        Configuration $configuration,
        ModuleData $data,
        Installation $installation,
        JsonFactory $jsonResultFactory,
        FullModuleList $fullModuleList,
        State $appState,
        LoggerInterface $logger
    ) {
        $this->configuration = $configuration;
        $this->helper = $data;
        $this->installation = $installation;
        $this->jsonResultFactory = $jsonResultFactory;
        $this->fullModuleList = $fullModuleList;
        $this->appState = $appState;
        $this->logger = $logger;
    }

    /**
     * Execute index action
     */
    public function execute()
    {
        $resultPage = $this->jsonResultFactory->create();
        $result = [];

        if ($this->configuration->getGeneralFlag('signature_verification')
            && !$this->helper->verifyRequestSignature()
        ) {
            return $resultPage->setData($result)->setHttpResponseCode(403);
        }

        try {
            $result = [
                'plugin' => $this->viewPlugin(),
                'platform' => $this->viewPlatform(),
                'system' => $this->viewSystem(),
            ];
        } catch (Exception $e) {
            $this->logger->error($e->getMessage(), ['exception' => $e]);
            $resultPage->setHttpResponseCode(500);
            $result = [
                'message' => $e->getMessage(),
                'exception' => $e->getTraceAsString()
            ];
        }

        return $resultPage->setData($result);
    }

    /**
     * Get plugin file errors.
     *
     * @return array
     */
    private function getPluginFileErrors(): array
    {
        $root = $this->helper->getExtensionRoot() . '/';
        $path = $root . 'etc/content.json';

        // phpcs:ignore
        if (!is_file($path)) {
            return [
                'missing' => [
                    'etc/content.json'
                ]
            ];
        }

        // phpcs:ignore
        if (!is_readable($path)) {
            return [
                'unreadable' => [
                    'etc/content.json'
                ]
            ];
        }

        // phpcs:ignore
        $files =  json_decode(file_get_contents($path), true);

        $errors = [];
        $dirs = [];
        list($dirs, $errors) = $this->processFiles($files, $root, $dirs, $errors);

        // phpcs:ignore
        $dirs = array_unique($dirs);
        foreach ($dirs as $dir) {
            $path = $root . $dir;
            if (!is_writeable($path)) { // phpcs:ignore
                $errors['unwritable'][] = $path;
            }
        }

        return $errors;
    }

    /**
     * Gets configuration.
     *
     * @param int|string|null $storeId
     * @return array
     */
    private function getConfiguration($storeId = null): array
    {
        $keys = [
            'short_name',
            'redirector_enabled',
            'footer',
            'head',
            'key',
            'secret_key',
            'checkout_url',
            'installation_notified_at',
            'installation_notified_retry',
            'last_request_error_load',
            'installation_mode',
            'pass_db_link',
            'bridge_file_update_url',
            'adminview_readable_paths',
            'adminview_writable_paths',
            'public_key',
            'signature_verification',
            'remote_assets_url',
            'url_install_magento',
            'url_load_magento',
        ];
        $data = [];
        foreach ($keys as $key) {
            $data[$key] = $this->configuration->getGeneralValue($key, $storeId);
        }

        return $data;
    }

    /**
     * Gets plugin data.
     *
     * @return array
     */
    private function viewPlugin(): array
    {
        return [
            'plugin_version' => $this->helper->getExtensionVersion(),
            'plugin_enabled' => $this->helper->isExtensionEnabled(),
            'store_token' => $this->helper->getGlopalKey(),
            'updates_dir' => null,
            'is_updatable' => false,
            'file_errors' => $this->getPluginFileErrors(),
            'active_settings' => $this->getConfiguration(),
        ];
    }

    /**
     * Gets platform data.
     *
     * @return array
     */
    private function viewPlatform(): array
    {
        $websitesData = $this->installation->getWebsitesData();
        foreach ($websitesData as &$website) {
            foreach ($website['stores'] as &$storeGroup) {
                foreach ($storeGroup['store_views'] as &$storeView) {
                    $storeView['active_settings'] = $this->getConfiguration($storeGroup['id']);
                }
            }
        }

        return [
            'platform_name' => 'magento2',
            'platform_version' => $this->helper->getMagentoVersion(),
            'magento' => [
                'compiler_enabled' => null,
                'app_state_mode' => $this->appState->getMode(),
                'websites' => $websitesData,
                'modules' => $this->fullModuleList->getAll(),
            ],
        ];
    }

    /**
     * Gets system data.
     *
     * @return array
     */
    private function viewSystem(): array
    {
        return [
            'php_version' => phpversion(),
            'curl_available' => $this->isCurlAvailable(),
            'curl_version' => $this->isCurlAvailable() ? $this->getCurlVersion() : null,
            'curl_ssl_support_enabled' => $this->isCurlAvailable() ? $this->isCurlSSLSupportEnabled() : null,
            'curl_ssl_version' => $this->isCurlSSLSupportEnabled() ? $this->getCurlSSLVersion() : null,
            'curl_http2_enabled' => $this->isCurlAvailable() ? $this->isCurlHTTP2SupportEnabled() : null,
            'curl_https_enabled' => $this->isCurlAvailable() ? $this->isCurlHTTPSSupportEnabled() : null,
            'openssl_available' => $this->isOpenSSLAvailable(),
            'openssl_version' => $this->isOpenSSLAvailable() ? $this->getOpenSSLVersion() : null,
            'db_support' => $this->getDbSupport(),
            'allow_url_fopen_enabled' => $this->isAllowUrlFopenEnabled(),
            'post_max_size' => $this->getPostMaxSize(),
            'php_time_limit' => $this->getMaxExecutionTime(),
            'opcache_enabled' => $this->isOpcacheEnabled(),
            'apc_enabled' => $this->isApcEnabled(),
            'current_datetime' => date('Y-m-d H:i:s'),
            'timezone' => date_default_timezone_get(),
            'php_sapi_name' => $this->getPhpSapiName(),
            'is_hhvm' => $this->isHhvm(),
            'is_suhosin' => $this->isSuhosin(),
            'operating_system' => $this->getOsInfo(),
            'server' => $this->getServerInfo(),
            'apache' => $this->getApacheInfo(),
            'php' => $this->getPhpInformation(),
            'php_extensions' => get_loaded_extensions(),
        ];
    }

    /**
     * Is Curl available?
     *
     * @return bool
     */
    private function isCurlAvailable(): bool
    {
        return extension_loaded('curl') && function_exists('curl_version') ? true : false;
    }

    /**
     * Get Curl version array.
     *
     * @return array|null
     */
    private function getCurlVersionInfoArray(): ?array
    {
        if (!function_exists('curl_version')) {
            return null;
        }

        // phpcs:ignore
        return curl_version();
    }

    /**
     * Get Curl version.
     *
     * @return string|null
     */
    private function getCurlVersion(): ?string
    {
        $curlVersion = $this->getCurlVersionInfoArray();

        if (!is_array($curlVersion) || ! isset($curlVersion['version'])) {
            return null;
        }

        return $curlVersion['version'];
    }

    /**
     * Is Curl SSL supported?
     *
     * @return bool|null
     */
    private function isCurlSSLSupportEnabled(): ?bool
    {
        $curlVersion = $this->getCurlVersionInfoArray();

        if (!is_array($curlVersion) || ! isset($curlVersion['features']) || !defined('CURL_VERSION_SSL')) {
            return null;
        }

        // checks if the CURL_VERSION_SSL bitwise flag is enabled
        return ($curlVersion['features'] & CURL_VERSION_SSL) == CURL_VERSION_SSL;
    }

    /**
     * Is Curl HTTPS supported?
     *
     * @return bool|null
     */
    private function isCurlHTTPSSupportEnabled(): ?bool
    {
        $curlVersion = $this->getCurlVersionInfoArray();

        if (!is_array($curlVersion) || !isset($curlVersion['protocols']) || !is_array($curlVersion['protocols'])) {
            return null;
        }

        return in_array('https', $curlVersion['protocols']);
    }

    /**
     * Is Curl HTTP2 supported?
     *
     * @return bool|null
     */
    private function isCurlHTTP2SupportEnabled(): ?bool
    {
        $curlVersion = $this->getCurlVersionInfoArray();

        if (!is_array($curlVersion)
            || !isset($curlVersion['features'])
            || !defined('CURL_VERSION_HTTP2')
        ) {
            return null;
        }

        return ($curlVersion['features'] & CURL_VERSION_HTTP2) == CURL_VERSION_HTTP2;
    }

    /**
     * Get Curl SSL version.
     *
     * @return mixed|null
     */
    private function getCurlSSLVersion()
    {
        if (!$this->isOpenSSLAvailable()) {
            return null;
        }

        $curl_version = $this->getCurlVersionInfoArray();

        if (!is_array($curl_version) || !isset($curl_version['ssl_version'])) {
            return null;
        }

        return $curl_version['ssl_version'];
    }

    /**
     * Is OpenSSL available?
     *
     * @return bool
     */
    private function isOpenSSLAvailable(): bool
    {
        return extension_loaded('openssl');
    }

    /**
     * Get OpenSSL varsion.
     *
     * @return null|string
     */
    private function getOpenSSLVersion(): ?string
    {
        if (!$this->isOpenSSLAvailable() || !defined('OPENSSL_VERSION_TEXT')) {
            return null;
        }

        return OPENSSL_VERSION_TEXT;
    }

    /**
     * Get database support data.
     *
     * @return array
     */
    private function getDbSupport(): array
    {
        return [
            'mysql' => function_exists('mysql_connect'),
            'mysqli' => function_exists('mysqli_connect'),
            'pg' => function_exists('pg_connect'),
            'pdo' => extension_loaded('PDO'),
            'pdo_mysql' => extension_loaded('pdo_mysql'),
            'pdo_pgsql' => extension_loaded('pdo_pgsql'),
            'pdo_drivers' => extension_loaded('PDO') ? PDO::getAvailableDrivers() : null
        ];
    }

    /**
     * Is Url fopen enabled?
     *
     * @return bool
     */
    private function isAllowUrlFopenEnabled(): bool
    {
        return ini_get('allow_url_fopen') == true;
    }

    /**
     * Gets post max size.
     *
     * @return string
     */
    private function getPostMaxSize(): string
    {
        return ini_get('post_max_size');
    }

    /**
     * Gets max execution time.
     *
     * @return string
     */
    private function getMaxExecutionTime(): string
    {
        return ini_get('max_execution_time');
    }

    /**
     * Is opcache enabled?
     *
     * @return bool
     */
    private function isOpcacheEnabled(): bool
    {
        return extension_loaded('Zend OPcache') && ini_get('opcache.enable') ? true : false;
    }

    /**
     * Is apc enabled?
     *
     * @return bool
     */
    private function isApcEnabled(): bool
    {
        return extension_loaded('apc') && ini_get('apc.enabled') ? true : false;
    }

    /**
     * Gets php sapi name.
     *
     * @return string
     */
    private function getPhpSapiName(): string
    {
        return php_sapi_name();
    }

    /**
     * Is hhvm?
     *
     * @return bool
     */
    private function isHhvm(): bool
    {
        return defined('HHVM_VERSION') ? true : false;
    }

    /**
     * Is suhosin?
     *
     * @return bool
     */
    private function isSuhosin(): bool
    {
        return extension_loaded('suhosin');
    }

    /**
     * Gets os information.
     *
     * @return array
     */
    private function getOsInfo(): array
    {
        return [
            'os_name' => php_uname('s'),
            'release_name' => php_uname('r'),
            'version' => php_uname('v'),
            'machine_type' => php_uname('m'),
            'os_host_name' => php_uname('n'),
        ];
    }

    /**
     * Gets serve information.
     *
     * @return array
     */
    private function getServerInfo(): array
    {
        // phpcs:disable
        return [
            'server_software' => $_SERVER['SERVER_SOFTWARE'] ?? null,
            'server_name' => $_SERVER['SERVER_NAME'] ?? null,
            'server_addr' => $_SERVER['SERVER_ADDR'] ?? null,
            'server_port' => $_SERVER['SERVER_PORT'] ?? null,
            'https' => $_SERVER['HTTPS'] ?? false,
            'document_root' => $_SERVER['DOCUMENT_ROOT'] ?? null,
            'script_filename' => $_SERVER['SCRIPT_FILENAME'] ?? null,
        ];
        // phpcs:enable
    }

    /**
     * Gets apache information.
     *
     * @return array
     */
    private function getApacheInfo(): array
    {
        if (!function_exists('apache_get_version') || !function_exists('apache_get_modules')) {
            return [];
        }

        return [
            'version' => apache_get_version(),
            'modules' => apache_get_modules()
        ];
    }

    /**
     * Gets php information.
     *
     * @return array
     */
    private function getPhpInformation(): array
    {
        return [
            'php_ini' => php_ini_loaded_file(),
            'parsed_ini_files' => php_ini_scanned_files()
        ];
    }

    /**
     * Process files.
     *
     * @param array $files
     * @param string $root
     * @param array $dirs
     * @param array $errors
     * @return array
     */
    private function processFiles(array $files, string $root, array $dirs, array $errors): array
    {
        foreach ($files as $file => $hash) {
            $path = $root . $file;
            $dirs[] = dirname($file); // phpcs:ignore

            // phpcs:ignore
            if (!file_exists($path)) {
                $errors['missing'][] = $path;
            } elseif (!is_readable($path)) { // phpcs:ignore
                $errors['unreadable'][] = $path;
            }

            // phpcs:ignore
            if ($hash !== md5_file($path)) {
                $errors['corrupted'][] = $path;
            }

            // phpcs:ignore
            if (!is_writeable($path)) {
                $errors['unwritable'][] = $path;
            }
        }

        return [$dirs, $errors];
    }
}
