<?php

namespace Glopal\Connector\Controller\Order;

use Glopal\Connector\Api\CreateRequestDTOInterfaceFactory;
use Glopal\Connector\Helper\Signature;
use Glopal\Connector\Processor\OrderCreate;
use Glopal\Connector\Validator\OrderCreateValidator;
use Magento\Framework\App\Action\HttpPostActionInterface;
use Magento\Framework\App\RequestInterface;
use Magento\Framework\Controller\Result\Json;
use Magento\Framework\Controller\Result\JsonFactory;
use Magento\Framework\Exception\NotFoundException;
use Magento\Framework\Serialize\SerializerInterface;
use Psr\Log\LoggerInterface;

class Index implements HttpPostActionInterface
{
    /** @var RequestInterface */
    private $request;

    /** @var SerializerInterface */
    private $jsonSerializer;

    /** @var OrderCreate */
    private $orderCreateProcessor;

    /** @var OrderCreateValidator */
    private $orderCreateValidator;

    /** @var JsonFactory */
    private $resultJsonFactory;

    /** @var LoggerInterface */
    private $logger;

    /** @var CreateRequestDTOInterfaceFactory */
    private $createRequestDTOInterfaceFactory;

    /** @var Signature */
    private $signature;

    /**
     * @param RequestInterface $request
     * @param SerializerInterface $jsonSerializer
     * @param OrderCreate $orderCreateProcessor
     * @param OrderCreateValidator $orderCreateValidator
     * @param JsonFactory $resultJsonFactory
     * @param LoggerInterface $logger
     * @param CreateRequestDTOInterfaceFactory $createRequestDTOInterfaceFactory
     * @param Signature $signature
     */
    public function __construct(
        RequestInterface $request,
        SerializerInterface $jsonSerializer,
        OrderCreate $orderCreateProcessor,
        OrderCreateValidator $orderCreateValidator,
        JsonFactory $resultJsonFactory,
        LoggerInterface $logger,
        CreateRequestDTOInterfaceFactory $createRequestDTOInterfaceFactory,
        Signature $signature
    ) {
        $this->request = $request;
        $this->jsonSerializer = $jsonSerializer;
        $this->orderCreateProcessor = $orderCreateProcessor;
        $this->orderCreateValidator = $orderCreateValidator;
        $this->resultJsonFactory = $resultJsonFactory;
        $this->logger = $logger;
        $this->createRequestDTOInterfaceFactory = $createRequestDTOInterfaceFactory;
        $this->signature = $signature;
    }

    /**
     * Execute index action.
     *
     * @return Json
     * @throws NotFoundException
     */
    public function execute(): Json
    {
        $params = $this->getRequestParams();

        if (!$this->signature->isAllowed($this->request)) {
            return $this->resultJsonFactory->create()
                ->setData(['message' => $this->signature->getMessage()])
                ->setHttpResponseCode(403);
        }

        $validationResult = $this->orderCreateValidator->validate($params);

        if (!$validationResult['status']) {
            $this->logger->error($validationResult['message']);

            return $this->resultJsonFactory->create()->setData($validationResult);
        }

        $createRequestDTO = $this->createRequestDTOInterfaceFactory->create(['data' => $params]);
        $result = $this->orderCreateProcessor->process($createRequestDTO);

        return $this->resultJsonFactory->create()->setData($result);
    }

    /**
     * Retrieve request params.
     *
     * @return array
     *
     * @throws NotFoundException if there are issues with the request
     */
    private function getRequestParams(): array
    {
        $errorMessage = 'Invalid Request params';
        $rawBody = $this->request->getContent();

        if ($rawBody) {
            $params = $this->jsonSerializer->unserialize($rawBody);

            if (empty($params)) {
                $this->logger->error($errorMessage, ['body' => $rawBody]);

                throw new NotFoundException(__($errorMessage));
            }

            return $params;
        }

        $this->logger->error($errorMessage, ['body' => $rawBody]);

        throw new NotFoundException(__($errorMessage));
    }
}
