<?php

namespace Glopal\Connector\Controller\Quote;

use Exception;
use Glopal\Connector\Response\CartResponseBuilder;
use Magento\Catalog\Api\Data\ProductInterface;
use Magento\Catalog\Model\ProductRepository;
use Magento\Directory\Model\Country;
use Magento\Directory\Model\ResourceModel\Region\Collection;
use Magento\Framework\App\ActionInterface;
use Magento\Framework\App\Config\ScopeConfigInterface;
use Magento\Framework\App\RequestInterface;
use Magento\Framework\Controller\Result\JsonFactory;
use Magento\Framework\DataObject;
use Magento\Framework\Exception\LocalizedException;
use Magento\Framework\Exception\NoSuchEntityException;
use Magento\Framework\Serialize\SerializerInterface;
use Magento\Quote\Model\Quote;
use Magento\Quote\Model\QuoteFactory;
use Magento\SalesRule\Model\Coupon;
use Magento\SalesRule\Model\Rule;
use Magento\Store\Model\StoreManagerInterface;
use Magento\Tax\Helper\Data as TaxHelper;

/**
 * @SuppressWarnings(PHPMD.AllPurposeAction)
 * @SuppressWarnings(PHPMD.CouplingBetweenObjects)
 */
class Index implements ActionInterface
{
    /** @var SerializerInterface */
    private $jsonSerializer;

    /** @var StoreManagerInterface */
    private $storeManager;

    /** @var ScopeConfigInterface */
    private $scopeConfig;

    /** @var QuoteFactory */
    private $quote;

    /** @var ProductRepository */
    private $productRepository;

    /** @var Country */
    private $country;

    /** @var TaxHelper */
    private $taxHelper;

    /** @var Coupon */
    private $coupon;

    /** @var Rule */
    private $saleRule;

    /** @var RequestInterface */
    private $request;

    /** @var JsonFactory */
    private $jsonFactory;

    /** @var array */
    private $requestData;

    /** @var Collection */
    private $regionCollection;

    /** @var CartResponseBuilder */
    private $cartResponseBuilder;

    /**
     * @param SerializerInterface $jsonSerializer
     * @param StoreManagerInterface $storeManager
     * @param ScopeConfigInterface $scopeConfig
     * @param QuoteFactory $quote
     * @param ProductRepository $productRepository
     * @param Country $country
     * @param TaxHelper $taxHelper
     * @param Coupon $coupon
     * @param Rule $saleRule
     * @param RequestInterface $request
     * @param JsonFactory $jsonFactory
     * @param CartResponseBuilder $cartResponseBuilder
     *
     * @SuppressWarnings(PHPMD.ExcessiveParameterList)
     */
    public function __construct(
        SerializerInterface $jsonSerializer,
        StoreManagerInterface $storeManager,
        ScopeConfigInterface $scopeConfig,
        QuoteFactory $quote,
        ProductRepository $productRepository,
        Country $country,
        TaxHelper $taxHelper,
        Coupon $coupon,
        Rule $saleRule,
        RequestInterface $request,
        JsonFactory $jsonFactory,
        CartResponseBuilder $cartResponseBuilder
    ) {
        $this->jsonSerializer = $jsonSerializer;
        $this->storeManager = $storeManager;
        $this->scopeConfig = $scopeConfig;
        $this->quote = $quote;
        $this->productRepository = $productRepository;
        $this->country = $country;
        $this->taxHelper = $taxHelper;
        $this->coupon = $coupon;
        $this->saleRule = $saleRule;
        $this->request = $request;
        $this->jsonFactory = $jsonFactory;
        $this->cartResponseBuilder = $cartResponseBuilder;
    }

    /**
     * Sets request data.
     *
     * @param string $json
     * @return void
     */
    private function setRequestData(string $json)
    {
        $this->requestData = $this->jsonSerializer->unserialize($json);
    }

    /**
     * Validate data.
     *
     * @return array
     */
    private function validateData(): array
    {
        $errors = [];
        $data = $this->requestData;
        $requiredState = explode(
            ',',
            (string)$this->scopeConfig->getValue('general/region/state_required')
        );

        $shippingAddressData = [];
        if ($this->hasShippingAddress()) {
            $shippingAddressData = $data['checkout']['shipping_address'];
        }

        if ($shippingAddressData && (
            in_array($shippingAddressData['country_code'], $requiredState)
                && (!isset($shippingAddressData['state']) || strlen($shippingAddressData['state']) == 0)
                && (!isset($shippingAddressData['state_code']) || strlen($shippingAddressData['state_code']) == 0)
        )
        ) {
            $errors[] = __('State or state code is required');
        }

        return $errors;
    }

    /**
     * Gets store id.
     *
     * @return mixed
     */
    private function getStoreId()
    {
        return $this->requestData['store']['store_view_id'];
    }

    /**
     * Gets items.
     *
     * @return mixed
     */
    private function getItems()
    {
        return $this->requestData['items'];
    }

    /**
     * Gets coupon code.
     *
     * @return mixed
     */
    private function getCouponCode()
    {
        return $this->requestData['checkout']['discount_code'];
    }

    /**
     * Gets shipping street.
     *
     * @return array
     */
    public function getShippingStreet(): array
    {
        $street = [];
        if (!empty($this->requestData['checkout']['shipping_address']['street1'])) {
            $street[] = $this->requestData['checkout']['shipping_address']['street1'];
        }

        if (!empty($this->requestData['checkout']['shipping_address']['street2'])) {
            $street[] = $this->requestData['checkout']['shipping_address']['street2'];
        }

        return $street;
    }

    /**
     * Gets region collection.
     *
     * @param int|string $countryId
     * @return Collection
     */
    public function getRegionCollection($countryId): Collection
    {
        if (!$this->regionCollection) {
            $this->regionCollection = $this->country->loadByCode($countryId)->getRegions();
        }

        return $this->regionCollection;
    }

    /**
     * Is shipping address exists?
     *
     * @return bool
     */
    private function hasShippingAddress(): bool
    {
        return isset($this->requestData['checkout']['shipping_address']);
    }

    /**
     * Gets shipping address.
     *
     * @return array
     */
    private function getShippingAddress(): array
    {
        $regionId = null;
        $sippingAddressData = $this->requestData['checkout']['shipping_address'];
        $reqStateCode = $sippingAddressData['state_code'] ?? null;
        $reqState = $sippingAddressData['state'] ?? null;

        $regions = $this->getRegionCollection($sippingAddressData['country_code']);
        foreach ($regions as $region) {
            if ($reqStateCode == $region->getCode() || $reqState == $region->getDefaultName()) {
                $regionId = $region->getRegionId();
            }
        }

        return [
            'country_id' => $sippingAddressData['country_code'],
            'post_code' => $sippingAddressData['postal_code'],
            'postcode' => $sippingAddressData['postal_code'],
            'email' => $sippingAddressData['email'],
            'first_name' => $sippingAddressData['contact']['first_name'],
            'last_name' => $sippingAddressData['contact']['last_name'],
            'company' => $sippingAddressData['contact']['company_name'],
            'vat_id' => $sippingAddressData['contact']['tax_number'],
            'city' => $sippingAddressData['city'],
            'street' => $this->getShippingStreet(),
            'region' => $reqState,
            'region_id' => $regionId,
            'telephone' => $sippingAddressData['phone'],
        ];
    }

    /**
     * Execute index action
     */
    public function execute()
    {
        $errors = [];

        try {
            $json = $this->request->getContent();
            $this->setRequestData($json);

            if ($errors = $this->validateData()) {
                return $this->jsonFactory->create()->setData([
                    'status' => 'error',
                    'errors' => $errors
                ]);
            }

            $response = $this->buildData();
        } catch (Exception $e) { // phpcs:ignore
            if ($e->getMessage()) {
                $errors[] = $e->getMessage();
            }

            $response = [
                'status' => 'error',
                'errors' => $errors
            ];
        }

        return $this->jsonFactory->create()->setData($response);
    }

    /**
     * Build data.
     *
     * @return array
     * @throws LocalizedException|NoSuchEntityException
     *
     * @SuppressWarnings(PHPMD.CyclomaticComplexity)
     * @SuppressWarnings(PHPMD.NPathComplexity)
     */
    private function buildData(): array
    {
        $warnings = [];

        $store = $this->storeManager->getStore($this->getStoreId());
        $quote = $this->quote->create();
        $quote->setStore($store);

        // Add products to quote
        foreach ($this->getItems() as $item) {
            $this->addProductToQuote($item, $quote);
        }

        $reqStateCode = $this->requestData['checkout']['shipping_address']['state_code'] ?? null;
        $reqState = $this->requestData['checkout']['shipping_address']['state'] ?? null;

        if ($this->hasShippingAddress()) {
            $shippingAddress = $this->getShippingAddress();

            if (empty($shippingAddress['region_id']) && (!empty($reqStateCode) || !empty($reqState))) {
                $warnings[] = __('Unable to map state/state_code to region');
            }

            $quote->getShippingAddress()->addData($shippingAddress);
        } else {
            $quote->getShippingAddress()->addData([]);
        }

        if ($coupon = $this->getCouponCode()) {
            $quote->setCouponCode($coupon);
        }

        $quote->collectTotals();
        $quote->getShippingAddress()->setCollectShippingRates(true)->collectShippingRates();

        if ($coupon = $this->getCouponCode()) {
            $oCoupon = $this->coupon->loadByCode($coupon);
            if ($oCoupon->getRuleId()) {
                $oRule = $this->saleRule->load($oCoupon->getRuleId());
                if ($oRule->getRuleId()) {
                    $appliedRules = explode(",", $quote->getAppliedRuleIds());
                    if (!in_array($oRule->getRuleId(), $appliedRules)) {
                        $quote->setCouponCode('');
                        $warnings[] = __('Coupon code is not applicable');
                    }
                } else {
                    $quote->setCouponCode('');
                    $warnings[] = __('Coupon code is not valid');
                }
            } else {
                $quote->setCouponCode('');
                $warnings[] = __('Coupon code is not valid');
            }
        }

        $rates = $quote->getShippingAddress()->getShippingRatesCollection();

        $response = $this->cartResponseBuilder->build($quote, false);

        if (!empty($warnings)) {
            $response['warnings'] = $warnings;
        }

        $response['shipping_options'] = [];
        foreach ($rates as $rate) {
            $quote->getShippingAddress()->setShippingMethod($rate->getCode());
            $shippingPrice = $this->taxHelper->getShippingPrice(
                $rate->getPrice(),
                true,
                $quote->getShippingAddress(),
                $quote->getCustomerTaxClassId(),
                $quote->getStoreId()
            );

            $response['shipping_options'][] = [
                'total_price' => $shippingPrice,
                'service_code' => $rate->getCode(),
                'service_type' => $rate->getCarrierTitle(),
                'service_name' => $rate->getMethodTitle(),
                'delivery_text' => $rate->getMethodDescription(),
                'attributes' => $rate->getData()
            ];
        }

        if ($this->hasShippingAddress()) {
            $response['checkout'] = [
                'shipping_address' => $quote->getShippingAddress()->getData(),
            ];
        }

        $response['quote']['currency'] = $store->getCurrentCurrencyCode();

        return $response;
    }

    /**
     * Add product to quote.
     *
     * @param array $item
     * @param Quote $quote
     *
     * @return void
     * @throws LocalizedException
     * @throws NoSuchEntityException
     * @throws Exception
     */
    private function addProductToQuote(array $item, Quote $quote): void
    {
        $storeId = $quote->getStoreId();
        $product = $this->productRepository->getById($item['id'], false, $storeId, true);
        if (!$product || !$product->getId()) {
            // phpcs:ignore Magento2.Exceptions.DirectThrow.FoundDirectThrow
            throw new Exception(__('Product %s does not exist', $item['id']));
        }

        $params = ['qty' => $item['quantity']];
        if (isset($item['variant_id'])) {
            if ($product->getTypeId() != 'configurable') {
                // phpcs:ignore Magento2.Exceptions.DirectThrow.FoundDirectThrow
                throw new Exception('Variants are not supported');
            }

            list($superAttribute, $item) = $this->processVariants($product, $item);

            $params['super_attribute'] = $superAttribute;
        }

        if (isset($item['options'])) {
            $options = [];
            foreach ($item['options'] as $option) {
                $options[$option['option_id']] = $option['value_id'] ?? $option['value'];
            }

            $params['options'] = $options;
        }

        $paramsObj = new DataObject($params);
        $quote->addProduct($product, $paramsObj);
    }

    /**
     * Process variants.
     *
     * @param ProductInterface $product
     * @param array $item
     *
     * @return array
     * @throws NoSuchEntityException
     * @throws Exception
     */
    private function processVariants(ProductInterface $product, array $item): array
    {
        $productAttributeOptions = $product->getTypeInstance(true)->getConfigurableAttributesAsArray($product);
        $variant = $this->productRepository->getById($item['variant_id']);
        if (!$variant || !$variant->getId()) {
            // phpcs:ignore Magento2.Exceptions.DirectThrow.FoundDirectThrow
            throw new Exception(__('Variant %s does not exist', $item['variant_id']));
        }

        $superAttribute = [];
        foreach ($productAttributeOptions as $productAttribute) {
            $allValues = array_column($productAttribute['values'], 'value_index');
            $currentProductValue = $variant->getData($productAttribute['attribute_code']);

            if (in_array($currentProductValue, $allValues)) {
                $superAttribute[$productAttribute['attribute_id']] = $currentProductValue;
                if (isset($item['options'])) {
                    foreach ($item['options'] as $key => $option) {
                        if ($option['option_id'] == $productAttribute['attribute_id']) {
                            unset($item['options'][$key]);
                        }
                    }
                }
            }
        }

        return [$superAttribute, $item];
    }
}
