<?php

namespace Glopal\Connector\Controller\V1;

use Glopal\Connector\Helper\Configuration;
use Glopal\Connector\Helper\ModuleData;
use Magento\Framework\App\Action\Context;
use Magento\Framework\Controller\Result\JsonFactory;
use Magento\Framework\DB\Transaction;
use Magento\Framework\Exception\InputException;
use Magento\Framework\Exception\LocalizedException;
use Magento\Framework\Exception\NoSuchEntityException;
use Magento\Framework\Exception\NotFoundException;
use Magento\Framework\Exception\RuntimeException;
use Magento\Sales\Model\OrderRepository;
use Magento\Sales\Model\Service\InvoiceService;

/**
 * @SuppressWarnings(PHPMD.AllPurposeAction)
 */
class Invoice extends V1Abstract
{
    /** @var string */
    private const INVOICE_COMMENT_CREATED_BY_GLOPAL = '--INVOICE WAS CREATED BY GLOPAL API--';

    /** @var OrderRepository */
    private $orderRepository;

    /** @var InvoiceService */
    private $invoiceService;

    /** @var Transaction */
    private $transaction;

    /**
     * @param Context $context
     * @param JsonFactory $jsonResultFactory
     * @param Configuration $configuration
     * @param ModuleData $helper
     * @param OrderRepository $orderRepository
     * @param InvoiceService $invoiceService
     * @param Transaction $transaction
     */
    public function __construct(
        Context $context,
        JsonFactory $jsonResultFactory,
        Configuration $configuration,
        ModuleData $helper,
        OrderRepository $orderRepository,
        InvoiceService $invoiceService,
        Transaction $transaction
    ) {
        parent::__construct($context, $jsonResultFactory, $configuration, $helper);

        $this->orderRepository = $orderRepository;
        $this->invoiceService = $invoiceService;
        $this->transaction = $transaction;
    }

    /**
     * @inheritdoc
     */
    protected function process(): array
    {
        if ($this->getRequest()->isGet()) {
            return $this->listInvoices();
        }
        if ($this->getRequest()->isPost()) {
            return $this->createInvoice();
        }

        throw new NotFoundException(__('Wrong method used'));
    }

    /**
     * Creating invoice.
     *
     * @return array
     * @throws NotFoundException if we can't find the order in the database or the order_id param was empty
     * @throws RuntimeException in case the order was already invoiced or it can't be invoiced
     * @throws InputException
     * @throws LocalizedException
     * @throws NoSuchEntityException
     */
    protected function createInvoice(): array
    {
        $orderId = $this->getRequest()->getParam('order_id', null);
        if (empty($orderId)) {
            throw new NotFoundException(__('Order Id was not provided'));
        }
        $order = $this->orderRepository->get($orderId);
        if (null == $order->getId()) {
            throw new NotFoundException(__('Wrong order Id was provided'));
        }
        if ($order->hasInvoices()) {
            throw new RuntimeException(__('The order was already invoiced'));
        }
        if (!$order->canInvoice()) {
            throw new RuntimeException(__('The order can not be invoiced'));
        }

        $invoice = $this->invoiceService->prepareInvoice($order);
        $comment = self::INVOICE_COMMENT_CREATED_BY_GLOPAL;
        $invoice->addComment($comment);
        $invoice->register();
        $transactionSave = $this->transaction->addObject(
            $invoice
        )->addObject(
            $invoice->getOrder()
        );
        $transactionSave->save();

        return [
            'invoice_id' => $invoice->getId(),
            'grand_total' => $invoice->getGrandTotal(),
            'total_qty' => $invoice->getTotalQty(),
            'increment_id' => $invoice->getIncrementId(),
            'createdByGlopal' => true
        ];
    }

    /**
     * Retrieve list of invoices.
     *
     * @return array
     * @throws InputException
     * @throws NoSuchEntityException
     * @throws NotFoundException if we can't find the order in the database or the order_id param was empty
     */
    protected function listInvoices(): array
    {
        $orderId = $this->getRequest()->getParam('order_id', null);
        if (empty($orderId)) {
            throw new NotFoundException(__('Order Id was not provided'));
        }
        $order = $this->orderRepository->get($orderId);
        if (null == $order->getId()) {
            throw new NotFoundException(__('Wrong order Id was provided'));
        }
        if (!$order->hasInvoices()) {
            return [];
        }
        $invoices = array_values($order->getInvoiceCollection()->getItems());

        return array_map(function ($invoice) {
            $comments = $invoice->getCommentsCollection()->getItems();
            $createdByGlopal = array_filter($comments, function ($comment) {
                return $comment->getComment() == Invoice::INVOICE_COMMENT_CREATED_BY_GLOPAL;
            });

            return [
                'invoice_id' => $invoice->getId(),
                'grand_total' => $invoice->getGrandTotal(),
                'total_qty' => $invoice->getTotalQty(),
                'increment_id' => $invoice->getIncrementId(),
                'createdByGlopal' => (bool)count($createdByGlopal)
            ];
        }, $invoices);
    }
}
