<?php

namespace Glopal\Connector\Controller\V1;

use Glopal\Connector\Exception\Forbidden;
use Glopal\Connector\Helper\Configuration;
use Glopal\Connector\Helper\ModuleData;
use Magento\Framework\App\Action\Action;
use Magento\Framework\App\Action\Context;
use Magento\Framework\Controller\Result\JsonFactory;
use Magento\Framework\Exception\NotFoundException;
use Magento\Framework\Exception\RuntimeException;
use Throwable;

/**
 * @SuppressWarnings(PHPMD.AllPurposeAction)
 */
class V1Abstract extends Action
{
    /** @var Configuration */
    protected $configuration;

    /** @var JsonFactory */
    private $jsonResultFactory;

    /** @var ModuleData */
    private $helper;

    /**
     * @param Context $context
     * @param JsonFactory $jsonResultFactory
     * @param Configuration $configuration
     * @param ModuleData $helper
     */
    public function __construct(
        Context $context,
        JsonFactory $jsonResultFactory,
        Configuration $configuration,
        ModuleData $helper
    ) {
        parent::__construct($context);

        $this->jsonResultFactory = $jsonResultFactory;
        $this->configuration = $configuration;
        $this->helper = $helper;
    }

    /**
     * Pre dispatch method.
     *
     * @return void
     */
    private function preDispatch(): void
    {
        if (!$this->configuration->getGeneralFlag('signature_verification')) {
            return;
        }

        $actionName = $this->getRequest()->getControllerName();
        $actionName = strtolower($actionName);
        try {
            switch ($actionName) {
                case 'v1':
                default:
                    $this->helper->verifyRequestSignatureWithException();
                    break;
            }
        } catch (Forbidden $e) {
            $this->send([], true, $e->getMessage(), 403);
        } catch (Throwable $e) {
            $this->send([], true, $e->getMessage(), 403);
        }
    }

    /**
     * @inheritdoc
     */
    public function execute()
    {
        $this->preDispatch();

        try {
            $this->send($this->process());
        } catch (RuntimeException $e) {
            $this->send([], true, $e->getMessage(), 500);
        } catch (NotFoundException $e) {
            $this->send([], true, $e->getMessage(), 400);
        } catch (Throwable $e) {
            $this->send([], true, $e->getMessage(), 500);
        }

        return $this->jsonResultFactory->create([]);
    }

    /**
     * Process method.
     *
     * @return array
     */
    protected function process(): array
    {
        return [];
    }

    /**
     * Send data.
     *
     * @param array $data
     * @param bool $error
     * @param string $message
     * @param bool|int $responseCode
     */
    final public function send(array $data, bool $error = false, string $message = '', bool $responseCode = false)// phpcs:ignore
    {
        $result = [
            'error' => $error,
            'message' => __($message),
            'data' => $data
        ];
        $resultJson = $this->jsonResultFactory->create();
        $resultJson->setData($result);

        if (is_numeric($responseCode)) {
            $resultJson->setHttpResponseCode($responseCode);
        }

        $resultJson->renderResult($this->getResponse());
        $this->getResponse()->sendResponse();

        // phpcs:ignore
        exit;
    }
}
