<?php

namespace Glopal\Connector\Helper;

use Magento\Config\Model\ResourceModel\Config;
use Magento\Framework\App\Cache\TypeListInterface;
use Magento\Framework\App\Config\ReinitableConfigInterface;
use Magento\Framework\App\Config\ScopeConfigInterface;
use Magento\Framework\Exception\NotFoundException;
use Magento\Framework\Serialize\SerializerInterface;
use Magento\Store\Model\ScopeInterface;
use Magento\Store\Model\StoreManagerInterface;

class Configuration
{
    private const CONFIG_PATH_GENERAL = 'glopal_connector/general';
    private const CONFIG_PATH_CART = 'glopal_cart_transfer';
    private const CONFIG_PATH_CART_PRODUCT_DATA = self::CONFIG_PATH_CART . '/product_data';
    private const CONFIG_PATH_CHECKOUT = 'glopal_checkout';
    private const CONFIG_PATH_OLD = 'webinterpret_connector';

    /** @var ScopeConfigInterface */
    private $scopeConfig;

    /** @var Config */
    private $resourceConfig;

    /** @var TypeListInterface */
    private $cacheTypeList;

    /** @var ReinitableConfigInterface */
    private $reinitableConfig;

    /** @var SerializerInterface */
    private $jsonSerializer;

    /** @var StoreManagerInterface */
    private $storeManager;

    /**
     * @param ScopeConfigInterface $scopeConfig
     * @param Config $resourceConfig
     * @param TypeListInterface $cacheTypeList
     * @param ReinitableConfigInterface $reinitableConfig
     * @param SerializerInterface $jsonSerializer
     * @param StoreManagerInterface $storeManager
     */
    public function __construct(
        ScopeConfigInterface $scopeConfig,
        Config $resourceConfig,
        TypeListInterface $cacheTypeList,
        ReinitableConfigInterface $reinitableConfig,
        SerializerInterface $jsonSerializer,
        StoreManagerInterface $storeManager
    ) {
        $this->scopeConfig = $scopeConfig;
        $this->resourceConfig = $resourceConfig;
        $this->cacheTypeList = $cacheTypeList;
        $this->reinitableConfig = $reinitableConfig;
        $this->jsonSerializer = $jsonSerializer;
        $this->storeManager = $storeManager;
    }

    /**
     * Get configuration flag.
     *
     * @param string $code
     * @param null|string|int $storeId
     * @return bool
     *
     * @SuppressWarnings(PHPMD.BooleanGetMethodName)
     */
    public function getFlag(string $code, $storeId = null): bool
    {
        $code = $this->handleOldSettings($code);

        return $this->scopeConfig->isSetFlag($code, ScopeInterface::SCOPE_STORE, $storeId);
    }

    /**
     * Get general configuration flag.
     *
     * @param string $code
     * @param null|string|int $storeId
     * @return bool
     *
     * @SuppressWarnings(PHPMD.BooleanGetMethodName)
     */
    public function getGeneralFlag(string $code, $storeId = null): bool
    {
        return $this->getFlag(self::CONFIG_PATH_GENERAL . '/' . $code, $storeId);
    }

    /**
     * Get scoped value.
     *
     * @param string $code
     * @param string $scope
     * @param null|string|int $storeId
     * @return string
     */
    public function getScopedValue(string $code, string $scope, $storeId = null): string
    {
        $code = $this->handleOldSettings($code);
        $scope = $this->handleReadScope($scope);
        $value = $this->scopeConfig->getValue($code, $scope, $storeId);

        return $value === null ? '' : $value;
    }

    /**
     * Get configuration value.
     *
     * @param string $code
     * @param null|string|int $storeId
     * @return string
     */
    public function getValue(string $code, $storeId = null): string
    {
        return $this->getScopedValue($code, ScopeInterface::SCOPE_STORE, $storeId);
    }

    /**
     * Get general configuration value.
     *
     * @param string $code
     * @param null|string|int $storeId
     * @return string
     */
    public function getGeneralValue(string $code, $storeId = null): string
    {
        return $this->getValue(self::CONFIG_PATH_GENERAL . '/' . $code, $storeId);
    }

    /**
     * Get cart configuration value.
     *
     * @param string $code
     * @param null|string|int $storeId
     * @return string
     */
    public function getCartValue(string $code, $storeId = null): string
    {
        return $this->getValue(self::CONFIG_PATH_CART . '/' . $code, $storeId);
    }

    /**
     * Save cart configuration value.
     *
     * @param string $code
     * @param string $value
     * @param string $scope
     * @param null|string|int $storeId
     * @return void
     * @throws NotFoundException
     */
    public function saveCartValue(
        string $code,
        string $value,
        string $scope = ScopeConfigInterface::SCOPE_TYPE_DEFAULT,
        $storeId = null
    ): void {
        $this->saveValue(self::CONFIG_PATH_CART . '/' . $code, $value, $scope, $storeId);
    }

    /**
     * Get cart product configuration.
     *
     * @param string $code
     * @param null|string|int $storeId
     * @return array|string
     */
    public function getCartProductDataValue(string $code, $storeId = null): array
    {
        $value = $this->getValue(self::CONFIG_PATH_CART_PRODUCT_DATA . '/' . $code, $storeId);

        if (is_string($value)) {
            $value = $this->jsonSerializer->unserialize($value);
        }

        return $value;
    }

    /**
     * Save cart product configuration value.
     *
     * @param string $code
     * @param string $value
     * @param string $scope
     * @param null|string|int $storeId
     * @return void
     * @throws NotFoundException
     */
    public function saveCartProductDataValue(
        string $code,
        string $value,
        string $scope = ScopeConfigInterface::SCOPE_TYPE_DEFAULT,
        $storeId = null
    ): void {
        $this->saveValue(self::CONFIG_PATH_CART_PRODUCT_DATA . '/' . $code, $value, $scope, $storeId);
    }

    /**
     * Get checkout configuration value.
     *
     * @param string $code
     * @param null|string|int $storeId
     * @return string
     */
    public function getCheckoutValue(string $code, $storeId = null): string
    {
        return $this->getValue(self::CONFIG_PATH_CHECKOUT . '/' . $code, $storeId);
    }

    /**
     * Get checkout configuration flag.
     *
     * @param string $code
     * @param null|string|int $storeId
     * @return bool
     *
     * @SuppressWarnings(PHPMD.BooleanGetMethodName)
     */
    public function getCheckoutFlag(string $code, $storeId = null): bool
    {
        return $this->getFlag(self::CONFIG_PATH_CHECKOUT . '/' . $code, $storeId);
    }

    /**
     * Save checkout configuration value.
     *
     * @param string $code
     * @param string $value
     * @param string $scope
     * @param null|string|int $storeId
     * @return void
     * @throws NotFoundException
     */
    public function saveCheckoutValue(
        string $code,
        string $value,
        string $scope = ScopeConfigInterface::SCOPE_TYPE_DEFAULT,
        $storeId = null
    ): void {
        $this->saveValue(self::CONFIG_PATH_CHECKOUT . '/' . $code, $value, $scope, $storeId);
    }

    /**
     * Save configuration value.
     *
     * @param string $code
     * @param string $value
     * @param string $scope
     * @param null|string|int $storeId
     * @return void
     * @throws NotFoundException
     */
    public function saveValue(
        string $code,
        string $value,
        string $scope = ScopeConfigInterface::SCOPE_TYPE_DEFAULT,
        $storeId = null
    ): void {
        $this->validateScope($scope, $storeId);

        $code = $this->handleOldSettings($code);
        $this->resourceConfig->saveConfig($code, $value, $scope, $storeId ?: 0);
        $this->cacheTypeList->cleanType('config');
    }

    /**
     * Save general configuration value.
     *
     * @param string $code
     * @param string $value
     * @param string $scope
     * @param null|string|int $storeId
     * @return void
     * @throws NotFoundException
     */
    public function saveGeneralValue(
        string $code,
        string $value,
        string $scope = ScopeConfigInterface::SCOPE_TYPE_DEFAULT,
        $storeId = null
    ): void {
        $this->saveValue(self::CONFIG_PATH_GENERAL . '/' . $code, $value, $scope, $storeId);
    }

    /**
     * Delete configuration value.
     *
     * @param string $code
     * @param string $scope
     * @param null|string|int $storeId
     * @return Config
     * @throws NotFoundException
     */
    public function deleteValue(
        string $code,
        string $scope = ScopeConfigInterface::SCOPE_TYPE_DEFAULT,
        $storeId = null
    ): Config {
        $this->validateScope($scope, $storeId);

        $code = $this->handleOldSettings($code);

        return $this->resourceConfig->deleteConfig($code, $scope, $storeId ?: 0);
    }

    /**
     * Reinitialise config.
     *
     * @return void
     */
    public function reinitialiseConfig(): void
    {
        $this->reinitableConfig->reinit();
    }

    /**
     * Handling old settings.
     *
     * @param string $code
     * @return string
     */
    private function handleOldSettings(string $code): string
    {
        $prefix = self::CONFIG_PATH_OLD . '/';
        $len = strlen($prefix);
        if (strncmp($code, $prefix, $len) === 0) {
            return self::CONFIG_PATH_GENERAL . '/' . substr($code, $len);
        }

        return $code;
    }

    /**
     * Handle read scope.
     *
     * @param string $scope
     * @return string
     */
    private function handleReadScope(string $scope): string
    {
        if ($scope === 'stores') {
            return ScopeInterface::SCOPE_STORE;
        } elseif ($scope === 'groups') {
            return ScopeInterface::SCOPE_GROUP;
        } elseif ($scope === 'websites') {
            return ScopeInterface::SCOPE_WEBSITE;
        } else {
            return $scope;
        }
    }

    /**
     * Validates if scope exists.
     *
     * @param string $scope
     * @param null|string|int $scopeId
     * @return void
     * @throws NotFoundException
     *
     * @SuppressWarnings(PHPMD.CyclomaticComplexity)
     */
    private function validateScope(string $scope, $scopeId)
    {
        // Skip validation when scope id is null.
        if (null === $scopeId) {
            return;
        }

        if ($scope === ScopeInterface::SCOPE_WEBSITES || $scope === ScopeInterface::SCOPE_WEBSITE) {
            $websiteIds = [];
            foreach ($this->storeManager->getWebsites(true) as $website) {
                $websiteIds[] = $website->getId();
            }

            if (!in_array($scopeId, $websiteIds)) {
                throw new NotFoundException(__(sprintf('Website with given ID: %s doesn\'t exists', $scopeId)));
            }
        }

        if ($scope === ScopeInterface::SCOPE_STORES || $scope === ScopeInterface::SCOPE_STORE) {
            $storeIds = [];
            foreach ($this->storeManager->getStores(true) as $store) {
                $storeIds[] = $store->getId();
            }

            if (!in_array($scopeId, $storeIds)) {
                throw new NotFoundException(__(sprintf('Store with given ID: %s doesn\'t exists', $scopeId)));
            }
        }
    }
}
