<?php

namespace Glopal\Connector\Helper;

use Magento\Framework\App\ResourceConnection;
use Magento\Framework\DB\Adapter\AdapterInterface;
use PDO;
use PDOException;
use PDOStatement;
use Psr\Log\LoggerInterface;
use Zend_Db;
use Zend_Db_Statement_Exception;
use Zend_Db_Statement_Interface;

class DbLink
{
    /** @var string[] */
    public $noResult = [
        'delete',
        'update',
        'move',
        'truncate',
        'insert',
        'set',
        'create',
        'drop',
        'replace',
        'start',
        'rollback',
        'commit'
    ];

    /** @var int */
    private $_insertedId = 0;

    /** @var int */
    private $_affectedRows = 0;

    /** @var bool|null|PDO */
    private $dataBaseHandle = null;

    /** @var LoggerInterface */
    private $logger;

    /**
     * @param ResourceConnection $resourceConnection
     * @param LoggerInterface $logger
     */
    public function __construct(
        ResourceConnection $resourceConnection,
        LoggerInterface $logger
    ) {
        $this->dataBaseHandle = $resourceConnection->getConnection();
        $this->logger = $logger;

        if (!$this->dataBaseHandle) {
            $this->logger->debug('[DbLink]: Can not connect to DB');

            // phpcs:ignore
            exit('[ERROR] MySQL Query Error: Can not connect to DB');
        }
    }

    /**
     * Run local query.
     *
     * @param string $sql sql query
     *
     * @return array|bool
     * @throws Zend_Db_Statement_Exception
     */
    public function localQuery(string $sql)
    {
        $sth = $this->dataBaseHandle->query($sql);

        foreach ($this->noResult as $statement) {
            if (!$sth || strpos(strtolower(trim($sql)), $statement) === 0) {
                return true;
            }
        }

        return $sth->fetchAll();
    }

    /**
     * Run query.
     *
     * @param string $sql sql query
     * @param int $fetchType fetch Type
     *
     * @return array
     * @throws Zend_Db_Statement_Exception
     */
    public function query(string $sql, int $fetchType): array
    {
        $result = [
            'result' => null,
            'message' => '',
            'fetchedFields' => []
        ];

        $this->disableChecks();
        $this->setCharset();

        try {
            $res = $this->runQuery($sql, $fetchType);
        } catch (PDOException $e) {
            $this->logger->debug(sprintf('[DBLink] Error %s, %s', $e->getCode(), $e->getMessage()));
            $result['message'] = '[ERROR] Mysql Query Error: ' . $e->getCode() . ', ' . $e->getMessage();

            return $result;
        }

        foreach ($this->noResult as $statement) {
            if (!$res || strpos(strtolower(trim($sql)), $statement) === 0) {
                $result['result'] = true;
                return $result;
            }
        }

        $rows = $res->fetchAll();

        // phpcs:ignore
        if (isset($_GET['disable_checks'])) {
            $this->localQuery("SET SQL_MODE=IFNULL(@OLD_SQL_MODE,'')");
            $this->localQuery("SET FOREIGN_KEY_CHECKS=IFNULL(@OLD_FOREIGN_KEY_CHECKS,0)");
        }

        $result['result'] = $rows;

        unset($res);

        return $result;
    }

    /**
     * Get last inserted id.
     *
     * @return int
     */
    public function getLastInsertId(): int
    {
        return $this->_insertedId;
    }

    /**
     * Get affected rows.
     *
     * @return int
     */
    public function getAffectedRows(): int
    {
        return $this->_affectedRows;
    }

    /**
     * Destruct.
     *
     * @return void
     */
    public function __destruct()
    {
        $this->dataBaseHandle = null;
    }

    /**
     * Disable SQL checks.
     *
     * @return void
     * @throws Zend_Db_Statement_Exception
     */
    private function disableChecks(): void
    {
        // phpcs:ignore
        if (isset($_GET['disable_checks'])) {
            $this->localQuery('SET @OLD_FOREIGN_KEY_CHECKS=@@FOREIGN_KEY_CHECKS, FOREIGN_KEY_CHECKS=0');
            $this->localQuery("SET @OLD_SQL_MODE=@@SQL_MODE, SQL_MODE='NO_AUTO_VALUE_ON_ZERO'");
        }
    }

    /**
     * Set SQL char set.
     *
     * @return void
     * @throws Zend_Db_Statement_Exception
     */
    private function setCharset(): void
    {
        // phpcs:disable
        if (isset($_REQUEST['set_names'])) {
            $this->localQuery("SET NAMES '" . ($_REQUEST['set_names']) . "'");
            $this->localQuery("SET CHARACTER SET '" . ($_REQUEST['set_names']) . "'");
            $this->localQuery("SET CHARACTER_SET_CONNECTION = '" . ($_REQUEST['set_names']) . "'");
        }
        // phpcs:enable
    }

    /**
     * Run sql query.
     *
     * @param string $sql
     * @param int $fetchType
     * @return bool|AdapterInterface|PDOStatement|Zend_Db_Statement_Interface
     * @throws Zend_Db_Statement_Exception
     */
    private function runQuery(string $sql, int $fetchType)
    {
        $cleanSql = strtolower(trim($sql));

        if ($cleanSql == 'start transaction') {
            $res = $this->dataBaseHandle->beginTransaction();
        } elseif ($cleanSql == 'commit') {
            $res = $this->dataBaseHandle->commit();
        } elseif ($cleanSql == 'rollback') {
            $res = $this->dataBaseHandle->rollBack();
        } else {
            $res = $this->dataBaseHandle->query($sql);

            switch ($fetchType) {
                case 3:
                    $res->setFetchMode(Zend_Db::FETCH_BOTH);
                    break;
                case 2:
                    $res->setFetchMode(Zend_Db::FETCH_NUM);
                    break;
                case 1:
                default:
                    $res->setFetchMode(Zend_Db::FETCH_ASSOC);
                    break;
            }
            $this->_affectedRows = $res->rowCount();
            $this->_insertedId = $this->dataBaseHandle->lastInsertId();
        }

        return $res;
    }
}
