<?php

namespace Glopal\Connector\Helper;

use DateTime;
use Exception;
use Magento\Framework\Serialize\SerializerInterface;
use Magento\Store\Model\StoreManagerInterface;

class Installation
{
    /** @var StoreManagerInterface */
    private $storeManager;

    /** @var SerializerInterface */
    private $jsonSerializer;

    /** @var ModuleData */
    private $helper;

    /** @var Configuration */
    private $configuration;

    /** @var CurlRequest */
    private $curlRequest;

    /**
     * @param StoreManagerInterface $storeManager
     * @param SerializerInterface $jsonSerializer
     * @param ModuleData $helper
     * @param Configuration $configuration
     * @param CurlRequest $curlRequest
     */
    public function __construct(
        StoreManagerInterface $storeManager,
        SerializerInterface $jsonSerializer,
        ModuleData $helper,
        Configuration $configuration,
        CurlRequest $curlRequest
    ) {
        $this->storeManager = $storeManager;
        $this->jsonSerializer = $jsonSerializer;
        $this->helper = $helper;
        $this->configuration = $configuration;
        $this->curlRequest = $curlRequest;
    }

    /**
     * Check if store view is notified.
     *
     * @param null|string|int $storeId
     *
     * @return bool
     * @throws Exception
     */
    public function isStoreInstallationNotified($storeId): bool
    {
        return strlen($this->configuration->getGeneralValue('installation_notified_at', $storeId)) > 0;
    }

    /**
     * Notify installation.
     *
     * @param bool $force
     * @return bool|void
     * @throws Exception
     */
    public function notifyInstallation(bool $force = false)
    {
        if ($this->isRetry($force)) {
            return;
        }

        $url = $this->configuration->getGeneralValue('url_install_magento');
        $requestBody = $this->getStoresData();
        $data = $this->jsonSerializer->serialize($requestBody);

        try {
            $response = $this->curlRequest->sendPostRequest($url, $data);

            if (($json = $this->jsonSerializer->unserialize($response)) === null) {
                // phpcs:ignore
                throw new Exception("Invalid JSON response: {$response}");
            }

            if (isset($json['magento_config'])
                && is_array($json['magento_config'])
                && count($json['magento_config'])
            ) {
                foreach ($json['magento_config'] as $c) {
                    $this->configuration->saveValue(
                        $c['path'],
                        $c['value'],
                        $c['scope'],
                        $c['scope_id']
                    );
                }

                $this->configuration->reinitialiseConfig();
            }

            return true;
        } catch (Exception $e) { // phpcs:ignore
            $this->configuration->saveGeneralValue(
                'last_request_error_load',
                $e->getMessage() . "\n" . $e->getTraceAsString()
            );
            $this->configuration->reinitialiseConfig();
        }

        $this->_scheduleNextNotify();

        return false;
    }

    /**
     * Is retry request?
     *
     * @param bool $force
     * @return bool
     * @throws Exception
     */
    private function isRetry(bool $force): bool
    {
        if (!$force) {
            $retry = $this->configuration->getGeneralValue('installation_notified_retry');

            if (!empty($retry) && (new DateTime() < new DateTime($retry))) {
                // scheduled for future attempt
                return true;
            }
        }

        return false;
    }

    /**
     * Schedule next installation.
     *
     * @return void
     */
    private function _scheduleNextNotify()
    {
        $newDate = new DateTime();
        $newDate->modify('+5 minutes');
        $stamp = $newDate->format('Y-m-d H:i:s');
        $this->configuration->saveGeneralValue('installation_notified_retry', $stamp);
        $this->configuration->reinitialiseConfig(); // TODO Do we need reinit here?
    }

    /**
     * Get store base url based on secure settings.
     *
     * @param null|string|int $storeId
     * @return string
     */
    public function getStoreBaseUrl($storeId = 0): string
    {
        if ($this->configuration->getFlag('web/secure/use_in_frontend', $storeId)) {
            return $this->configuration->getValue('web/secure/base_url', $storeId);
        }

        return $this->configuration->getValue('web/unsecure/base_url', $storeId);
    }

    /**
     * Get all website / stores data.
     *
     * @return array
     */
    public function getStoresData(): array
    {
        $key = $this->helper->getGlopalKey();

        return [
            'plugin_version' => $this->helper->getExtensionVersion(),
            'magento_version' => $this->helper->getMagentoVersion(),
            'webinterpret_key' => $key,  // deprecated, use store_key instead
            'store_key' => $key,
            'websites' => $this->getWebsitesData(),
        ];
    }

    /**
     * Get websites data.
     *
     * @return array
     */
    public function getWebsitesData(): array
    {
        $data = [];
        $websites = $this->storeManager->getWebsites();

        foreach ($websites as $website) {
            $websiteData = [
                'id' => $website->getWebsiteId(),
                'code' => $website->getCode(),
                'name' => $website->getName(),
                'is_default' => $website->getIsDefault(),
                'stores' => [],
            ];
            $defaultGroup = $website->getDefaultGroupId();
            $groups = $website->getGroups();

            foreach ($groups as $group) {
                $groupData = [
                    'id' => $group->getGroupId(),
                    'name' => $group->getName(),
                    'is_default' => $defaultGroup == $group->getGroupId(),
                    'store_views' => [],
                ];
                $defaultStore = $group->getDefaultStoreId();
                $stores = $group->getStores();

                foreach ($stores as $store) {
                    $storeId = $store->getStoreId();
                    $storeData = [
                        'id' => $storeId,
                        'website_id' => $website->getWebsiteId(),
                        'group_id' => $group->getGroupId(),
                        'code' => $store->getCode(),
                        'name' => $store->getName(),
                        'is_default' => $defaultStore == $storeId,
                        'is_active' => $store->getIsActive(),
                        'url' => $this->getStoreBaseUrl($storeId),
                        'country' => $this->configuration->getValue(
                            'general/store_information/country_id',
                            $storeId
                        ),
                        'shipping_origin' => $this->configuration->getValue(
                            'shipping/origin/country_id',
                            $storeId
                        ),
                        'locale' => $this->configuration->getValue(
                            'general/locale/code',
                            $storeId
                        ),
                        'currency' => $this->configuration->getValue(
                            'currency/options/base',
                            $storeId
                        ),
                        'store_name' => $this->configuration->getValue(
                            'general/store_information/name',
                            $storeId
                        ),
                        'store_contact_email' => $this->configuration->getValue(
                            'trans_email/ident_general/email',
                            $storeId
                        ),
                        'store_contact_phone' => $this->configuration->getValue(
                            'general/store_information/phone',
                            $storeId
                        ),
                        'wi_installation_notified_at' => $this->configuration->getGeneralValue(
                            'installation_notified_at',
                            $storeId
                        ),
                        'wi_short_name' => $this->configuration->getGeneralValue(
                            'short_name',
                            $storeId
                        ),
                    ];

                    $groupData['store_views'][]= $storeData;
                }

                $websiteData['stores'][] = $groupData;
            }

            $data[] = $websiteData;
        }

        return $data;
    }
}
