<?php

namespace Glopal\Connector\Helper;

use Exception;
use Glopal\Connector\Exception\Forbidden;
use Magento\Framework\App\ProductMetadataInterface;
use Magento\Framework\Module\Dir;
use Magento\Framework\Module\Manager as ModuleManager;
use Magento\Framework\Module\ModuleListInterface;

class ModuleData
{
    private const MODULE_NAME = 'Glopal_Connector';

    /** @var ProductMetadataInterface */
    private $productMetadata;

    /** @var ModuleListInterface */
    private $moduleList;

    /** @var Dir */
    private $moduleDir;

    /** @var Configuration */
    private $configuration;

    /** @var ModuleManager */
    private $moduleManager;

    /**
     * @param ProductMetadataInterface $productMetadata
     * @param ModuleManager $moduleManager
     * @param ModuleListInterface $moduleList
     * @param Dir $moduleDir
     * @param Configuration $configuration
     */
    public function __construct(
        ProductMetadataInterface $productMetadata,
        ModuleManager $moduleManager,
        ModuleListInterface $moduleList,
        Dir $moduleDir,
        Configuration $configuration
    ) {
        $this->productMetadata = $productMetadata;
        $this->moduleList = $moduleList;
        $this->moduleDir = $moduleDir;
        $this->configuration = $configuration;
        $this->moduleManager = $moduleManager;
    }

    /**
     * Is pass database link enabled?
     *
     * @return bool
     */
    public function isPassDbLinkEnabled(): bool
    {
        return $this->configuration->getGeneralFlag('pass_db_link');
    }

    /**
     * Gets Glopal key.
     *
     * @return string
     */
    public function getGlopalKey(): string
    {
        $key = $this->configuration->getGeneralValue('key');
        $key2 = trim($key);
        if ($key != $key2) {
            $this->configuration->saveGeneralValue('key', $key);
        }

        if (empty($key2)) {
            // phpcs:ignore
            $key2 = md5(uniqid(mt_rand(), true));
            $this->configuration->saveGeneralValue('key', $key2);
            $this->configuration->reinitialiseConfig();
        }

        return (string) $key2;
    }

    /**
     * Get extension version.
     *
     * @return string
     */
    public function getExtensionVersion(): string
    {
        return (string) $this->moduleList->getOne(self::MODULE_NAME)['setup_version'];
    }

    /**
     * Is extension enabled?
     *
     * @return bool
     */
    public function isExtensionEnabled(): bool
    {
        try {
            return (bool) $this->moduleManager->isOutputEnabled(self::MODULE_NAME);
        } catch (Exception $e) {
            return (bool) $this->moduleManager->isEnabled(self::MODULE_NAME);
        }
    }

    /**
     * Get Magento version.
     *
     * @return string
     */
    public function getMagentoVersion(): string
    {
        return $this->productMetadata->getVersion();
    }

    /**
     * Get extension root folder.
     *
     * @return string
     */
    public function getExtensionRoot(): string
    {
        return $this->moduleDir->getDir(self::MODULE_NAME);
    }

    /**
     * Verify request signature.
     *
     * @return bool
     */
    public function verifyRequestSignature(): bool
    {
        // phpcs:disable
        $verificationResult = $this->verifySignature();
        if (!$verificationResult['status']) {
            echo $verificationResult['message'] . PHP_EOL;
        }

        return $verificationResult['status'];
        // phpcs:enable
    }

    /**
     * Verify request signature and throw exception in any issue.
     *
     * @return bool
     * @throws Forbidden
     */
    public function verifyRequestSignatureWithException(): bool
    {
        // phpcs:disable
        $verificationResult = $this->verifySignature();
        if (!$verificationResult['status']) {
            throw new Forbidden(__($verificationResult['message']));
        }

        return $verificationResult['status'];
        // phpcs:enable
    }

    /**
     * Verify signature, returns status and reason message.
     *
     * @return array
     */
    private function verifySignature(): array
    {
        // phpcs:disable
        if (!function_exists('openssl_verify')) {
            return ['status' => false, 'message' => 'OpenSSL is required.'];
        }

        if (empty($_SERVER['HTTP_WEBINTERPRET_REQUEST_ID'])
            || empty($_SERVER['HTTP_WEBINTERPRET_SIGNATURE'])
            || empty($_SERVER['HTTP_WEBINTERPRET_SIGNATURE_VERSION'])
        ) {
            return ['status' => false, 'message' => 'Request needs to be signed by Glopal.'];
        }

        self::filterAllowedKeys($_GET);
        self::filterAllowedKeys($_POST);

        $requestId = $_SERVER['HTTP_WEBINTERPRET_REQUEST_ID'];
        $signature = $_SERVER['HTTP_WEBINTERPRET_SIGNATURE'];
        $signature = base64_decode($signature);

        $version = $_SERVER['HTTP_WEBINTERPRET_SIGNATURE_VERSION'];

        if ($version == 2) {
            // Query data
            $queryArray = $_GET;
            ksort($queryArray);
            $queryData = serialize($queryArray);

            // Post data
            $postData = $_POST;
            ksort($postData);
            $postData = serialize($postData);

            $data = $requestId . $queryData . $postData;
            // Verify signature
            $publicKey = $this->configuration->getGeneralValue('public_key');

            $ok = openssl_verify($data, $signature, $publicKey, OPENSSL_ALGO_SHA1);
            if ($ok == 1) {
                return ['status' => true, 'message' => ''];
            }

            $message = 'Error checking signature';
            if ($ok == 0) {
                $message = 'Incorrect signature.';
            }

            return ['status' => false, 'message' => $message];
        } else {
            return ['status' => false, 'message' => 'Not supported signature version.'];
        }
        // phpcs:enable
    }

    /**
     * Leave only original keys so that signature verification can pass
     *
     * @param array $array
     * @return void
     */
    private function filterAllowedKeys(array &$array): void
    {
        if (isset($array['allowed_keys'])) {
            $allowedKeys = explode(',', $array['allowed_keys']);
            $allowedKeys[] = 'allowed_keys';
            $array = array_intersect_key($array, array_flip($allowedKeys));
        }
    }
}
