<?php

namespace Glopal\Connector\Helper;

use Magento\Customer\Model\Address\Config as AddressConfig;
use Magento\Directory\Model\ResourceModel\Region\CollectionFactory;
use Magento\Sales\Api\Data\OrderInterface;

class Order
{
    /** @var CollectionFactory */
    private $regionCollectionFactory;

    /** @var Configuration */
    private $configuration;

    /** @var AddressConfig */
    private $addressConfig;

    /**
     * @param CollectionFactory $regionCollectionFactory
     * @param Configuration $configuration
     * @param AddressConfig $addressConfig
     */
    public function __construct(
        CollectionFactory $regionCollectionFactory,
        Configuration $configuration,
        AddressConfig $addressConfig
    ) {
        $this->regionCollectionFactory = $regionCollectionFactory;
        $this->configuration = $configuration;
        $this->addressConfig = $addressConfig;
    }

    /**
     * Convert requested address array data to Magento specific address data.
     *
     * @param array $address
     *
     * @return array
     */
    public function convertAddressFromRequest(array $address): array
    {
        return [
            'firstname' => $address['first_name'],
            'lastname' => $address['last_name'],
            'street' => $address['address1'],
            'city' => $address['city'],
            'country_id' => $address['country_code'],
            'region' => $address['state'] ?? '',
            'region_id' => $this->getRegionIdByCode($address['state'] ?? '', $address['country_code']),
            'postcode' => $address['postcode'],
            'telephone' => $address['telephone'] ?? '',
            'fax' => '',
            'save_in_address_book' => 0,
        ];
    }

    /**
     * Is create invoice during order creation.
     *
     * @param null|string|int $storeId
     * @return bool
     */
    public function isCreateInvoice($storeId = null): bool
    {
        return $this->configuration->getValue('glopal_order_creation/settings/create_invoice', $storeId);
    }

    /**
     * Get shipping address for the invoice
     *
     * @param null|string|int $storeId
     * @return string
     */
    public function getInvoiceShippingAddress($storeId = null): string
    {
        return $this->configuration->getValue('glopal_order_creation/settings/invoice_shipping_address', $storeId);
    }

    /**
     * Is Glopal order
     *
     * @param OrderInterface $order
     * @return bool
     */
    public function isGlopalOrder(OrderInterface $order): bool
    {
        $extensionAttributes =  $order->getExtensionAttributes();
        if (null === $extensionAttributes) {
            return false;
        }

        return (bool)$extensionAttributes->getGlAttribute();
    }

    /**
     * Get customer origin shipping address
     *
     * @param OrderInterface $order
     * @param string $typeCode
     * @return string|null
     */
    public function getCustomerOriginShippingAddress(OrderInterface $order, string $typeCode = 'html'): ?string
    {
        if (!$this->isGlopalOrder($order)) {
            return null;
        }

        $extensionAttributes = $order->getExtensionAttributes();
        $orderAttribute = $extensionAttributes->getGlAttribute();
        $formatType = $this->addressConfig->getFormatByCode($typeCode);

        if (!$formatType || !$formatType->getRenderer()) {
            return null;
        }

        $addressData = $orderAttribute->getCustomerShippingAddress();

        return (null !== $addressData) ? $formatType->getRenderer()->renderArray($addressData) : $addressData;
    }

    /**
     * Get region identity by region code
     *
     * @param string $region
     * @param string $countryId
     *
     * @return string
     */
    private function getRegionIdByCode(string $region, string $countryId): string
    {
        if (empty($region)) {
            return '';
        }

        $regionCollection = $this->regionCollectionFactory->create()
            ->addCountryFilter($countryId)
            ->addRegionNameFilter($region);

        $regions = $regionCollection->load()->getItems();

        if (empty($regions)) {
            return '';
        }

        $region = reset($regions);

        return $region->getRegionId();
    }
}
