<?php

namespace Glopal\Connector\Helper;

use Magento\Framework\App\RequestInterface;

class Signature
{
    private const REQUEST_ID_HEADER = 'X-Glopal-Request-Id';
    private const SIGNATURE_HEADER = 'X-Glopal-Signature';
    private const SIGNATURE_VERSION_HEADER = 'X-Glopal-Signature-Version';
    private const DEFAULT_SIGNATURE_VERSION = 3;

    /** @var Configuration */
    private $configuration;

    /**
     * Default error message.
     *
     * @var string
     */
    private $message = 'Incorrect signature.';

    /**
     * @param Configuration $configuration
     */
    public function __construct(Configuration $configuration)
    {
        $this->configuration = $configuration;
    }

    /**
     * Is signature allowed?
     *
     * @param RequestInterface $request
     * @return bool
     */
    public function isAllowed(RequestInterface $request): bool
    {
        $signatureVersion = $request->getHeader(self::SIGNATURE_VERSION_HEADER);

        if (!$signatureVersion && (int) $signatureVersion !== self::DEFAULT_SIGNATURE_VERSION) {
            $this->message = 'Not supported signature version.';

            return false;
        }

        $requestId = $request->getHeader(self::REQUEST_ID_HEADER);
        $receivedSignature = $request->getHeader(self::SIGNATURE_HEADER);

        if (!$requestId || !$receivedSignature) {
            $this->message = 'Missing signature.';

            return false;
        }

        return $this->isHmacSignatureValid($receivedSignature, $request);
    }

    /**
     * Retrieve error message.
     *
     * @return string
     */
    public function getMessage(): string
    {
        return $this->message;
    }

    /**
     *  Validate Hmac signature.
     *
     * @param string $receivedHmacSignature
     * @param RequestInterface $request
     * @return bool
     */
    private function isHmacSignatureValid(string $receivedHmacSignature, RequestInterface $request): bool
    {
        $requestId = $request->getHeader(self::REQUEST_ID_HEADER);

        //TODO if signature fail we can try again recalculation a normalized json
        // E.g. $this->normalizeJson($request->getContent())
        $normalizedJson = $request->getContent();
        $methodName = $request->getMethod();
        $fullActionName = $request->getFullActionName('/');

        $expectedHmacSignature = hash_hmac(
            'sha256',
            $requestId . $normalizedJson . $methodName . $fullActionName,
            $this->configuration->getGeneralValue('key')
        );

        return hash_equals($receivedHmacSignature, $expectedHmacSignature);
    }

    /**
     * Normalize json body
     *
     * @param string $jsonBody
     * @return string
     *
     * @SuppressWarnings(PHPMD.UnusedPrivateMethod)
     */
    private function normalizeJson(string $jsonBody): string
    {
        $decodedJson = json_decode($jsonBody, true);
        $sortedJson = $this->sortArrayKeysRecursively($decodedJson);

        return json_encode($sortedJson, JSON_UNESCAPED_SLASHES | JSON_UNESCAPED_UNICODE);
    }

    /**
     * Sorting array keys, recursively.
     *
     * @param array|string $data
     * @return array|string
     */
    private function sortArrayKeysRecursively($data)
    {
        if (!is_array($data)) {
            return $data;
        }

        ksort($data);

        foreach ($data as $key => $value) {
            $data[$key] = $this->sortArrayKeysRecursively($value);
        }

        return $data;
    }
}
