<?php

namespace Glopal\Connector\Model\Order;

use Glopal\Connector\Api\CreateRequestDTOInterface;

/**
 * @SuppressWarnings(PHPMD.TooManyFields)
 */
class CreateRequestDTO implements CreateRequestDTOInterface
{
    /** @var int */
    private $storeId;

    /** @var string */
    private $createdAt;

    /**@+
     * Customer data variables
     */

    /** @var string */
    private $customerFirstName;
    /** @var string */
    private $customerLastName;
    /** @var string */
    private $customerEmail;
    /** @var string */
    private $customerEmailAlias;

    /**@-*/

    /**@+
     * Addresses
     */

    /** @var array */
    private $billingAddress;
    /** @var array */
    private $shippingAddress;
    /** @var array */
    private $hubAddress;

    /**@-*/

    /** @var string */
    private $currency;

    /** @var string */
    private $exchangeCurrency;

    /** @var float */
    private $exchangeRate;

    /** @var array */
    private $summary;

    /** @var array */
    private $totalCost;

    /** @var array */
    private $shipping;

    /** @var array */
    private $items;

    /** @var array */
    private $discounts;

    /** @var string */
    private $internalNote;

    /** @var string */
    private $customerNote;

    /** @var bool */
    private $sendOriginNotifications;

    /**
     * @param array $data
     */
    public function __construct(array $data = [])
    {
        $this->init($data);
    }

    /**
     * @inheritdoc
     */
    public function isSendOriginNotifications(): bool
    {
        return $this->sendOriginNotifications;
    }

    /**
     * @inheritdoc
     */
    public function getStoreId(): int
    {
        return $this->storeId;
    }

    /**
     * @inheritdoc
     */
    public function getCreatedAt(): string
    {
        return $this->createdAt;
    }

    /**
     * @inheritdoc
     */
    public function getCustomerFirstName(): string
    {
        return $this->customerFirstName;
    }

    /**
     * @inheritdoc
     */
    public function getCustomerLastName(): string
    {
        return $this->customerLastName;
    }

    /**
     * @inheritdoc
     */
    public function getCustomerEmail(): string
    {
        return $this->customerEmail;
    }

    /**
     * @inheritdoc
     */
    public function getCustomerEmailAlias(): ?string
    {
        return $this->customerEmailAlias;
    }

    /**
     * @inheritdoc
     */
    public function getBillingAddress(): array
    {
        return $this->billingAddress;
    }

    /**
     * @inheritdoc
     */
    public function getShippingAddress(): array
    {
        return $this->shippingAddress;
    }

    /**
     * @inheritdoc
     */
    public function getHubAddress(): ?array
    {
        return $this->hubAddress;
    }

    /**
     * @inheritdoc
     */
    public function getCurrency(): string
    {
        return $this->currency;
    }

    /**
     * @inheritdoc
     */
    public function isDiffCurrency(): bool
    {
        return isset($this->exchangeCurrency);
    }

    /**
     * @inheritdoc
     */
    public function getExchangeCurrency(): string
    {
        return $this->exchangeCurrency ?? '';
    }

    /**
     * @inheritdoc
     */
    public function getExchangeRate(): float
    {
        return $this->exchangeRate ?? 1.0;
    }

    /**
     * @inheritdoc
     */
    public function getSummary(): array
    {
        return $this->summary;
    }

    /**
     * @inheritdoc
     */
    public function getTotalCost(): array
    {
        return $this->totalCost;
    }

    /**
     * @inheritdoc
     */
    public function isTotalCostOrder(): bool
    {
        return !empty($this->totalCost);
    }

    /**
     * @inheritdoc
     */
    public function getShipping(): array
    {
        return $this->shipping;
    }

    /**
     * @inheritdoc
     */
    public function isParcelForwarding(): bool
    {
        return !empty($this->hubAddress);
    }

    /**
     * @inheritdoc
     */
    public function getItems(): array
    {
        return $this->items;
    }

    /**
     * @inheritdoc
     */
    public function getDiscounts(): array
    {
        return $this->discounts;
    }

    /**
     * @inheritdoc
     */
    public function getInternalNote(): ?string
    {
        return $this->internalNote;
    }

    /**
     * @inheritdoc
     */
    public function getCustomerNoteNote(): ?string
    {
        return $this->customerNote;
    }

    /**
     * @inheritdoc
     */
    public function getMagentoTotals(): array
    {
        $summaryItems = $this->getSummary()['items'];
        $summaryDiscount = $this->getSummary()['discounts'];
        $summaryTotal = $this->getSummary()['total'];
        $summaryShipping = $this->getSummary()['shipping'];
        $exchangeRate = $this->getExchangeRate();

        $summaryItemsAmountNet = $summaryItems['amount_net'];
        $summaryItemsTaxOnAmount = $summaryItems['tax_on_amount'];
        $taxTotal = $summaryTotal['tax_on_amount'] + ($summaryTotal['tax_on_duty'] ?? 0.0);

        $shippingAmountNet = $summaryShipping['amount_net'];
        $shippingTaxOnAmount = $summaryShipping['tax_on_amount'];

        return [
            'subtotal' => $summaryItemsAmountNet,
            'base_subtotal' => $summaryItemsAmountNet / $exchangeRate,
            'subtotal_incl_tax' => $summaryItemsAmountNet + $summaryItemsTaxOnAmount,
            'base_subtotal_incl_tax' => ($summaryItemsAmountNet + $summaryItemsTaxOnAmount) / $exchangeRate,
            'tax_amount' => $taxTotal,
            'base_tax_amount' => $taxTotal / $exchangeRate,
            'discount_amount' => -1 * ($summaryDiscount['amount_net'] + $summaryDiscount['tax_on_amount']),
            'base_discount_amount' => -1
                * ($summaryDiscount['amount_net'] + $summaryDiscount['tax_on_amount'])
                / $exchangeRate,
            'grand_total' => $summaryTotal['total'],
            'base_grand_total' => $summaryTotal['total'] / $exchangeRate,
            'shipping_amount' => $shippingAmountNet,
            'base_shipping_amount' => $shippingAmountNet / $exchangeRate,
            'shipping_tax_amount' => $shippingTaxOnAmount,
            'base_shipping_tax_amount' => $shippingTaxOnAmount / $exchangeRate,
            'shipping_incl_tax' => $shippingAmountNet + $shippingTaxOnAmount,
            'base_shipping_incl_tax' => ($shippingAmountNet + $shippingTaxOnAmount) / $exchangeRate
        ];
    }

    /**
     * Init data for Data Transfer Object.
     *
     * @param array $requestParams
     *
     * @return void
     */
    private function init(array $requestParams): void
    {
        $this->storeId = (int) $requestParams['attributes']['magento_store_view_id'];
        $this->sendOriginNotifications = (bool) $requestParams['attributes']['send_origin_notifications'];
        $this->createdAt = date('Y-m-d H:i:s', strtotime($requestParams['created_at']));

        $customerData = $requestParams['customer'];
        $this->customerFirstName = $customerData['first_name'];
        $this->customerLastName = $customerData['last_name'];
        $this->customerEmail = $customerData['email'];
        $this->customerEmailAlias = $customerData['email_alias'];
        unset($customerData);

        $this->billingAddress = $requestParams['billing_address'];
        $this->shippingAddress = $requestParams['shipping_address'];
        $this->hubAddress = null;

        $this->currency = $requestParams['currency'];

        if ($requestParams['exchange_rates']) {
            $this->exchangeCurrency = array_keys($requestParams['exchange_rates'])[0];
            $this->exchangeRate = $requestParams['exchange_rates'][$this->exchangeCurrency]['rate'];
        }

        $this->totalCost = $requestParams['total_cost'] ?? null;
        $this->summary = $requestParams['summary'];

        $this->shipping = $requestParams['shipping'];
        if (isset($this->shipping['parcel_forwarding'])) {
            $this->hubAddress = $this->shipping['parcel_forwarding']['hub_address'];
        }

        $itemsData = $requestParams['items'];
        foreach ($itemsData as $itemsDatum) {
            $this->items[$itemsDatum['sku']] = [
                'product_id' => $itemsDatum['product_id'],
                'sku' => $itemsDatum['sku'],
                'variant_id' => $itemsDatum['variant_id'] ?? null,
                'quantity' => $itemsDatum['quantity'],
                'options' => $itemsDatum['options'],
                'unit_price' => $itemsDatum['unit_price'],
                'line_price' => $itemsDatum['line_price'],
                'line_discounts' => $itemsDatum['allocations']['discounts']['total']['line_charge'] ?? null,
                'tax_rate' => $itemsDatum['tax_rate'],
                'duty_rate' => $itemsDatum['total_cost']['duty_rate'] ?? null,
            ];
        }

        unset($itemsData);

        $this->discounts = $requestParams['discounts'] ?? [];

        // Glopal's note for the merchant
        $this->internalNote = $requestParams['comment_note'] ?? null;

        // Gift wrapping message
        $this->customerNote = $requestParams['customer_note'] ?? null;
    }
}
