<?php

namespace Glopal\Connector\Model\Order;

use Glopal\Connector\Api\Data\OrderAttributeInterface;
use Glopal\Connector\Api\Data\OrderItemTotalCostInterface;
use Glopal\Connector\Api\Data\OrderTotalCostInterface;
use Glopal\Connector\Api\OrderAttributeRepositoryInterface;
use Glopal\Connector\Api\OrderItemTotalCostRepositoryInterface;
use Glopal\Connector\Api\OrderTotalCostRepositoryInterface;
use Magento\Framework\Api\SearchCriteriaBuilder;
use Magento\Framework\Exception\CouldNotSaveException;
use Magento\Framework\Exception\NoSuchEntityException;
use Magento\Sales\Api\Data\OrderInterface;
use Magento\Sales\Api\Data\OrderItemInterface;

class Management
{
    /** @var OrderAttributeRepositoryInterface */
    private $orderAttributeRepository;

    /** @var OrderTotalCostRepositoryInterface */
    private $orderTotalCostRepository;

    /** @var OrderItemTotalCostRepositoryInterface */
    private $itemTotalCostRepository;

    /** @var SearchCriteriaBuilder */
    private $searchCriteriaBuilder;

    /**
     * @param OrderAttributeRepositoryInterface $orderAttributeRepository
     * @param OrderTotalCostRepositoryInterface $orderTotalCostRepository
     * @param OrderItemTotalCostRepositoryInterface $itemTotalCostRepository
     * @param SearchCriteriaBuilder $searchCriteriaBuilder
     */
    public function __construct(
        OrderAttributeRepositoryInterface $orderAttributeRepository,
        OrderTotalCostRepositoryInterface $orderTotalCostRepository,
        OrderItemTotalCostRepositoryInterface $itemTotalCostRepository,
        SearchCriteriaBuilder $searchCriteriaBuilder
    ) {
        $this->orderAttributeRepository = $orderAttributeRepository;
        $this->orderTotalCostRepository = $orderTotalCostRepository;
        $this->itemTotalCostRepository = $itemTotalCostRepository;
        $this->searchCriteriaBuilder = $searchCriteriaBuilder;
    }

    /**
     * Retrieve order attribute.
     *
     * @param OrderInterface $order
     * @return OrderAttributeInterface|null
     */
    public function getOrderAttribute(OrderInterface $order): ?OrderAttributeInterface
    {
        if (!$order->getId()) {
            return null;
        }

        return $this->orderAttributeRepository->getByOrderId($order->getId());
    }

    /**
     * Adds order attribute.
     *
     * @param OrderInterface $order
     * @param array $attributeData
     * @return OrderAttributeInterface|null
     * @throws CouldNotSaveException
     *
     * @SuppressWarnings(PHPMD.CyclomaticComplexity)
     * @SuppressWarnings(PHPMD.NPathComplexity)
     */
    public function addOrderAttribute(OrderInterface $order, array $attributeData): ?OrderAttributeInterface
    {
        if (!$order->getId()) {
            return null;
        }

        $orderAttribute = $this->orderAttributeRepository->getEmpty();
        $orderAttribute->setOrderId($order->getId());

        if (isset($attributeData[OrderAttributeInterface::EXCHANGE_RATE])) {
            $orderAttribute->setExchangeRate($attributeData[OrderAttributeInterface::EXCHANGE_RATE]);
        }

        if (isset($attributeData[OrderAttributeInterface::FROM_CURRENCY])) {
            $orderAttribute->setFromCurrency($attributeData[OrderAttributeInterface::FROM_CURRENCY]);
        }

        if (isset($attributeData[OrderAttributeInterface::TO_CURRENCY])) {
            $orderAttribute->setToCurrency($attributeData[OrderAttributeInterface::TO_CURRENCY]);
        }

        if (isset($attributeData[OrderAttributeInterface::CUSTOMER_ORIGIN_EMAIL])) {
            $orderAttribute->setCustomerOriginEmail($attributeData[OrderAttributeInterface::CUSTOMER_ORIGIN_EMAIL]);
        }

        if (isset($attributeData[OrderAttributeInterface::CUSTOMER_SHIPPING_ADDRESS])) {
            $orderAttribute->setCustomerShippingAddress(
                $attributeData[OrderAttributeInterface::CUSTOMER_SHIPPING_ADDRESS]
            );
        }

        if (isset($attributeData[OrderAttributeInterface::TRANSACTION_ID])) {
            $orderAttribute->setTransactionId($attributeData[OrderAttributeInterface::TRANSACTION_ID]);
        }

        if (isset($attributeData[OrderAttributeInterface::BASE_EXTRA_AMOUNT_NET])) {
            $orderAttribute->setBaseExtraAmount($attributeData[OrderAttributeInterface::BASE_EXTRA_AMOUNT_NET]);
        }

        if (isset($attributeData[OrderAttributeInterface::EXTRA_AMOUNT_NET])) {
            $orderAttribute->setExtraAmount($attributeData[OrderAttributeInterface::EXTRA_AMOUNT_NET]);
        }

        if (isset($attributeData[OrderAttributeInterface::BASE_EXTRA_TAX_ON_AMOUNT])) {
            $orderAttribute->setBaseExtraTaxOnAmount($attributeData[OrderAttributeInterface::BASE_EXTRA_TAX_ON_AMOUNT]);
        }

        if (isset($attributeData[OrderAttributeInterface::EXTRA_TAX_ON_AMOUNT])) {
            $orderAttribute->setExtraTaxOnAmount($attributeData[OrderAttributeInterface::EXTRA_TAX_ON_AMOUNT]);
        }

        return $this->orderAttributeRepository->save($orderAttribute);
    }

    /**
     * Retrieve order total cost.
     *
     * @param OrderInterface $order
     * @return OrderTotalCostInterface|null
     * @throws NoSuchEntityException
     */
    public function getTotalCost(OrderInterface $order): ?OrderTotalCostInterface
    {
        if (!$order->getId()) {
            return null;
        }

        return $this->orderTotalCostRepository->getByOrderId($order->getId());
    }

    /**
     * Adds order total cost.
     *
     * @param OrderInterface $order
     * @param array $totalCostData
     * @return OrderTotalCostInterface|null
     * @throws CouldNotSaveException
     *
     * @SuppressWarnings(PHPMD.CyclomaticComplexity)
     * @SuppressWarnings(PHPMD.NPathComplexity)
     */
    public function addTotalCost(OrderInterface $order, array $totalCostData): ?OrderTotalCostInterface
    {
        if (!$order->getId()) {
            return null;
        }

        $totalCost = $this->orderTotalCostRepository->getEmpty();
        $totalCost->setOrderId($order->getId());

        if (isset($totalCostData[OrderTotalCostInterface::BASE_TOTAL_DUTY_AMOUNT])) {
            $totalCost->setBaseTotalDutyAmount($totalCostData[OrderTotalCostInterface::BASE_TOTAL_DUTY_AMOUNT]);
        }

        if (isset($totalCostData[OrderTotalCostInterface::TOTAL_DUTY_AMOUNT])) {
            $totalCost->setTotalDutyAmount($totalCostData[OrderTotalCostInterface::TOTAL_DUTY_AMOUNT]);
        }

        if (isset($totalCostData[OrderTotalCostInterface::BASE_TOTAL_TAX_ON_DUTY])) {
            $totalCost->setBaseTotalTaxOnDutyAmount($totalCostData[OrderTotalCostInterface::BASE_TOTAL_TAX_ON_DUTY]);
        }

        if (isset($totalCostData[OrderTotalCostInterface::TOTAL_TAX_ON_DUTY])) {
            $totalCost->setTotalTaxOnDutyAmount($totalCostData[OrderTotalCostInterface::TOTAL_TAX_ON_DUTY]);
        }

        if (isset($totalCostData[OrderTotalCostInterface::BASE_ITEMS_DUTY_AMOUNT])) {
            $totalCost->setBaseItemsDutyAmount($totalCostData[OrderTotalCostInterface::BASE_ITEMS_DUTY_AMOUNT]);
        }

        if (isset($totalCostData[OrderTotalCostInterface::ITEMS_DUTY_AMOUNT])) {
            $totalCost->setItemsDutyAmount($totalCostData[OrderTotalCostInterface::ITEMS_DUTY_AMOUNT]);
        }

        if (isset($totalCostData[OrderTotalCostInterface::BASE_ITEMS_TAX_ON_DUTY])) {
            $totalCost->setBaseItemsTaxOnDutyAmount($totalCostData[OrderTotalCostInterface::BASE_ITEMS_TAX_ON_DUTY]);
        }

        if (isset($totalCostData[OrderTotalCostInterface::ITEMS_TAX_ON_DUTY])) {
            $totalCost->setItemsTaxOnDutyAmount($totalCostData[OrderTotalCostInterface::ITEMS_TAX_ON_DUTY]);
        }

        if (isset($totalCostData[OrderTotalCostInterface::BASE_DISCOUNT_DUTY_AMOUNT])) {
            $totalCost->setBaseDiscountDutyAmount($totalCostData[OrderTotalCostInterface::BASE_DISCOUNT_DUTY_AMOUNT]);
        }

        if (isset($totalCostData[OrderTotalCostInterface::DISCOUNT_DUTY_AMOUNT])) {
            $totalCost->setDiscountDutyAmount($totalCostData[OrderTotalCostInterface::DISCOUNT_DUTY_AMOUNT]);
        }

        if (isset($totalCostData[OrderTotalCostInterface::BASE_DISCOUNT_TAX_ON_DUTY])) {
            $totalCost->setBaseDiscountTaxOnDutyAmount(
                $totalCostData[OrderTotalCostInterface::BASE_DISCOUNT_TAX_ON_DUTY]
            );
        }

        if (isset($totalCostData[OrderTotalCostInterface::DISCOUNT_TAX_ON_DUTY])) {
            $totalCost->setDiscountTaxOnDutyAmount($totalCostData[OrderTotalCostInterface::DISCOUNT_TAX_ON_DUTY]);
        }

        if (isset($totalCostData[OrderTotalCostInterface::BASE_SHIPPING_DUTY_AMOUNT])) {
            $totalCost->setBaseShippingDutyAmount($totalCostData[OrderTotalCostInterface::BASE_SHIPPING_DUTY_AMOUNT]);
        }

        if (isset($totalCostData[OrderTotalCostInterface::SHIPPING_DUTY_AMOUNT])) {
            $totalCost->setShippingDutyAmount($totalCostData[OrderTotalCostInterface::SHIPPING_DUTY_AMOUNT]);
        }

        if (isset($totalCostData[OrderTotalCostInterface::BASE_SHIPPING_TAX_ON_DUTY])) {
            $totalCost->setBaseShippingTaxOnDutyAmount(
                $totalCostData[OrderTotalCostInterface::BASE_SHIPPING_TAX_ON_DUTY]
            );
        }

        if (isset($totalCostData[OrderTotalCostInterface::SHIPPING_TAX_ON_DUTY])) {
            $totalCost->setShippingTaxOnDutyAmount($totalCostData[OrderTotalCostInterface::SHIPPING_TAX_ON_DUTY]);
        }

        if (isset($totalCostData[OrderTotalCostInterface::BASE_EXTRA_DUTY_AMOUNT])) {
            $totalCost->setBaseExtraDutyAmount($totalCostData[OrderTotalCostInterface::BASE_EXTRA_DUTY_AMOUNT]);
        }

        if (isset($totalCostData[OrderTotalCostInterface::EXTRA_DUTY_AMOUNT])) {
            $totalCost->setExtraDutyAmount($totalCostData[OrderTotalCostInterface::EXTRA_DUTY_AMOUNT]);
        }

        if (isset($totalCostData[OrderTotalCostInterface::BASE_EXTRA_TAX_ON_DUTY])) {
            $totalCost->setBaseExtraTaxOnDutyAmount(
                $totalCostData[OrderTotalCostInterface::BASE_EXTRA_TAX_ON_DUTY]
            );
        }

        if (isset($totalCostData[OrderTotalCostInterface::EXTRA_TAX_ON_DUTY])) {
            $totalCost->setExtraTaxOnDutyAmount($totalCostData[OrderTotalCostInterface::EXTRA_TAX_ON_DUTY]);
        }

        return $this->orderTotalCostRepository->save($totalCost);
    }

    /**
     * Retrieve items total cost.
     *
     * @param int $orderTccId
     *
     * @return OrderItemTotalCostInterface[]
     */
    public function getItemsTotalCost(int $orderTccId): array
    {
        return $this->itemTotalCostRepository->getList(
            $this->searchCriteriaBuilder->addFilter('order_tcc_id', $orderTccId)->create()
        )->getItems();
    }

    /**
     * Retrieve items total cost by order item id.
     *
     * @param int $orderItemId
     *
     * @return null|OrderItemTotalCostInterface
     */
    public function getItemsTotalCostByOrderItemId(int $orderItemId): ?OrderItemTotalCostInterface
    {
        $itemsTotalCost = $this->itemTotalCostRepository->getList(
            $this->searchCriteriaBuilder->addFilter('order_item_id', $orderItemId)->create()
        )->getItems();

        if (!count($itemsTotalCost)) {
            return null;
        }

        return reset($itemsTotalCost);
    }

    /**
     * Adds item total cost.
     *
     * @param OrderItemInterface $orderItem
     * @param array $itemTotalCostData
     * @return OrderItemTotalCostInterface|null
     * @throws CouldNotSaveException
     */
    public function addItemTotalCost(
        OrderItemInterface $orderItem,
        array $itemTotalCostData
    ): ?OrderItemTotalCostInterface {
        if (!$orderItem->getId()) {
            return null;
        }

        $itemTotalCost = $this->itemTotalCostRepository->getEmpty();
        $itemTotalCost->setOrderItemId($orderItem->getId());

        if (isset($itemTotalCostData[OrderItemTotalCostInterface::ORDER_TCC_ID])) {
            $itemTotalCost->setOrderTccId($itemTotalCostData[OrderItemTotalCostInterface::ORDER_TCC_ID]);
        }

        if (isset($itemTotalCostData[OrderItemTotalCostInterface::BASE_DUTY_AMOUNT])) {
            $itemTotalCost->setBaseDutyAmount($itemTotalCostData[OrderItemTotalCostInterface::BASE_DUTY_AMOUNT]);
        }

        if (isset($itemTotalCostData[OrderItemTotalCostInterface::DUTY_AMOUNT])) {
            $itemTotalCost->setDutyAmount($itemTotalCostData[OrderItemTotalCostInterface::DUTY_AMOUNT]);
        }

        if (isset($itemTotalCostData[OrderItemTotalCostInterface::BASE_TAX_ON_DUTY])) {
            $itemTotalCost->setBaseTaxOnDutyAmount($itemTotalCostData[OrderItemTotalCostInterface::BASE_TAX_ON_DUTY]);
        }

        if (isset($itemTotalCostData[OrderItemTotalCostInterface::TAX_ON_DUTY])) {
            $itemTotalCost->setTaxOnDutyAmount($itemTotalCostData[OrderItemTotalCostInterface::TAX_ON_DUTY]);
        }

        if (isset($itemTotalCostData[OrderItemTotalCostInterface::DUTY_PERCENT])) {
            $itemTotalCost->setDutyPercent($itemTotalCostData[OrderItemTotalCostInterface::DUTY_PERCENT]);
        }

        return $this->itemTotalCostRepository->save($itemTotalCost);
    }
}
