<?php

namespace Glopal\Connector\Model\Price;

use Glopal\Connector\Api\Data\Price\QuoteItemDataInterface;
use Glopal\Connector\Api\Data\Price\QuoteItemDataInterfaceFactory;

class Calculator
{
    /** @var QuoteItemDataInterfaceFactory */
    private $priceQuoteItemFactory;

    /** @var bool */
    private $priceIncludeTax;

    /** @var bool */
    private $applyTaxAfterDiscount;

    /** @var float */
    private $exchangeRate;

    /**
     * @param QuoteItemDataInterfaceFactory $priceQuoteItemFactory
     * @param bool $priceIncludeTax
     * @param bool $applyTaxAfterDiscount
     */
    public function __construct(
        QuoteItemDataInterfaceFactory $priceQuoteItemFactory,
        bool $priceIncludeTax,
        bool $applyTaxAfterDiscount
    ) {
        $this->priceQuoteItemFactory = $priceQuoteItemFactory;

        $this->priceIncludeTax = $priceIncludeTax;
        $this->applyTaxAfterDiscount = $applyTaxAfterDiscount;
    }

    /**
     * Calculate quote item price.
     *
     * @param array $itemPriceData
     * @return QuoteItemDataInterface
     */
    public function calculateQuoteItemPrice(array $itemPriceData): QuoteItemDataInterface
    {
        $this->exchangeRate = $itemPriceData['exchange_rate'] ?? 1.0;

        $itemAmountNet = $itemPriceData['unit_price']['amount_net'];
        $itemTaxOnAmountNet = $itemPriceData['unit_price']['tax_on_amount'];

        $lineAmountNet = $itemPriceData['line_price']['amount_net'];
        $lineDutyOnAmount = $itemPriceData['line_price']['duty_on_amount'] ?? 0.00;
        $lineTaxOnAmount = $itemPriceData['line_price']['tax_on_amount'];
        $lineTaxOnDuty = $itemPriceData['line_price']['tax_on_duty'] ?? 0.0;
        $lineDiscount = $itemPriceData['discounts'] ?? [];

        /** @var QuoteItemDataInterface $quoteItemPrice */
        $quoteItemPrice = $this->priceQuoteItemFactory->create()
            ->setOriginalPrice($itemAmountNet)
            ->setBaseOriginalPrice($this->getBasePrice($itemAmountNet))
            ->setPrice($itemAmountNet)
            ->setBasePrice($this->getBasePrice($itemAmountNet))
            ->setPriceInclTax($itemAmountNet + $itemTaxOnAmountNet)
            ->setBasePriceInclTax($this->getBasePrice($itemAmountNet + $itemTaxOnAmountNet))
            ->setTaxBeforeDiscount(0.0)
            ->setBaseTaxBeforeDiscount(0.0);

        $discountAmount = $lineDiscount['total'] ?? 0.0;
        $discountTaxCompensationAmount = 0.0;

        if ($this->applyTaxAfterDiscount) {
            $taxAmount = $lineTaxOnAmount
                + $lineTaxOnDuty
                - ($lineDiscount['tax_on_amount'] ?? 0.0)
                - ($lineDiscount['tax_on_duty'] ?? 0.0);
            $quoteItemPrice->setTaxAmount($taxAmount);
            $quoteItemPrice->setBaseTaxAmount($this->getBasePrice($taxAmount));

            if ($this->priceIncludeTax) {
                $discountTaxCompensationAmount = ($lineDiscount['tax_on_amount'] ?? 0.0)
                    + ($lineDiscount['tax_on_duty'] ?? 0.0);
            }

            if (!$this->priceIncludeTax) {
                $discountAmount = ($lineDiscount['amount_net'] ?? 0.0) + ($lineDiscount['duty_on_amount'] ?? 0.00);
            }
        } else {
            $quoteItemPrice->setTaxAmount($lineTaxOnAmount + $lineDutyOnAmount)
                ->setBaseTaxAmount($this->getBasePrice($lineTaxOnAmount + $lineDutyOnAmount))
                ->setTaxBeforeDiscount($lineTaxOnAmount + $lineTaxOnDuty)
                ->setBaseTaxBeforeDiscount($this->getBasePrice($lineTaxOnAmount + $lineTaxOnDuty));
        }

        $quoteItemPrice->setDiscountAmount($discountAmount);
        $quoteItemPrice->setBaseDiscountAmount($this->getBasePrice($discountAmount));
        $quoteItemPrice->setDiscountTaxCompensationAmount($discountTaxCompensationAmount);
        $quoteItemPrice->setBaseDiscountTaxCompensationAmount($this->getBasePrice($discountTaxCompensationAmount));

        $quoteItemPrice->setRowTotal($lineAmountNet);
        $quoteItemPrice->setBaseRowTotal($this->getBasePrice($lineAmountNet));

        $quoteItemPrice->setRowTotalInclTax($lineAmountNet + $lineTaxOnAmount);
        $quoteItemPrice->setBaseRowTotalInclTax($this->getBasePrice($lineAmountNet + $lineTaxOnAmount));
        $quoteItemPrice->setRowTotalWithDiscount($lineAmountNet - $discountAmount);

        return $quoteItemPrice;
    }

    /**
     * Returns base price (price divided by exchange rate).
     *
     * @param float $priceValue
     * @return float
     */
    private function getBasePrice(float $priceValue): float
    {
        return $priceValue / $this->exchangeRate;
    }
}
