<?php

namespace Glopal\Connector\Processor\CartTransfer;

use Glopal\Connector\Provider\CartTransfer\CombineProductDataProvider;
use Glopal\Connector\Provider\CartTransfer\OrderOptionsProvider;
use Glopal\Connector\Provider\CartTransfer\SmallImageProvider;
use Glopal\Connector\Provider\CartTransfer\SpecificProductAttributesProvider;
use Magento\Catalog\Api\Data\ProductInterface;
use Magento\Catalog\Api\ProductRepositoryInterface;
use Magento\Framework\App\RequestInterface;
use Magento\Quote\Api\Data\CartInterface;
use Magento\Quote\Api\Data\CartItemInterface;

class QuoteItems implements CartTransferProcessorInterface
{
    private const KEY = 'wi_items';

    /** @var ProductRepositoryInterface */
    private $productRepository;

    /** @var RequestInterface */
    private $request;

    /** @var CombineProductDataProvider */
    private $combineProductDataProvider;

    /** @var SmallImageProvider */
    private $smallImageProvider;

    /** @var SpecificProductAttributesProvider */
    private $specificProductAttributesProvider;

    /** @var OrderOptionsProvider */
    private $orderOptionsProvider;

    /**
     * @param ProductRepositoryInterface $productRepository
     * @param RequestInterface $request
     * @param CombineProductDataProvider $combineProductDataProvider
     * @param SmallImageProvider $smallImageProvider
     * @param SpecificProductAttributesProvider $specificProductAttributesProvider
     * @param OrderOptionsProvider $orderOptionsProvider
     */
    public function __construct(
        ProductRepositoryInterface $productRepository,
        RequestInterface $request,
        CombineProductDataProvider $combineProductDataProvider,
        SmallImageProvider $smallImageProvider,
        SpecificProductAttributesProvider $specificProductAttributesProvider,
        OrderOptionsProvider $orderOptionsProvider
    ) {
        $this->productRepository = $productRepository;
        $this->request = $request;
        $this->combineProductDataProvider = $combineProductDataProvider;
        $this->smallImageProvider = $smallImageProvider;
        $this->specificProductAttributesProvider = $specificProductAttributesProvider;
        $this->orderOptionsProvider = $orderOptionsProvider;
    }

    /**
     * @inheritdoc
     */
    public function process(CartInterface $quote, array &$data): void
    {
        $result = [];
        $storeId = $quote->getStoreId();

        /** @var CartItemInterface $item */
        foreach ($quote->getAllVisibleItems() as $item) {
            /** @var ProductInterface $modelProduct */
            $modelProduct = $this->productRepository->getById($item->getProductId(), false, $storeId);

            $quoteItemData = $item->getData();
            $quoteItemData['product'] = []; // block object

            /** @var ProductInterface $modelProduct */
            $itemProduct = $item->getProduct();
            $quoteItemData['wi_product'] = $this->prepareProductData($modelProduct, $itemProduct, $storeId);

            if ($this->isDebugAvailable()) {
                $quoteItemData['wi_product_model'] = $modelProduct->getData();
                $quoteItemData['wi_product_item'] = $itemProduct->getData();
            }

            // Variant is simple product that belongs to configurable
            $simpleProduct = null;
            $simpleOption = $item->getOptionByCode('simple_product');
            if (null != $simpleOption) {
                $simpleProduct = $simpleOption->getProduct();
            }

            $itemProductTypeInstance = $itemProduct->getTypeInstance(true);
            $quoteItemData['wi_order_options'] = $this->orderOptionsProvider->get(
                $itemProductTypeInstance->getOrderOptions($itemProduct),
                $simpleProduct
            );
            $quoteItemData['wi_buy_request'] = $item->getBuyRequest()->getData();

            $result[] = $quoteItemData;
        }

        $data[self::KEY] = $result;
    }

    /**
     * Check is debug available.
     *
     * @return bool
     */
    private function isDebugAvailable(): bool
    {
        return (bool)$this->request->getParam('debug');
    }

    /**
     * Combine product data, small_image and specific attributes.
     *
     * @param ProductInterface $modelProduct
     * @param ProductInterface $itemProduct
     * @param null|string|int $storeId
     *
     * @return array
     */
    private function prepareProductData(ProductInterface $modelProduct, ProductInterface $itemProduct, $storeId): array
    {
        $wiProductData = $this->combineProductDataProvider->get($modelProduct, $itemProduct);
        $smallImageUrl = $this->smallImageProvider->get($wiProductData['small_image'] ?? '');
        if (null != $smallImageUrl) {
            $wiProductData['small_image'] = $smallImageUrl;
        }

        return array_merge($wiProductData, $this->specificProductAttributesProvider->get($modelProduct, $storeId));
    }
}
