<?php

namespace Glopal\Connector\Processor\Order;

use Glopal\Connector\Api\CreateRequestDTOInterface;
use Glopal\Connector\Model\Price\Calculator;
use Glopal\Connector\Provider\PriceCalculatorProvider;
use Magento\Catalog\Api\ProductRepositoryInterface;
use Magento\Catalog\Model\Product as ModelProduct;
use Magento\Catalog\Model\ResourceModel\Product as ProductResource;
use Magento\ConfigurableProduct\Model\Product\Type\Configurable;
use Magento\Framework\DataObject;
use Magento\Quote\Api\Data\CartInterface;
use Magento\Quote\Model\Quote\Item;

class Product implements OrderCreateProcessorInterface
{
    /** @var ProductRepositoryInterface */
    private $productRepository;

    /** @var PriceCalculatorProvider */
    private $priceCalculatorProvider;

    /** @var ProductResource */
    private $productResource;

    /** @var Configurable */
    private $configurableType;

    /**
     * @param ProductRepositoryInterface $productRepository
     * @param PriceCalculatorProvider $priceCalculatorProvider
     * @param ProductResource $productResource
     * @param Configurable $configurableType
     */
    public function __construct(
        ProductRepositoryInterface $productRepository,
        PriceCalculatorProvider $priceCalculatorProvider,
        ProductResource $productResource,
        Configurable $configurableType
    ) {
        $this->productRepository = $productRepository;
        $this->priceCalculatorProvider = $priceCalculatorProvider;
        $this->productResource = $productResource;
        $this->configurableType = $configurableType;
    }

    /**
     * @inheritdoc
     */
    public function process(CartInterface $quote, CreateRequestDTOInterface $createRequestDTO): void
    {
        $itemsData = $createRequestDTO->getItems();
        $storeId = $quote->getStoreId();

        /** @var Calculator $priceCalculator */
        $priceCalculator = $this->priceCalculatorProvider->get($storeId);

        foreach ($itemsData as $itemDatum) {
            $productId = $itemDatum['product_id'];
            $variantId = $itemDatum['variant_id'] ?? null;
            $requestedQty = $itemDatum['quantity'];
            $buyRequest = new DataObject(['qty' => $requestedQty, 'product' => $productId]);
            /** @var ModelProduct $product */
            $product = $this->productRepository->getById($productId, false, $storeId);

            if (!empty($variantId)) {
                $attributesArray = (array)$this->configurableType->getConfigurableAttributesAsArray($product);

                $ids = array_map(function ($at) {
                    return $at['attribute_id'];
                }, $attributesArray);

                $ids = $this->productResource->getAttributeRawValue(
                    $variantId,
                    $ids,
                    $storeId
                );

                $superAttribute = [];
                foreach ($attributesArray as $attribute) {
                    $superAttribute[$attribute['attribute_id']] = $ids[$attribute['attribute_code']];
                }

                $buyRequest->setData('super_attribute', $superAttribute);
            }

            /**
             * Whe have to use __clone() with the original product
             * due to prevent adding issues with configurable and few simple products
             * @see \Magento\Quote\Model\Quote::getItemByProduct
             * @var Item $item
             */
            $item = $quote->addProduct(clone $product, $buyRequest);

            $priceModel = $priceCalculator->calculateQuoteItemPrice([
                'exchange_rate' => $createRequestDTO->getExchangeRate(),
                'unit_price' => $itemDatum['unit_price'],
                'line_price' => $itemDatum['line_price'],
                'discounts' => $itemDatum['line_discounts'] ?? [],
            ]);

            $item->setOriginalPrice($priceModel->getOriginalPrice());
            $item->setBaseOriginalPrice($priceModel->getBaseOriginalPrice());

            $item->setPrice($priceModel->getPrice());
            $item->setBasePrice($priceModel->getBasePrice());

            $item->setPriceInclTax($priceModel->getPriceInclTax());
            $item->setBasePriceInclTax($priceModel->getBasePriceInclTax());

            $item->setTaxPercent($itemDatum['tax_rate']);

            $item->setTaxBeforeDiscount($priceModel->getTaxBeforeDiscount());
            $item->setBaseTaxBeforeDiscount($priceModel->getBaseTaxBeforeDiscount());
            $item->setTaxAmount($priceModel->getTaxAmount());
            $item->setBaseTaxAmount($priceModel->getBaseTaxAmount());

            // DB Row total appears as subtotal in order view items table
            $item->setRowTotal($priceModel->getRowTotal());
            $item->setBaseRowTotal($priceModel->getBaseRowTotal());
            $item->setRowTotalInclTax($priceModel->getRowTotalInclTax());
            $item->setBaseRowTotalInclTax($priceModel->getBaseRowTotalInclTax());

            $item->setDiscountAmount($priceModel->getDiscountAmount());
            $item->setBaseDiscountAmount($priceModel->getBaseDiscountAmount());
            $item->setRowTotalWithDiscount($priceModel->getRowTotalWithDiscount());

            $item->setDiscountTaxCompensationAmount($priceModel->getDiscountTaxCompensationAmount());
            $item->setBaseDiscountTaxCompensationAmount($priceModel->getBaseDiscountTaxCompensationAmount());

            $item->setHiddenTaxAmount(0);
            $item->setBaseHiddenTaxAmount(0);
        }
    }
}
