<?php

namespace Glopal\Connector\Processor;

use Exception;
use Glopal\Connector\Api\CreateRequestDTOInterface;
use Glopal\Connector\Processor\AfterOrder\AfterOrderCreateProcessorInterface;
use Glopal\Connector\Processor\Order\OrderCreateProcessorInterface;
use InvalidArgumentException;
use Magento\Quote\Api\CartRepositoryInterface;
use Magento\Quote\Api\Data\CartInterface;
use Magento\Quote\Model\QuoteFactory;
use Magento\Quote\Model\QuoteManagement;
use Magento\Sales\Api\Data\OrderInterface;
use Magento\Sales\Api\OrderRepositoryInterface;
use Magento\Sales\Model\Order;
use Magento\Store\Model\StoreManagerInterface;
use Psr\Log\LoggerInterface;

/**
 * @SuppressWarnings(PHPMD.CouplingBetweenObjects)
 */
class OrderCreate
{
    /** @var QuoteFactory */
    private $quoteFactory;

    /** @var QuoteManagement */
    private $quoteManagement;

    /** @var OrderCreateProcessorInterface[] */
    private $prepareProcessors;

    /** @var OrderCreateProcessorInterface[] */
    private $afterProcessors;

    /** @var LoggerInterface */
    private $logger;

    /** @var StoreManagerInterface */
    private $storeManager;

    /** @var OrderRepositoryInterface */
    private $orderRepository;

    /** @var CartRepositoryInterface */
    private $cartRepository;

    /**
     * @param QuoteFactory $quoteFactory
     * @param QuoteManagement $quoteManagement
     * @param LoggerInterface $logger
     * @param StoreManagerInterface $storeManager
     * @param OrderRepositoryInterface $orderRepository
     * @param CartRepositoryInterface $cartRepository
     * @param array $prepareProcessors
     * @param array $afterProcessors
     */
    public function __construct(
        QuoteFactory $quoteFactory,
        QuoteManagement $quoteManagement,
        LoggerInterface $logger,
        StoreManagerInterface $storeManager,
        OrderRepositoryInterface $orderRepository,
        CartRepositoryInterface $cartRepository,
        array $prepareProcessors = [],
        array $afterProcessors = []
    ) {
        $this->quoteFactory = $quoteFactory;
        $this->quoteManagement = $quoteManagement;
        $this->logger = $logger;
        $this->storeManager = $storeManager;
        $this->orderRepository = $orderRepository;
        $this->cartRepository = $cartRepository;

        $this->prepareProcessors = $prepareProcessors;
        $this->afterProcessors = $afterProcessors;

        // Sorting an array based on name (from di.xml)
        ksort($this->prepareProcessors);
        ksort($this->afterProcessors);
    }

    /**
     * Order Create process
     *
     * @param CreateRequestDTOInterface $createRequestDTO
     * @return array
     */
    public function process(CreateRequestDTOInterface $createRequestDTO): array
    {
        $result = [
            'status' => true,
            'message' => ''
        ];

        try {
            /** @var CartInterface $quote */
            $quote = $this->quoteFactory->create();
            $storeId = $createRequestDTO->getStoreId();

            if ($storeId) {
                $this->storeManager->setCurrentStore($storeId);
                $quote->setStoreId($storeId);
            }

            // Set availability for "Glopal" payment method
            $quote->setForcePaymentAvailability(true);
            $quote->setCreatedAt($createRequestDTO->getCreatedAt());

            foreach ($this->prepareProcessors as $prepareProcessor) {
                if (!$prepareProcessor instanceof OrderCreateProcessorInterface) {
                    throw new InvalidArgumentException(
                        'Processor must implement ' . OrderCreateProcessorInterface::class
                    );
                }

                $prepareProcessor->process($quote, $createRequestDTO);
            }

            $quote->setInventoryProcessed(false);
            $this->cartRepository->save($quote);

            // Workaround for setting correct currency for quote and order.
            $this->setCurrency($createRequestDTO, $quote);

            // Create an Order From Quote
            $order = $this->quoteManagement->submit($quote, $this->getOrderData($quote, $createRequestDTO));

            foreach ($this->afterProcessors as $afterProcessorProcessor) {
                if (!$afterProcessorProcessor instanceof AfterOrderCreateProcessorInterface) {
                    throw new InvalidArgumentException(
                        'Processor must implement ' . AfterOrderCreateProcessorInterface::class
                    );
                }

                $afterProcessorProcessor->process($order, $createRequestDTO);
            }

            $order = $this->orderRepository->save($order);

            if (!$order->getEntityId()) {
                return [
                    'status' => false,
                    'message' => 'Something goes wrong, order is not created'
                ];
            }

            $result['message'] = [
                'order_id' => $order->getEntityId(),
                'increment_id' => $order->getRealOrderId()
            ];
            $invoicesData = $this->getInvoicesDataByOrder($order);

            if (!empty($invoicesData)) {
                $result['message']['invoices'] = $invoicesData;
            }
        } catch (Exception $e) {
            $result['status'] = false;
            $result['message'] = $e->getMessage();

            $this->logger->error($e->getMessage(), ['exception' => $e]);
        }

        return $result;
    }

    /**
     * Prepare correct order data for creation
     *
     * @param CartInterface $quote
     * @param CreateRequestDTOInterface $createRequestDTO
     *
     * @return array
     */
    private function getOrderData(CartInterface $quote, CreateRequestDTOInterface $createRequestDTO): array
    {
        $weight = array_sum(array_map(function ($item) {
            if ($item->getParentItem()) {
                return 0;
            }

            return $item->getWeight() * $item->getQty();
        }, $quote->getAllVisibleItems()));

        return array_merge(
            $createRequestDTO->getMagentoTotals(),
            [
                'state' => Order::STATE_NEW,
                'created_at' => $createRequestDTO->getCreatedAt(),
                'email_sent' => $createRequestDTO->isSendOriginNotifications(),
                'weight' => $weight,
                'customer_is_guest' => $quote->getCustomerIsGuest(),
            ]
        );
    }

    /**
     * Set different currencies to quote, before place an order
     *
     * @param CreateRequestDTOInterface $createRequestDTO
     * @param CartInterface $quote
     * @return void
     */
    private function setCurrency(CreateRequestDTOInterface $createRequestDTO, CartInterface $quote): void
    {
        $currency = $createRequestDTO->getCurrency();
        if ($createRequestDTO->isDiffCurrency()) {
            $exchangeCurrency = $createRequestDTO->getExchangeCurrency();
            $exchangeRate = $createRequestDTO->getExchangeRate();

            $quote->setGlobalCurrencyCode($exchangeCurrency)
                ->setBaseCurrencyCode($exchangeCurrency)
                ->setStoreCurrencyCode($currency)
                ->setQuoteCurrencyCode($currency)
                ->setStoreToBaseRate($exchangeRate)
                ->setStoreToQuoteRate(1.00)
                ->setBaseToGlobalRate(1.00)
                ->setBaseToQuoteRate($exchangeRate);
        }
    }

    /**
     * Get invoices data by order.
     *
     * @param OrderInterface $order
     * @return array
     */
    private function getInvoicesDataByOrder(OrderInterface $order): array
    {
        $result = [];

        if ($order->hasInvoices()) {
            $invoices = $order->getInvoiceCollection()->getItems();
            foreach ($invoices as $invoice) {
                $result[] = [
                    'invoice_id' => $invoice->getId(),
                    'invoice_increment_id' => $invoice->getIncrementId()
                ];
            }
        }

        return $result;
    }
}
