<?php

namespace Glopal\Connector\Provider\CartTransfer;

use Exception;
use Magento\GiftMessage\Api\CartRepositoryInterface;
use Magento\GiftMessage\Api\Data\MessageInterface;
use Magento\GiftMessage\Api\ItemRepositoryInterface;

class GiftMessageProvider
{
    /** @var CartRepositoryInterface */
    private $giftMessageCartRepository;

    /** @var ItemRepositoryInterface */
    private $giftMessageItemRepository;

    /** @var array */
    private $loadedQuoteGiftMessage;

    /** @var array */
    private $loadedQuoteItemGiftMessage;

    /**
     * @param CartRepositoryInterface $giftMessageCartRepository
     * @param ItemRepositoryInterface $giftMessageItemRepository
     */
    public function __construct(
        CartRepositoryInterface $giftMessageCartRepository,
        ItemRepositoryInterface $giftMessageItemRepository
    ) {
        $this->giftMessageCartRepository = $giftMessageCartRepository;
        $this->giftMessageItemRepository = $giftMessageItemRepository;
    }

    /**
     * Retrieve gift message by quote.
     *
     * @param int $quoteId
     *
     * @return string|null
     */
    public function getQuoteGiftMessage(int $quoteId): ?string
    {
        $messageObject = $this->loadQuoteGiftMessage($quoteId);

        return null === $messageObject ? $messageObject : $this->getMessageText($messageObject);
    }

    /**
     * Retrieve gift message sender by quote.
     *
     * @param int $quoteId
     *
     * @return string|null
     */
    public function getQuoteGiftMessageSender(int $quoteId): ?string
    {
        $messageObject = $this->loadQuoteGiftMessage($quoteId);

        return null === $messageObject ? $messageObject : $messageObject->getSender();
    }

    /**
     * Retrieve gift message recipient by quote.
     *
     * @param int $quoteId
     *
     * @return string|null
     */
    public function getQuoteGiftMessageRecipient(int $quoteId): ?string
    {
        $messageObject = $this->loadQuoteGiftMessage($quoteId);

        return null === $messageObject ? $messageObject : $messageObject->getRecipient();
    }

    /**
     * Retrieve gift message by quote and item.
     *
     * @param int $quoteId
     * @param int $itemId
     *
     * @return string|null
     */
    public function getQuoteItemGiftMessage(int $quoteId, int $itemId): ?string
    {
        $messageObject = $this->loadQuoteItemGiftMessage($quoteId, $itemId);

        return null === $messageObject ? $messageObject : $this->getMessageText($messageObject);
    }

    /**
     * Retrieve gift message sender by quote.
     *
     * @param int $quoteId
     * @param int $itemId
     *
     * @return string|null
     */
    public function getQuoteItemGiftMessageSender(int $quoteId, int $itemId): ?string
    {
        $messageObject = $this->loadQuoteItemGiftMessage($quoteId, $itemId);

        return null === $messageObject ? $messageObject : $messageObject->getSender();
    }

    /**
     * Retrieve gift message recipient by quote and item.
     *
     * @param int $quoteId
     * @param int $itemId
     *
     * @return string|null
     */
    public function getQuoteItemGiftMessageRecipient(int $quoteId, int $itemId): ?string
    {
        $messageObject = $this->loadQuoteItemGiftMessage($quoteId, $itemId);

        return null === $messageObject ? $messageObject : $messageObject->getRecipient();
    }

    /**
     * Load quote gift message object.
     *
     * @param int $quoteId
     *
     * @return MessageInterface|null
     */
    private function loadQuoteGiftMessage(int $quoteId): ?MessageInterface
    {
        if (isset($this->loadedQuoteGiftMessage[$quoteId])) {
            return $this->loadedQuoteGiftMessage[$quoteId];
        }

        try {
            $this->loadedQuoteGiftMessage[$quoteId] = $this->giftMessageCartRepository->get($quoteId);
        } catch (Exception $e) {
            return null;
        }

        return $this->loadedQuoteGiftMessage[$quoteId] ?? null;
    }

    /**
     * Load quote item gift message object.
     *
     * @param int $quoteId
     * @param int $itemId
     *
     * @return MessageInterface|null
     */
    private function loadQuoteItemGiftMessage(int $quoteId, int $itemId): ?MessageInterface
    {
        $key = $quoteId . $itemId;
        if (isset($this->loadedQuoteItemGiftMessage[$key])) {
            return $this->loadedQuoteItemGiftMessage[$key];
        }

        try {
            $this->loadedQuoteItemGiftMessage[$key] = $this->giftMessageItemRepository->get($quoteId, $itemId);
        } catch (Exception $e) {
            return null;
        }

        return $this->loadedQuoteItemGiftMessage[$key] ?? null;
    }

    /**
     * Retrieve message text.
     *
     * @param MessageInterface $message
     *
     * @return string
     */
    private function getMessageText(MessageInterface $message): string
    {
        $messageText = $message->getMessage();
        if ($messageText === null || trim($messageText) == '') {
            return '';
        }

        return $messageText;
    }
}
