<?php

namespace Glopal\Connector\Provider\CartTransfer;

use Glopal\Connector\Helper\Configuration;
use Magento\Framework\Locale\Resolver;
use Magento\Framework\Stdlib\CookieManagerInterface;
use Magento\Framework\UrlInterface;
use Magento\Store\Api\Data\StoreInterface;

/**
 * @SuppressWarnings(PHPMD.CookieAndSessionMisuse)
 */
class RequestUrlProvider
{
    /** @var string[] */
    private const TRACKING_COOKIES = [
        '_gcl_aw',
        '_gcl_dc',
        ' _gcl_gb',
        '_gcl_au',
        '_fbc',
        '_fbp'
    ];

    /** @var Configuration */
    private $configuration;

    /** @var UrlInterface */
    private $urlBuilder;

    /** @var StoreInterface */
    private $store;

    /** @var CookieManagerInterface */
    private $cookieManager;

    /** @var Resolver */
    private $localeResolver;

    /**
     * @param Configuration $configuration
     * @param UrlInterface $urlBuilder
     * @param StoreInterface $store
     * @param CookieManagerInterface $cookieManager
     * @param Resolver $localeResolver
     */
    public function __construct(
        Configuration $configuration,
        UrlInterface $urlBuilder,
        StoreInterface $store,
        CookieManagerInterface $cookieManager,
        Resolver $localeResolver
    ) {
        $this->configuration = $configuration;
        $this->urlBuilder = $urlBuilder;
        $this->store = $store;
        $this->cookieManager = $cookieManager;
        $this->localeResolver = $localeResolver;
    }

    /**
     * Provides cart transfer request url.
     *
     * @return string|null
     */
    public function get(): ?string
    {
        $storeShortName = $this->configuration->getGeneralValue('short_name');
        $storeCode = $this->configuration->getCheckoutValue('general/store_code');
        $requestUrlPattern = $this->configuration->getGeneralValue('request_url_pattern');

        if (empty($storeShortName) || empty($storeCode) || empty($requestUrlPattern)) {
            return null;
        }

        $requestUrl = sprintf($requestUrlPattern, $storeShortName, $storeCode);
        $this->addReferer($requestUrl);
        $this->addVisitorCurrency($requestUrl);
        $this->addVisitorLanguage($requestUrl);
        $this->addTracking($requestUrl);
        $this->addGoogleAnalytics($requestUrl);

        return $requestUrl;
    }

    /**
     * Adds referer param to url.
     *
     * @param string $url
     *
     * @return void
     */
    private function addReferer(string &$url): void
    {
        $url .= '?referrer=' . $this->urlBuilder->getUrl('checkout/cart');
    }

    /**
     * Adds visitor currency param to url.
     *
     * @param string $url
     *
     * @return void
     */
    private function addVisitorCurrency(string &$url): void
    {
        $url .= '&visitorCurrency=' . $this->store->getCurrentCurrencyCode();
    }

    /**
     * Adds visitor language param to url.
     *
     * @param string $url
     *
     * @return void
     */
    private function addVisitorLanguage(string &$url): void
    {
        $url .= '&visitorLanguage=' . str_replace('_', '-', $this->localeResolver->getLocale());
    }

    /**
     * Adds tracking params to url.
     *
     * @param string $url
     *
     * @return void
     */
    private function addTracking(string &$url): void
    {
        foreach (self::TRACKING_COOKIES as $cookie) {
            $value = $this->cookieManager->getCookie($cookie);

            if (null !== $value) {
                $url .= '&tracking[cookies][' . $cookie . ']=' . $this->urlEncode($value);
            }
        }
    }

    /**
     * Adds Google Analytics params to url.
     *
     * @param string $url
     *
     * @return void
     */
    private function addGoogleAnalytics(string &$url): void
    {
        $gaId = $this->cookieManager->getCookie('_ga');
        if (null !== $gaId) {
            $gaParts = explode('.', $gaId);
            if (isset($gaParts[2], $gaParts[3])) {
                $clientId = $this->urlEncode($gaParts[2] . '.' . $gaParts[3]);
                $url .= '&clientIds[ga]=' . $clientId;
            }
        }

        $ga4MeasurementId = $this->configuration->getGeneralValue('analytics/ga4_measurement_id');
        if (!empty($ga4MeasurementId)) {
            $ga4Id =  $this->cookieManager->getCookie('_ga_' . $ga4MeasurementId);

            if (null !== $ga4Id) {
                $ga4Parts = explode('.', $ga4Id);
                if (isset($ga4Parts[2])) {
                    $url .= '&clientIds[ga4_session_id]=' . $this->urlEncode($ga4Parts[2]);
                }
            }
        }
    }

    /**
     * Encoding url value.
     *
     * @param string $value
     *
     * @return string
     */
    private function urlEncode(string $value): string
    {
        // phpcs:ignore
        return urlencode($value);
    }
}
