<?php

namespace Glopal\Connector\Test\Integration\Controller\Order;

use Glopal\Connector\Helper\Configuration;
use Glopal\Connector\Model\Order\Management;
use Magento\Framework\App\Request\Http as HttpRequest;
use Magento\Framework\App\Utility\Files;
use Magento\Framework\Serialize\SerializerInterface;
use Magento\Sales\Api\Data\OrderInterface;
use Magento\Sales\Api\OrderRepositoryInterface;
use Magento\TestFramework\Helper\Bootstrap;
use Magento\TestFramework\TestCase\AbstractController;

/**
 * @magentoAppArea frontend
 */
class IndexTest extends AbstractController
{
    private const CONTROLLER_URL = 'glopal_connector/order/index';

    /** @var OrderRepositoryInterface */
    private $orderRepository;

    /** @var Management */
    private $glopalOrderManagement;

    /** @var SerializerInterface */
    private $serializer;

    /** @var Configuration */
    private $configuration;

    /** @var array */
    private $payloadFiles;

    protected function setUp(): void
    {
        parent::setUp();

        $this->orderRepository = Bootstrap::getObjectManager()->create(OrderRepositoryInterface::class);
        $this->glopalOrderManagement = Bootstrap::getObjectManager()->create(Management::class);
        $this->serializer = $this->_objectManager->get(SerializerInterface::class);
        $this->configuration = $this->_objectManager->get(Configuration::class);
        $this->payloadFiles = Files::getFiles([__DIR__ . '/_files/payloads'], '*.json');
    }

    public function testCreateOrder()
    {
        foreach ($this->payloadFiles as $payloadFile) {
            $payload = file_get_contents($payloadFile);

            $this->getRequest()->setContent($payload);
            $this->getRequest()->setMethod(HttpRequest::METHOD_POST);
            $this->getRequest()->getHeaders()->addHeaderLine('X-Glopal-Signature-Version', 3);
            $this->getRequest()->getHeaders()->addHeaderLine('X-Glopal-Request-Id', 123);
            $this->getRequest()->getHeaders()->addHeaderLine(
                'X-Glopal-Signature',
                $this->generateSignature($payload)
            );

            $this->dispatch(self::CONTROLLER_URL);

            $json = $this->getResponse()->getBody();

            if (method_exists($this, 'assertStringContainsString')) {
                $this->assertStringContainsString('order_id', $json);
                $this->assertStringContainsString('increment_id', $json);
            } else {
                $this->assertContains('order_id', $json);
                $this->assertContains('increment_id', $json);
            }

            $response = $this->serializer->unserialize($json);
            $id = $response['message']['order_id'];
            $order = $this->orderRepository->get($id);

            $this->assertOrder($payload, $order);

            $this->getRequest()->getHeaders()->clearHeaders();
        }
    }

    /**
     * Generates signature for requests.
     *
     * @param $payload
     * @return string
     */
    private function generateSignature($payload): string
    {
        return hash_hmac(
            'sha256',
            '123' . $payload . 'POST' . self::CONTROLLER_URL,
            $this->configuration->getGeneralValue('key')
        );
    }

    private function assertOrder($payload, OrderInterface $order)
    {
        $expectedData = $this->serializer->unserialize($payload);

        // Assert summary total
        $this->assertEquals($expectedData['summary']['total']['total'], $order->getGrandTotal());

        // Assert order email
        $emailAlias = $expectedData['customer']['email_alias'] ?? null;
        if (!empty($emailAlias)) {
            $this->assertEquals($emailAlias, $order->getCustomerEmail());
        }

        // Assert order discounts
        $orderDiscount = $order->getDiscountAmount();
        if (isset($expectedData['summary']['discounts']['total'])
            && $expectedData['summary']['discounts']['total'] > 0
        ) {
            $this->assertEquals(
                $expectedData['summary']['discounts']['total'] * -1,
                $orderDiscount
            );
        }

        // Assert Order Item Discounts
        $isAnyItemDiscounted = false;
        foreach ($expectedData['items'] as $payloadItem) {
            if (isset($payloadItem['allocations']['discounts']['total']['line_charge'])
                && $payloadItem['allocations']['discounts']['total']['line_charge']['amount_net'] > 0
            ) {
                $isAnyItemDiscounted = true;
            }
        }

        if ($isAnyItemDiscounted) {
            foreach ($order->getItems() as $orderItem) {
                foreach ($expectedData['items'] as $payloadItem) {
                    if ($orderItem->getSku() == $payloadItem['sku']) {
                        $this->assertEquals(
                            $payloadItem['allocations']['discounts']['total']['line_charge']['amount_net'],
                            $orderItem->getDiscountAmount()
                        );
                    }
                }
            }
        }

        // Assert order taxes
        $orderTax = $order->getTaxAmount();
        if (!empty($orderTax)) {
            $this->assertEquals(
                $expectedData['summary']['total']['tax_on_amount'] + $expectedData['summary']['total']['tax_on_duty'],
                $orderTax
            );
        }

        // Assert total items count
        $orderItemsCount = $order->getTotalItemCount();
        $this->assertEquals(count($expectedData['items']), $orderItemsCount);

        // Assert Glopal Attributes
        $glopalOrderAttribute = $this->glopalOrderManagement->getOrderAttribute($order);
        $actualExchangeRate = $glopalOrderAttribute->getExchangeRate();
        $actualFromCurrency = $glopalOrderAttribute->getFromCurrency();
        $actualToCurrency = $glopalOrderAttribute->getToCurrency();
        $actualCustomerOriginEmail = $glopalOrderAttribute->getCustomerOriginEmail();

        $this->assertEquals($expectedData['exchange_rates'][$actualToCurrency]['rate'], $actualExchangeRate);
        $this->assertEquals($expectedData['currency'], $actualFromCurrency);
        $this->assertEquals($expectedData['customer']['email'], $actualCustomerOriginEmail);

        $transactionId = $expectedData['total_cost']['transaction_id'] ?? null;
        if (!empty($transactionId)) {
            $actualTransactionId = $glopalOrderAttribute->getTransactionId();
            $this->assertEquals($transactionId, $actualTransactionId);
        }

        // Assert Shipping data (amount, address)
        $shippingAmount = $order->getShippingAmount();
        if (!empty($shippingAmount)) {
            $this->assertEquals($expectedData['summary']['shipping']['amount_net'], $shippingAmount);
        }

        $shippingAddress = $order->getShippingAddress();
        $expectedShippingAddress = $expectedData['shipping_address'];
        $this->assertEquals($expectedShippingAddress['address1'], $shippingAddress->getStreet()[0]);
        $this->assertEquals($expectedShippingAddress['city'], $shippingAddress->getCity());
        $this->assertEquals($expectedShippingAddress['first_name'], $shippingAddress->getFirstname());
        $this->assertEquals($expectedShippingAddress['last_name'], $shippingAddress->getLastname());
        $this->assertEquals($expectedShippingAddress['postcode'], $shippingAddress->getPostcode());
        $this->assertEquals($expectedShippingAddress['telephone'], $shippingAddress->getTelephone());

        // Assert Duties data
        $totalCost = $this->glopalOrderManagement->getTotalCost($order);
        if (!empty($totalCost) && !empty($totalCost->getTotalDutyAmount())) {
            $dutyAmount = $totalCost->getDiscountDutyAmount();
            $taxOnDuty = $totalCost->getTotalTaxOnDutyAmount();
            $shippingDutyAmount = $totalCost->getShippingDutyAmount();

            $this->assertEquals($expectedData['summary']['total']['duty_amount'], $dutyAmount);
            $this->assertEquals($expectedData['summary']['total']['tax_on_duty'], $taxOnDuty);
            $this->assertEquals($expectedData['summary']['shipping']['duty_amount'], $shippingDutyAmount);
        }

        // Assert Items Data (SKU and Qty)
        $orderItemsData = [];
        foreach ($order->getItems() as $orderItem) {
            $orderItemsData[$orderItem->getSku()] = (int)$orderItem->getQtyOrdered();
        }

        $payloadItemsData = [];
        foreach ($expectedData['items'] as $payloadItem) {
            $payloadItemsData[$payloadItem['sku']] = $payloadItem['quantity'];
        }

        ksort($orderItemsData);
        ksort($payloadItemsData);

        $this->assertEquals($payloadItemsData, $orderItemsData);
    }
}
