<?php

namespace Glopal\Connector\Test\Unit\Model\Order;

use Glopal\Connector\Api\Data\OrderAttributeInterface;
use Glopal\Connector\Api\Data\OrderItemTotalCostInterface;
use Glopal\Connector\Api\Data\OrderItemTotalCostSearchResultsInterface;
use Glopal\Connector\Api\Data\OrderTotalCostInterface;
use Glopal\Connector\Api\OrderAttributeRepositoryInterface;
use Glopal\Connector\Api\OrderItemTotalCostRepositoryInterface;
use Glopal\Connector\Api\OrderTotalCostRepositoryInterface;
use Glopal\Connector\Model\Order\Management;
use Magento\Framework\Api\SearchCriteria;
use Magento\Framework\Api\SearchCriteriaBuilder;
use Magento\Sales\Api\Data\OrderInterface;
use Magento\Sales\Api\Data\OrderItemInterface;
use PHPUnit\Framework\MockObject\MockObject;
use PHPUnit\Framework\TestCase;

// phpcs:ignoreFile
class ManagementTest extends TestCase
{
    /**
     * @var OrderAttributeRepositoryInterface|(OrderAttributeRepositoryInterface&MockObject)|MockObject
     */
    private $orderAttributeRepositoryMock;

    /**
     * @var OrderTotalCostRepositoryInterface|(OrderTotalCostRepositoryInterface&MockObject)|MockObject
     */
    private $orderTotalCostRepositoryMock;

    /**
     * @var OrderItemTotalCostRepositoryInterface|(OrderItemTotalCostRepositoryInterface&MockObject)|MockObject
     */
    private $itemTotalCostRepositoryMock;

    /**
     * @var SearchCriteriaBuilder|(SearchCriteriaBuilder&MockObject)|MockObject
     */
    private $searchCriteriaBuilderMock;

    /**
     * @var Management
     */
    private $orderManagement;

    /**
     * @var OrderInterface|(OrderInterface&MockObject)|MockObject
     */
    private $orderInterfaceMock;

    /**
     * @var OrderAttributeInterface|(OrderAttributeInterface&MockObject)|MockObject
     */
    private $orderAttributeInterfaceMock;

    /**
     * @var OrderTotalCostInterface|(OrderTotalCostInterface&MockObject)|MockObject
     */
    private $orderTotalCostInterfaceMock;

    /**
     * @var OrderItemTotalCostSearchResultsInterface|(OrderItemTotalCostSearchResultsInterface&MockObject)|MockObject
     */
    private $orderItemTotalCostSearchResultsInterfaceMock;

    /**
     * @var OrderItemTotalCostInterface|(OrderItemTotalCostInterface&MockObject)|MockObject
     */
    private $orderItemTotalCostInterfaceMock;

    /**
     * @var SearchCriteria|(SearchCriteria&MockObject)|MockObject
     */
    private $searchCriteriaMock;

    protected function setUp(): void
    {
        $this->orderAttributeRepositoryMock = $this->getMockForAbstractClass(
            OrderAttributeRepositoryInterface::class
        );
        $this->orderTotalCostRepositoryMock = $this->getMockForAbstractClass(
            OrderTotalCostRepositoryInterface::class
        );
        $this->itemTotalCostRepositoryMock = $this->getMockForAbstractClass(
            OrderItemTotalCostRepositoryInterface::class
        );
        $this->searchCriteriaBuilderMock = $this->createMock(SearchCriteriaBuilder::class);

        $this->orderManagement = new Management(
            $this->orderAttributeRepositoryMock,
            $this->orderTotalCostRepositoryMock,
            $this->itemTotalCostRepositoryMock,
            $this->searchCriteriaBuilderMock
        );

        $this->orderInterfaceMock = $this->getMockForAbstractClass(
            OrderInterface::class,
            [],
            '',
            false,
            false,
            true,
            ['getId']
        );
        $this->orderInterfaceMock
            ->method('getId')
            ->willReturn(12);

        $this->orderAttributeInterfaceMock = $this->getMockForAbstractClass(OrderAttributeInterface::class);
        $this->orderTotalCostInterfaceMock = $this->getMockForAbstractClass(OrderTotalCostInterface::class);
        $this->orderItemTotalCostSearchResultsInterfaceMock = $this->getMockForAbstractClass(
            OrderItemTotalCostSearchResultsInterface::class
        );
        $this->orderItemTotalCostInterfaceMock = $this->getMockForAbstractClass(OrderItemTotalCostInterface::class);
        $this->searchCriteriaMock = $this->createMock(SearchCriteria::class);

        $this->searchCriteriaBuilderMock
            ->method('create')
            ->willReturn($this->searchCriteriaMock);
    }

    public function testGetOrderAttribute()
    {
        $this->orderAttributeRepositoryMock
            ->method('getByOrderId')
            ->with(12)
            ->willReturn($this->orderAttributeInterfaceMock);

        $result = $this->orderManagement->getOrderAttribute($this->orderInterfaceMock);

        $this->assertEquals($this->orderAttributeInterfaceMock, $result);
    }

    public function testGetOrderAttributeReturnsNullWhenThereIsNoOrderAttributeForGivenOrderId()
    {
        $this->orderAttributeRepositoryMock
            ->method('getByOrderId')
            ->with(12)
            ->willReturn(null);

        $result = $this->orderManagement->getOrderAttribute($this->orderInterfaceMock);

        $this->assertEquals(null, $result);
    }

    public function testAddOrderAttribute()
    {
        $attributeData = [
            'exchange_rate' => 1.22,
            'from_currency' => 'EUR',
            'to_currency' => 'USD',
            'customer_origin_email' => 'email@example.com',
            'customer_shipping_address' => ['Some address'],
            'base_extra_amount_net' => 5.0,
            'extra_amount_net' => 5.0,
            'base_extra_tax_on_amount' => 22.0,
            'extra_tax_on_amount' => 22.0,
        ];

        $this->setUpOrderAttributeInterface();

        $this->orderAttributeInterfaceMock
            ->expects($this->once())
            ->method('setCustomerShippingAddress')
            ->with(['Some address']);

        $this->orderAttributeInterfaceMock
            ->expects($this->once())
            ->method('setBaseExtraAmount')
            ->with(5.0);

        $this->orderAttributeInterfaceMock
            ->expects($this->once())
            ->method('setExtraAmount')
            ->with(5.0);

        $result = $this->orderManagement->addOrderAttribute($this->orderInterfaceMock, $attributeData);
        $this->assertEquals($this->orderAttributeInterfaceMock, $result);
    }

    public function testAddOrderAttributeWhenThereIsNotCompleteAttributeData()
    {
        $attributeData = [
            'exchange_rate' => 1.22,
            'from_currency' => 'EUR',
            'to_currency' => 'USD',
            'customer_origin_email' => 'email@example.com',
            'base_extra_tax_on_amount' => 22.0,
            'extra_tax_on_amount' => 22.0,
        ];

        $this->setUpOrderAttributeInterface();

        $this->orderAttributeInterfaceMock
            ->expects($this->never())
            ->method('setCustomerShippingAddress');

        $this->orderAttributeInterfaceMock
            ->expects($this->never())
            ->method('setBaseExtraAmount');

        $this->orderAttributeInterfaceMock
            ->expects($this->never())
            ->method('setExtraAmount');

        $result = $this->orderManagement->addOrderAttribute($this->orderInterfaceMock, $attributeData);
        $this->assertEquals($this->orderAttributeInterfaceMock, $result);
    }

    private function setUpOrderAttributeInterface(): void
    {
        $this->orderAttributeRepositoryMock
            ->expects($this->once())
            ->method('getEmpty')
            ->willReturn($this->orderAttributeInterfaceMock);

        $this->orderAttributeInterfaceMock
            ->expects($this->once())
            ->method('setOrderId')
            ->with(12);

        $this->orderAttributeInterfaceMock
            ->expects($this->once())
            ->method('setExchangeRate')
            ->with(1.22);

        $this->orderAttributeInterfaceMock
            ->expects($this->once())
            ->method('setFromCurrency')
            ->with('EUR');

        $this->orderAttributeInterfaceMock
            ->expects($this->once())
            ->method('setToCurrency')
            ->with('USD');

        $this->orderAttributeInterfaceMock
            ->expects($this->once())
            ->method('setCustomerOriginEmail')
            ->with('email@example.com');

        $this->orderAttributeInterfaceMock
            ->expects($this->once())
            ->method('setBaseExtraTaxOnAmount')
            ->with(22.0);

        $this->orderAttributeInterfaceMock
            ->expects($this->once())
            ->method('setExtraTaxOnAmount')
            ->with(22.0);

        $this->orderAttributeRepositoryMock
            ->expects($this->once())
            ->method('save')
            ->with($this->orderAttributeInterfaceMock)
            ->willReturn($this->orderAttributeInterfaceMock);
    }

    public function testGetTotalCost()
    {
        $this->orderTotalCostRepositoryMock
            ->method('getByOrderId')
            ->with(12)
            ->willReturn($this->orderTotalCostInterfaceMock);

        $result = $this->orderManagement->getTotalCost($this->orderInterfaceMock);

        $this->assertEquals($this->orderTotalCostInterfaceMock, $result);
    }

    public function testGetTotalCostReturnsNullWhenThereIsNoOrderAttributeForGivenOrderId()
    {
        $this->orderTotalCostRepositoryMock
            ->method('getByOrderId')
            ->with(12)
            ->willReturn(null);

        $result = $this->orderManagement->getTotalCost($this->orderInterfaceMock);

        $this->assertEquals(null, $result);
    }

    public function testAddTotalCost()
    {
        $totalCostData = [
            'base_total_duty_amount' => 5.5,
            'duty_total_amount' => 6.5,
            'base_total_tax_on_duty' => 2.45,
            'total_tax_on_duty' => 5.7,
            'base_items_duty_amount' => 5.9,
            'duty_items_amount' => 1.0,
            'base_items_tax_on_duty' => 20.0,
            'items_tax_on_duty' => 21.5,
            'base_discount_duty_amount' => 14.0,
            'discount_duty_amount' => 14.0,
            'base_discount_tax_on_duty' => 18.0,
            'discount_tax_on_duty' => 19.0,
            'base_shipping_duty_amount' => 16.4,
            'shipping_duty_amount' => 7.9,
            'base_shipping_tax_on_duty' => 5.0,
            'shipping_tax_on_duty' => 15.0,
            'base_extra_duty_amount' => 5.0,
            'extra_duty_amount' => 5.5,
            'base_extra_tax_on_duty' => 19.0,
            'extra_tax_on_duty' => 19.0,
        ];

        $this->setUpOrderTotalCostInterface();

        $this->orderTotalCostInterfaceMock
            ->expects($this->once())
            ->method('setItemsDutyAmount')
            ->with(1.0);

        $this->orderTotalCostInterfaceMock
            ->expects($this->once())
            ->method('setBaseItemsTaxOnDutyAmount')
            ->with(20.0);

        $this->orderTotalCostInterfaceMock
            ->expects($this->once())
            ->method('setItemsTaxOnDutyAmount')
            ->with(21.5);

        $this->orderTotalCostInterfaceMock
            ->expects($this->once())
            ->method('setBaseDiscountDutyAmount')
            ->with(14.0);

        $this->orderTotalCostInterfaceMock
            ->expects($this->once())
            ->method('setDiscountDutyAmount')
            ->with(14.0);

        $this->orderTotalCostInterfaceMock
            ->expects($this->once())
            ->method('setBaseDiscountTaxOnDutyAmount')
            ->with(18.0);

        $this->orderTotalCostInterfaceMock
            ->expects($this->once())
            ->method('setDiscountTaxOnDutyAmount')
            ->with(19.0);

        $this->orderTotalCostInterfaceMock
            ->expects($this->once())
            ->method('setBaseShippingDutyAmount')
            ->with(16.4);

        $this->orderTotalCostInterfaceMock
            ->expects($this->once())
            ->method('setShippingDutyAmount')
            ->with(7.9);

        $this->orderTotalCostInterfaceMock
            ->expects($this->once())
            ->method('setBaseShippingTaxOnDutyAmount')
            ->with(5.0);

        $this->orderTotalCostInterfaceMock
            ->expects($this->once())
            ->method('setShippingTaxOnDutyAmount')
            ->with(15.0);

        $result = $this->orderManagement->addTotalCost($this->orderInterfaceMock, $totalCostData);
        $this->assertEquals($this->orderTotalCostInterfaceMock, $result);
    }

    public function testAddTotalCostWhenThereIsNotCompleteTotalCostData()
    {
        $totalCostData = [
            'base_total_duty_amount' => 5.5,
            'duty_total_amount' => 6.5,
            'base_total_tax_on_duty' => 2.45,
            'total_tax_on_duty' => 5.7,
            'base_items_duty_amount' => 5.9,
            'base_items_tax_on_duty' => 20.0,
            'items_tax_on_duty' => 21.5,
            'base_discount_duty_amount' => 14.0,
            'discount_duty_amount' => 14.0,
            'base_discount_tax_on_duty' => 18.0,
            'discount_tax_on_duty' => 19.0,
            'base_extra_duty_amount' => 5.0,
            'extra_duty_amount' => 5.5,
            'base_extra_tax_on_duty' => 19.0,
            'extra_tax_on_duty' => 19.0,
        ];

        $this->setUpOrderTotalCostInterface();

        $this->orderTotalCostInterfaceMock
            ->expects($this->never())
            ->method('setItemsDutyAmount');

        $this->orderTotalCostInterfaceMock
            ->expects($this->never())
            ->method('setBaseShippingDutyAmount');

        $this->orderTotalCostInterfaceMock
            ->expects($this->never())
            ->method('setShippingDutyAmount');

        $this->orderTotalCostInterfaceMock
            ->expects($this->never())
            ->method('setBaseShippingTaxOnDutyAmount');

        $this->orderTotalCostInterfaceMock
            ->expects($this->never())
            ->method('setShippingTaxOnDutyAmount');

        $result = $this->orderManagement->addTotalCost($this->orderInterfaceMock, $totalCostData);
        $this->assertEquals($this->orderTotalCostInterfaceMock, $result);
    }

    private function setUpOrderTotalCostInterface(): void
    {
        $this->orderTotalCostRepositoryMock
            ->expects($this->once())
            ->method('getEmpty')
            ->willReturn($this->orderTotalCostInterfaceMock);

        $this->orderTotalCostInterfaceMock
            ->expects($this->once())
            ->method('setOrderId')
            ->with(12);

        $this->orderTotalCostInterfaceMock
            ->expects($this->once())
            ->method('setBaseTotalDutyAmount')
            ->with(5.5);

        $this->orderTotalCostInterfaceMock
            ->expects($this->once())
            ->method('setTotalDutyAmount')
            ->with(6.5);

        $this->orderTotalCostInterfaceMock
            ->expects($this->once())
            ->method('setBaseTotalTaxOnDutyAmount')
            ->with(2.45);

        $this->orderTotalCostInterfaceMock
            ->expects($this->once())
            ->method('setTotalTaxOnDutyAmount')
            ->with(5.7);

        $this->orderTotalCostInterfaceMock
            ->expects($this->once())
            ->method('setBaseItemsDutyAmount')
            ->with(5.9);

        $this->orderTotalCostInterfaceMock
            ->expects($this->once())
            ->method('setBaseExtraDutyAmount')
            ->with(5.0);

        $this->orderTotalCostInterfaceMock
            ->expects($this->once())
            ->method('setExtraDutyAmount')
            ->with(5.5);

        $this->orderTotalCostInterfaceMock
            ->expects($this->once())
            ->method('setBaseExtraTaxOnDutyAmount')
            ->with(19.0);

        $this->orderTotalCostInterfaceMock
            ->expects($this->once())
            ->method('setExtraTaxOnDutyAmount')
            ->with(19.0);

        $this->orderTotalCostRepositoryMock
            ->expects($this->once())
            ->method('save')
            ->with($this->orderTotalCostInterfaceMock)
            ->willReturn($this->orderTotalCostInterfaceMock);
    }

    public function testGetItemsTotalCost()
    {
        $orderTccId = 32;
        $this->searchCriteriaBuilderMock
            ->method('addFilter')
            ->with('order_tcc_id', $orderTccId)
            ->willReturnSelf();

        $this->itemTotalCostRepositoryMock
            ->method('getList')
            ->with($this->searchCriteriaMock)
            ->willReturn($this->orderItemTotalCostSearchResultsInterfaceMock);

        $this->orderItemTotalCostSearchResultsInterfaceMock
            ->method('getItems')
            ->willReturn([$this->orderItemTotalCostInterfaceMock]);

        $result = $this->orderManagement->getItemsTotalCost($orderTccId);
        $this->assertEquals([$this->orderItemTotalCostInterfaceMock], $result);
    }

    public function testGetItemsTotalCostReturnsEmptyArrayWhenThereIsNoRecordsFound()
    {
        $orderTccId = 32;
        $this->searchCriteriaBuilderMock
            ->method('addFilter')
            ->with('order_tcc_id', $orderTccId)
            ->willReturnSelf();

        $this->itemTotalCostRepositoryMock
            ->method('getList')
            ->with($this->searchCriteriaMock)
            ->willReturn($this->orderItemTotalCostSearchResultsInterfaceMock);

        $this->orderItemTotalCostSearchResultsInterfaceMock
            ->method('getItems')
            ->willReturn([]);

        $result = $this->orderManagement->getItemsTotalCost($orderTccId);
        $this->assertEquals([], $result);
    }

    public function testGetItemsTotalCostByOrderItemId()
    {
        $orderItemId = 23;
        $this->searchCriteriaBuilderMock
            ->method('addFilter')
            ->with('order_item_id', $orderItemId)
            ->willReturnSelf();

        $this->itemTotalCostRepositoryMock
            ->method('getList')
            ->with($this->searchCriteriaMock)
            ->willReturn($this->orderItemTotalCostSearchResultsInterfaceMock);

        $this->orderItemTotalCostSearchResultsInterfaceMock
            ->method('getItems')
            ->willReturn([$this->orderItemTotalCostInterfaceMock]);

        $result = $this->orderManagement->getItemsTotalCostByOrderItemId($orderItemId);
        $this->assertEquals($this->orderItemTotalCostInterfaceMock, $result);
    }

    public function testGetItemsTotalCostByOrderItemIdReturnsNullWhenThereIsNoRecordsFound()
    {
        $orderItemId = 23;
        $this->searchCriteriaBuilderMock
            ->method('addFilter')
            ->with('order_item_id', $orderItemId)
            ->willReturnSelf();

        $this->itemTotalCostRepositoryMock
            ->method('getList')
            ->with($this->searchCriteriaMock)
            ->willReturn($this->orderItemTotalCostSearchResultsInterfaceMock);

        $this->orderItemTotalCostSearchResultsInterfaceMock
            ->method('getItems')
            ->willReturn([]);

        $result = $this->orderManagement->getItemsTotalCostByOrderItemId($orderItemId);
        $this->assertEquals(null, $result);
    }

    public function testAddItemTotalCost()
    {
        $itemTotalCostData = [
            'order_tcc_id' => 24,
            'base_duty_amount' => 7.7,
            'duty_amount' => 8.7,
            'base_tax_on_duty' => 16.0,
            'tax_on_duty' => 17.6,
            'duty_percent' => 25.0,
        ];

        $orderItemInterfaceMock = $this->getMockForAbstractClass(
            OrderItemInterface::class,
            [],
            '',
            false,
            false,
            true,
            ['getId']
        );
        $orderItemInterfaceMock
            ->method('getId')
            ->willReturn(21);

        $this->itemTotalCostRepositoryMock
            ->expects($this->once())
            ->method('getEmpty')
            ->willReturn($this->orderItemTotalCostInterfaceMock);

        $this->orderItemTotalCostInterfaceMock
            ->expects($this->once())
            ->method('setOrderItemId')
            ->with(21);

        $this->orderItemTotalCostInterfaceMock
            ->expects($this->once())
            ->method('setOrderTccId')
            ->with(24);

        $this->orderItemTotalCostInterfaceMock
            ->expects($this->once())
            ->method('setBaseDutyAmount')
            ->with(7.7);

        $this->orderItemTotalCostInterfaceMock
            ->expects($this->once())
            ->method('setDutyAmount')
            ->with(8.7);

        $this->orderItemTotalCostInterfaceMock
            ->expects($this->once())
            ->method('setBaseTaxOnDutyAmount')
            ->with(16.0);

        $this->orderItemTotalCostInterfaceMock
            ->expects($this->once())
            ->method('setTaxOnDutyAmount')
            ->with(17.6);

        $this->orderItemTotalCostInterfaceMock
            ->expects($this->once())
            ->method('setDutyPercent')
            ->with(25.0);

        $this->itemTotalCostRepositoryMock
            ->expects($this->once())
            ->method('save')
            ->with($this->orderItemTotalCostInterfaceMock)
            ->willReturn($this->orderItemTotalCostInterfaceMock);

        $result = $this->orderManagement->addItemTotalCost($orderItemInterfaceMock, $itemTotalCostData);
        $this->assertEquals($this->orderItemTotalCostInterfaceMock, $result);
    }
}
