<?php

namespace Glopal\Connector\Test\Unit\Model\Price;

use Glopal\Connector\Api\Data\Price\QuoteItemDataInterface;
use Glopal\Connector\Api\Data\Price\QuoteItemDataInterfaceFactory;
use Glopal\Connector\Model\Price\Calculator;
use Glopal\Connector\Model\Price\Quote\ItemData;
use Magento\Framework\TestFramework\Unit\Helper\ObjectManager;
use PHPUnit\Framework\TestCase;

// phpcs:ignoreFile
class CalculatorTest extends TestCase
{
    private $objectManager;
    private $priceQuoteItemFactory;

    /**
     * @inheritdoc
     */
    protected function setUp(): void
    {
        $this->objectManager = new ObjectManager($this);

        $this->priceQuoteItemFactory = $this->getMockBuilder(QuoteItemDataInterfaceFactory::class)
            ->disableOriginalConstructor()
            ->setMethods(['create'])
            ->getMock();
        $this->priceQuoteItemFactory->method('create')->willReturn(new ItemData());
    }

    public function testPriceCalculatorWhenPriceExclTaxApplyTaxAfterDiscountNoDuty()
    {
        $calculator = $this->objectManager->getObject(
            Calculator::class,
            [
                'priceQuoteItemFactory' => $this->priceQuoteItemFactory,
                'applyTaxAfterDiscount' => true,
                'priceIncludeTax' => false
            ]
        );

        $priceData = [
            'exchange_rate' => 0.95,
            'unit_price' => [
                'amount_net' => 83.33,
                'tax_on_amount' => 16.67,
                'total' => 104.9998,
                'duty_on_amount' => 0.0,
                'tax_on_duty' => 0.0,
            ],
            'line_price' => [
                'amount_net' => 83.33,
                'tax_on_amount' => 16.67,
                'total' => 104.9998,
                'duty_on_amount' => 0.0,
                'tax_on_duty' => 0.0,
            ],
            'discounts' => [
                'amount_net' => 8.33,
                'tax_on_amount' => 1.67,
                'total' => 10.4948,
                'duty_on_amount' => 0.0,
                'tax_on_duty' => 0.0,
            ]
        ];

        $expectedPriceModel = $this->getExpectedPriceModel($priceData);
        // Specific cases
        $expectedPriceModel->setTaxAmount(15)
            ->setBaseTaxAmount(15.7894)
            ->setDiscountAmount(8.33)
            ->setBaseDiscountAmount(8.7684)
            ->setDiscountTaxCompensationAmount(0.00)
            ->setBaseDiscountTaxCompensationAmount(0.00)
            ->setRowTotalWithDiscount(75);

        /** @var QuoteItemDataInterface $priceModel */
        $actualPriceModel = $calculator->calculateQuoteItemPrice($priceData);

        $this->assertPriceModels($expectedPriceModel, $actualPriceModel);
    }

    public function testPriceCalculatorWhenPriceExclTaxApplyTaxAfterDiscountWithDuty()
    {
        $calculator = $this->objectManager->getObject(
            Calculator::class,
            [
                'priceQuoteItemFactory' => $this->priceQuoteItemFactory,
                'applyTaxAfterDiscount' => true,
                'priceIncludeTax' => false
            ]
        );

        $priceData = [
            'exchange_rate' => 0.95,
            'unit_price' => [
                'amount_net' => 83.33,
                'tax_on_amount' => 16.67,
                'total' => 104.9998,
                'duty_on_amount' => 4.1665,
                'tax_on_duty' => 0.8333,
            ],
            'line_price' => [
                'amount_net' => 83.33,
                'tax_on_amount' => 16.67,
                'total' => 104.9998,
                'duty_on_amount' => 4.1665,
                'tax_on_duty' => 0.8333,
            ],
            'discounts' => [
                'amount_net' => 8.33,
                'tax_on_amount' => 1.67,
                'total' => 10.4948,
                'duty_on_amount' => 0.4165,
                'tax_on_duty' => 0.0833,
            ]
        ];

        $expectedPriceModel = $this->getExpectedPriceModel($priceData);
        // Specific cases
        $expectedPriceModel->setTaxAmount(15.75)
            ->setBaseTaxAmount(16.5789)
            ->setDiscountAmount(8.7465)
            ->setBaseDiscountAmount(9.2068)
            ->setDiscountTaxCompensationAmount(0.00)
            ->setBaseDiscountTaxCompensationAmount(0.00)
            ->setRowTotalWithDiscount(74.5835);

        /** @var QuoteItemDataInterface $priceModel */
        $actualPriceModel = $calculator->calculateQuoteItemPrice($priceData);

        $this->assertPriceModels($expectedPriceModel, $actualPriceModel);
    }

    public function testPriceCalculatorWhenPriceExclTaxApplyTaxBeforeDiscountNoDuty()
    {
        $calculator = $this->objectManager->getObject(
            Calculator::class,
            [
                'priceQuoteItemFactory' => $this->priceQuoteItemFactory,
                'applyTaxAfterDiscount' => false,
                'priceIncludeTax' => false
            ]
        );

        $priceData = [
            'exchange_rate' => 0.95,
            'unit_price' => [
                'amount_net' => 83.33,
                'tax_on_amount' => 16.67,
                'total' => 104.9998,
                'duty_on_amount' => 0.0,
                'tax_on_duty' => 0.0,
            ],
            'line_price' => [
                'amount_net' => 83.33,
                'tax_on_amount' => 16.67,
                'total' => 104.9998,
                'duty_on_amount' => 0.0,
                'tax_on_duty' => 0.0,
            ],
            'discounts' => [
                'amount_net' => 6.94,
                'tax_on_amount' => 1.39,
                'total' => 8.33,
                'duty_on_amount' => 0.0,
                'tax_on_duty' => 0.0,
            ]
        ];

        $expectedPriceModel = $this->getExpectedPriceModel($priceData);
        // Specific cases
        $expectedPriceModel->setTaxAmount(16.67)
            ->setBaseTaxAmount(17.5473)
            ->setDiscountAmount(8.33)
            ->setBaseDiscountAmount(8.7684)
            ->setDiscountTaxCompensationAmount(0.00)
            ->setBaseDiscountTaxCompensationAmount(0.00)
            ->setTaxBeforeDiscount(16.67)
            ->setBaseTaxBeforeDiscount(17.5473)
            ->setRowTotalWithDiscount(75);

        /** @var QuoteItemDataInterface $priceModel */
        $actualPriceModel = $calculator->calculateQuoteItemPrice($priceData);

        $this->assertPriceModels($expectedPriceModel, $actualPriceModel);
    }

    public function testPriceCalculatorWhenPriceExclTaxApplyTaxBeforeDiscountWithDuty()
    {
        $calculator = $this->objectManager->getObject(
            Calculator::class,
            [
                'priceQuoteItemFactory' => $this->priceQuoteItemFactory,
                'applyTaxAfterDiscount' => false,
                'priceIncludeTax' => false
            ]
        );

        $priceData = [
            'exchange_rate' => 0.95,
            'unit_price' => [
                'amount_net' => 83.33,
                'tax_on_amount' => 16.67,
                'total' => 104.9998,
                'duty_on_amount' => 4.1665,
                'tax_on_duty' => 0.8333,
            ],
            'line_price' => [
                'amount_net' => 83.33,
                'tax_on_amount' => 16.67,
                'total' => 104.9998,
                'duty_on_amount' => 4.1665,
                'tax_on_duty' => 0.8333,
            ],
            'discounts' => [
                'amount_net' => 6.94,
                'tax_on_amount' => 1.39,
                'total' => 8.7464,
                'duty_on_amount' => 0.347,
                'tax_on_duty' => 0.0694,
            ]
        ];

        $expectedPriceModel = $this->getExpectedPriceModel($priceData);
        // Specific cases
        $expectedPriceModel->setTaxAmount(20.8365)
            ->setBaseTaxAmount(21.9331)
            ->setDiscountAmount(8.7464)
            ->setBaseDiscountAmount(9.2067)
            ->setDiscountTaxCompensationAmount(0.00)
            ->setBaseDiscountTaxCompensationAmount(0.00)
            ->setTaxBeforeDiscount(17.5033)
            ->setBaseTaxBeforeDiscount(18.4245)
            ->setRowTotalWithDiscount(74.5836);

        /** @var QuoteItemDataInterface $priceModel */
        $actualPriceModel = $calculator->calculateQuoteItemPrice($priceData);

        $this->assertPriceModels($expectedPriceModel, $actualPriceModel);
    }

    public function testPriceCalculatorWhenPriceInclTaxApplyTaxAfterDiscountNoDuty()
    {
        $calculator = $this->objectManager->getObject(
            Calculator::class,
            [
                'priceQuoteItemFactory' => $this->priceQuoteItemFactory,
                'applyTaxAfterDiscount' => true,
                'priceIncludeTax' => true
            ]
        );

        $priceData = [
            'exchange_rate' => 0.95,
            'unit_price' => [
                'amount_net' => 83.33,
                'tax_on_amount' => 16.67,
                'total' => 104.9998,
                'duty_on_amount' => 0.0,
                'tax_on_duty' => 0.0,
            ],
            'line_price' => [
                'amount_net' => 83.33,
                'tax_on_amount' => 16.67,
                'total' => 104.9998,
                'duty_on_amount' => 0.0,
                'tax_on_duty' => 0.0,
            ],
            'discounts' => [
                'amount_net' => 6.94,
                'tax_on_amount' => 1.39,
                'total' => 8.33,
                'duty_on_amount' => 0.0,
                'tax_on_duty' => 0.0,
            ]
        ];

        $expectedPriceModel = $this->getExpectedPriceModel($priceData);
        // Specific cases
        $expectedPriceModel->setTaxAmount(15.28)
            ->setBaseTaxAmount(16.0842)
            ->setDiscountAmount(8.33)
            ->setBaseDiscountAmount(8.7684)
            ->setDiscountTaxCompensationAmount(1.39)
            ->setBaseDiscountTaxCompensationAmount(1.4631)
            ->setRowTotalWithDiscount(75);

        /** @var QuoteItemDataInterface $priceModel */
        $actualPriceModel = $calculator->calculateQuoteItemPrice($priceData);

        $this->assertPriceModels($expectedPriceModel, $actualPriceModel);
    }

    public function testPriceCalculatorWhenPriceInclTaxApplyTaxAfterDiscountWithDuty()
    {
        $calculator = $this->objectManager->getObject(
            Calculator::class,
            [
                'priceQuoteItemFactory' => $this->priceQuoteItemFactory,
                'applyTaxAfterDiscount' => true,
                'priceIncludeTax' => true
            ]
        );

        $priceData = [
            'exchange_rate' => 0.95,
            'unit_price' => [
                'amount_net' => 83.33,
                'tax_on_amount' => 16.67,
                'total' => 104.9998,
                'duty_on_amount' => 4.1665,
                'tax_on_duty' => 0.8333,
            ],
            'line_price' => [
                'amount_net' => 83.33,
                'tax_on_amount' => 16.67,
                'total' => 104.9998,
                'duty_on_amount' => 4.1665,
                'tax_on_duty' => 0.8333,
            ],
            'discounts' => [
                'amount_net' => 6.94,
                'tax_on_amount' => 1.39,
                'total' => 8.7476,
                'duty_on_amount' => 0.347,
                'tax_on_duty' => 0.0694,
            ]
        ];

        $expectedPriceModel = $this->getExpectedPriceModel($priceData);
        // Specific cases
        $expectedPriceModel->setTaxAmount(16.0439)
            ->setBaseTaxAmount(16.8883)
            ->setDiscountAmount(8.7476)
            ->setBaseDiscountAmount(9.208)
            ->setDiscountTaxCompensationAmount(1.4594)
            ->setBaseDiscountTaxCompensationAmount(1.5362)
            ->setRowTotalWithDiscount(74.5824);

        /** @var QuoteItemDataInterface $priceModel */
        $actualPriceModel = $calculator->calculateQuoteItemPrice($priceData);

        $this->assertPriceModels($expectedPriceModel, $actualPriceModel);
    }

    public function testPriceCalculatorWhenPriceInclTaxApplyTaxBeforeDiscountNoDuty()
    {
        $calculator = $this->objectManager->getObject(
            Calculator::class,
            [
                'priceQuoteItemFactory' => $this->priceQuoteItemFactory,
                'applyTaxAfterDiscount' => false,
                'priceIncludeTax' => true
            ]
        );

        $priceData = [
            'exchange_rate' => 0.95,
            'unit_price' => [
                'amount_net' => 83.33,
                'tax_on_amount' => 16.67,
                'total' => 104.9998,
                'duty_on_amount' => 0.0,
                'tax_on_duty' => 0.0,
            ],
            'line_price' => [
                'amount_net' => 83.33,
                'tax_on_amount' => 16.67,
                'total' => 104.9998,
                'duty_on_amount' => 0.0,
                'tax_on_duty' => 0.0,
            ],
            'discounts' => [
                'amount_net' => 6.94,
                'tax_on_amount' => 1.39,
                'total' => 8.33,
                'duty_on_amount' => 0.0,
                'tax_on_duty' => 0.0,
            ]
        ];

        $expectedPriceModel = $this->getExpectedPriceModel($priceData);
        // Specific cases
        $expectedPriceModel->setTaxAmount(16.67)
            ->setBaseTaxAmount(17.5473)
            ->setDiscountAmount(8.33)
            ->setBaseDiscountAmount(8.7684)
            ->setDiscountTaxCompensationAmount(0.00)
            ->setBaseDiscountTaxCompensationAmount(0.00)
            ->setTaxBeforeDiscount(16.67)
            ->setBaseTaxBeforeDiscount(17.5473)
            ->setRowTotalWithDiscount(75);

        /** @var QuoteItemDataInterface $priceModel */
        $actualPriceModel = $calculator->calculateQuoteItemPrice($priceData);

        $this->assertPriceModels($expectedPriceModel, $actualPriceModel);
    }

    public function testPriceCalculatorWhenPriceInclTaxApplyTaxBeforeDiscountWithDuty()
    {
        $calculator = $this->objectManager->getObject(
            Calculator::class,
            [
                'priceQuoteItemFactory' => $this->priceQuoteItemFactory,
                'applyTaxAfterDiscount' => false,
                'priceIncludeTax' => true
            ]
        );

        $priceData = [
            'exchange_rate' => 0.95,
            'unit_price' => [
                'amount_net' => 83.33,
                'tax_on_amount' => 16.67,
                'total' => 104.9998,
                'duty_on_amount' => 4.1665,
                'tax_on_duty' => 0.8333,
            ],
            'line_price' => [
                'amount_net' => 83.33,
                'tax_on_amount' => 16.67,
                'total' => 104.9998,
                'duty_on_amount' => 4.1665,
                'tax_on_duty' => 0.8333,
            ],
            'discounts' => [
                'amount_net' => 6.94,
                'tax_on_amount' => 1.39,
                'total' => 8.7464,
                'duty_on_amount' => 0.347,
                'tax_on_duty' => 0.0694,
            ]
        ];

        $expectedPriceModel = $this->getExpectedPriceModel($priceData);
        // Specific cases
        $expectedPriceModel->setTaxAmount(20.8365)
            ->setBaseTaxAmount(21.9331)
            ->setDiscountAmount(8.7464)
            ->setBaseDiscountAmount(9.2067)
            ->setDiscountTaxCompensationAmount(0.00)
            ->setBaseDiscountTaxCompensationAmount(0.00)
            ->setTaxBeforeDiscount(17.5033)
            ->setBaseTaxBeforeDiscount(18.4245)
            ->setRowTotalWithDiscount(74.5836);

        /** @var QuoteItemDataInterface $priceModel */
        $actualPriceModel = $calculator->calculateQuoteItemPrice($priceData);

        $this->assertPriceModels($expectedPriceModel, $actualPriceModel);
    }

    private function getExpectedPriceModel(array $priceData): ItemData
    {
        $expected = new ItemData();
        $exchangeRate = $priceData['exchange_rate'];
        $expected->setOriginalPrice($priceData['unit_price']['amount_net'])
            ->setBaseOriginalPrice($priceData['unit_price']['amount_net'] / $exchangeRate)
            ->setPrice($priceData['unit_price']['amount_net'])
            ->setBasePrice($priceData['unit_price']['amount_net'] / $exchangeRate)
            ->setPriceInclTax(
                $priceData['unit_price']['amount_net']
                + $priceData['unit_price']['tax_on_amount']
            )->setBasePriceInclTax(
                ($priceData['unit_price']['amount_net'] + $priceData['unit_price']['tax_on_amount'])
                / $exchangeRate
            )->setTaxBeforeDiscount(0.0)
            ->setBaseTaxBeforeDiscount(0.0)
            ->setRowTotal($priceData['line_price']['amount_net'])
            ->setBaseRowTotal(
                $priceData['line_price']['amount_net'] / $exchangeRate
            )->setRowTotalInclTax(
                $priceData['line_price']['amount_net'] + $priceData['line_price']['tax_on_amount']
            )->setBaseRowTotalInclTax(
                ($priceData['line_price']['amount_net'] + $priceData['line_price']['tax_on_amount'])
                / $exchangeRate
            );

        return $expected;
    }

    private function assertPriceModels(ItemData $expected, ItemData $actual): void
    {
        // Original price
        $this->assertEquals($expected->getOriginalPrice(), $actual->getOriginalPrice());
        $this->assertPrice($expected->getBaseOriginalPrice(), $actual->getBaseOriginalPrice());

        // Price
        $this->assertEquals($expected->getPrice(), $actual->getPrice());
        $this->assertPrice($expected->getBasePrice(), $actual->getBasePrice());

        // Price includes tax
        $this->assertEquals($expected->getPriceInclTax(), $actual->getPriceInclTax());
        $this->assertPrice($expected->getBasePriceInclTax(), $actual->getBasePriceInclTax());

        // Tax before discount
        $this->assertPrice($expected->getTaxBeforeDiscount(), $actual->getTaxBeforeDiscount());
        $this->assertPrice($expected->getBaseTaxBeforeDiscount(), $actual->getBaseTaxBeforeDiscount());

        // Tax amount
        $this->assertPrice($expected->getTaxAmount(), $actual->getTaxAmount());
        $this->assertPrice($expected->getBaseTaxAmount(), $actual->getBaseTaxAmount());

        // Row total
        $this->assertEquals($expected->getRowTotal(), $actual->getRowTotal());
        $this->assertPrice($expected->getBaseRowTotal(), $actual->getBaseRowTotal());

        // Row total included tax
        $this->assertEquals($expected->getRowTotalInclTax(), $actual->getRowTotalInclTax());
        $this->assertPrice($expected->getBaseRowTotalInclTax(), $actual->getBaseRowTotalInclTax());

        // Row total with discount
        $this->assertPrice($expected->getRowTotalWithDiscount(), $actual->getRowTotalWithDiscount());

        // Discount amount
        $this->assertEquals($expected->getDiscountAmount(), $actual->getDiscountAmount());
        $this->assertPrice($expected->getBaseDiscountAmount(), $actual->getBaseDiscountAmount());

        // Discount tax compensation amount
        $this->assertPrice($expected->getDiscountTaxCompensationAmount(), $actual->getDiscountTaxCompensationAmount());
        $this->assertPrice($expected->getBaseDiscountTaxCompensationAmount(), $actual->getBaseDiscountTaxCompensationAmount());
    }

    private function assertPrice(float $expected, float $actual, float $delta = 0.0001): void
    {
        if (method_exists($this, 'assertEqualsWithDelta')) {
            $this->assertEqualsWithDelta($expected, $actual, $delta);
        } else {
            $this->assertEquals($expected, $actual, '', $delta);
        }
    }
}
