<?php

namespace Glopal\Connector\Test\Unit\Processor\CartTransfer;

use Glopal\Connector\Processor\CartTransfer\QuoteItems;
use Glopal\Connector\Provider\CartTransfer\CombineProductDataProvider;
use Glopal\Connector\Provider\CartTransfer\OrderOptionsProvider;
use Glopal\Connector\Provider\CartTransfer\SmallImageProvider;
use Glopal\Connector\Provider\CartTransfer\SpecificProductAttributesProvider;
use Magento\Catalog\Api\Data\ProductInterface;
use Magento\Catalog\Api\ProductRepositoryInterface;
use Magento\Catalog\Model\Product\Type\AbstractType;
use Magento\Framework\App\RequestInterface;
use Magento\Framework\DataObject;
use Magento\Quote\Api\Data\CartInterface;
use Magento\Quote\Api\Data\CartItemInterface;
use Magento\Quote\Model\Quote\Item\Option;
use PHPUnit\Framework\TestCase;

// phpcs:ignoreFile
class QuoteItemsTest extends TestCase
{
    private $productRepositoryMock;
    private $requestMock;
    private $combineProductDataProviderMock;
    private $smallImageProviderMock;
    private $specificProductAttributesProviderMock;
    private $orderOptionsProviderMock;
    private $cartInterfaceMock;
    private $cartItemInterfaceMock;
    private $modelProductMock;
    private $itemProductMock;
    private $simpleProductMock;
    private $optionMock;
    private $buyRequestInstanceMock;
    private $quoteItemsProcessor;

    protected function setUp(): void
    {
        $this->productRepositoryMock = $this->getMockForAbstractClass(
            ProductRepositoryInterface::class,
            [],
            '',
            false,
            false,
            true,
            ['getById']
        );
        $this->requestMock = $this->getMockForAbstractClass(
            RequestInterface::class,
            [],
            '',
            false,
            false,
            true,
            ['getParam']
        );
        $this->combineProductDataProviderMock = $this->createMock(CombineProductDataProvider::class);
        $this->smallImageProviderMock = $this->createMock(SmallImageProvider::class);
        $this->specificProductAttributesProviderMock = $this->createMock(SpecificProductAttributesProvider::class);
        $this->orderOptionsProviderMock = $this->createMock(OrderOptionsProvider::class);
        $this->cartInterfaceMock = $this->getMockForAbstractClass(
            CartInterface::class,
            [],
            '',
            false,
            false,
            true,
            ['getStoreId', 'getAllVisibleItems']
        );
        $this->cartItemInterfaceMock = $this->getMockForAbstractClass(
            CartItemInterface::class,
            [],
            '',
            false,
            false,
            true,
            ['getProductId', 'getData', 'getProduct', 'getOptionByCode', 'getBuyRequest']
        );
        $this->modelProductMock = $this->getMockForAbstractClass(
            ProductInterface::class,
            [],
            '',
            false,
            false,
            true,
            ['getData']
        );
        $this->itemProductMock = $this->getMockForAbstractClass(
            ProductInterface::class,
            [],
            '',
            false,
            false,
            true,
            ['getTypeInstance', 'getData']
        );
        $this->simpleProductMock = $this->getMockForAbstractClass(ProductInterface::class);
        $this->optionMock = $this->createMock(Option::class);
        $this->buyRequestInstanceMock = $this->getMockBuilder(DataObject::class)
            ->disableOriginalConstructor()
            ->setMethods(['getData'])
            ->getMock();

        $this->quoteItemsProcessor = new QuoteItems(
            $this->productRepositoryMock,
            $this->requestMock,
            $this->combineProductDataProviderMock,
            $this->smallImageProviderMock,
            $this->specificProductAttributesProviderMock,
            $this->orderOptionsProviderMock
        );
    }

    public function testProcess()
    {
        $this->prepareMocks(false);

        $expectedResult = [
            'wi_items' => [
                [
                    'item_id' => '5',
                    'quote_id' => '3',
                    'created_at' => '2023-03-16 16:23:37',
                    'updated_at' => '2023-03-16 16:23:37',
                    'product_id' => '1396',
                    'store_id' => '',
                    'product' => [],
                    'wi_product' => [
                        'name' => 'Product',
                        'description' => 'This is the product',
                        'price' => 15.00,
                        'small_image' => 'https://example.com/media/catalog/product/w/j/image.jpg',
                        'key' => 'some_value'
                    ],
                    'wi_order_options' => [
                        'attributes_info' => [
                            [
                                'label' => 'Size',
                                'value' => 'S',
                                'option_id' => '44',
                                'option_value' => '166',
                            ],
                        ],
                        'simple_name' => 'Product name',
                        'simple_sku' => 'SKU',
                    ],
                    'wi_buy_request' => [
                        'product' => '1396',
                        'super_attribute' => [
                            '144' => '166',
                            '93' => '49'
                        ],
                        'qty' => 1,
                        'original_qty' => 1
                    ],
                ]
            ]
        ];

        $this->quoteItemsProcessor->process($this->cartInterfaceMock, $expectedResult);
    }

    public function testProcessWithDebug()
    {
        $this->prepareMocks(true);

        $this->modelProductMock
            ->expects($this->once())
            ->method('getData')
            ->willReturn([
                'store_id' => 1,
                'entity_id' => '1396',
                'attribute_set_id' => '9',
            ]);

        $this->itemProductMock
            ->expects($this->once())
            ->method('getData')
            ->willReturn([
                'store_id' => 1,
                'entity_id' => '1396',
                'attribute_set_id' => '9',
            ]);

        $expectedResult = [
            'wi_items' => [
                [
                    'item_id' => '5',
                    'quote_id' => '3',
                    'created_at' => '2023-03-16 16:23:37',
                    'updated_at' => '2023-03-16 16:23:37',
                    'product_id' => '1396',
                    'store_id' => '',
                    'product' => [],
                    'wi_product' => [
                        'name' => 'Product',
                        'description' => 'This is the product',
                        'price' => 15.00,
                        'small_image' => 'https://example.com/media/catalog/product/w/j/image.jpg',
                        'key' => 'some_value'
                    ],
                    'wi_order_options' => [
                        'attributes_info' => [
                            [
                                'label' => 'Size',
                                'value' => 'S',
                                'option_id' => '44',
                                'option_value' => '166',
                            ],
                        ],
                        'simple_name' => 'Product name',
                        'simple_sku' => 'SKU',
                    ],
                    'wi_buy_request' => [
                        'product' => '1396',
                        'super_attribute' => [
                            '144' => '166',
                            '93' => '49'
                        ],
                        'qty' => 1,
                        'original_qty' => 1
                    ],
                    'wi_product_model' => [
                        'store_id' => 1,
                        'entity_id' => '1396',
                        'attribute_set_id' => '9',
                    ],
                    'wi_product_item' => [
                        'store_id' => 1,
                        'entity_id' => '1396',
                        'attribute_set_id' => '9',
                    ],

                ]
            ]
        ];

        $this->quoteItemsProcessor->process($this->cartInterfaceMock, $expectedResult);
    }

    private function prepareMocks(bool $withDebug): void
    {
        $storeId = 1;
        $this->cartInterfaceMock
            ->expects($this->once())
            ->method('getStoreId')
            ->willReturn($storeId);

        $this->cartInterfaceMock
            ->expects($this->once())
            ->method('getAllVisibleItems')
            ->willReturn([$this->cartItemInterfaceMock]);

        $this->cartItemInterfaceMock
            ->expects($this->once())
            ->method('getProductId')
            ->willReturn(10);

        $this->cartItemInterfaceMock
            ->expects($this->once())
            ->method('getData')
            ->willReturn([
                'item_id' => '5',
                'quote_id' => '3',
                'created_at' => '2023-03-16 16:23:37',
                'updated_at' => '2023-03-16 16:23:37',
                'product_id' => '1396',
                'store_id' => '',
            ]);

        $this->cartItemInterfaceMock
            ->expects($this->once())
            ->method('getProduct')
            ->willReturn($this->itemProductMock);

        $this->cartItemInterfaceMock
            ->expects($this->once())
            ->method('getOptionByCode')
            ->with($this->equalTo('simple_product'))
            ->willReturn($this->optionMock);

        $this->productRepositoryMock
            ->expects($this->once())
            ->method('getById')
            ->with(10, false, $storeId)
            ->willReturn($this->modelProductMock);

        $this->combineProductDataProviderMock
            ->expects($this->once())
            ->method('get')
            ->with($this->modelProductMock, $this->itemProductMock)
            ->willReturn([
                'name' => 'Product',
                'description' => 'This is the product',
                'price' => 15.00,
                'small_image' => 'image.jpg',
            ]);

        $this->smallImageProviderMock
            ->expects($this->once())
            ->method('get')
            ->with('image.jpg')
            ->willReturn('https://example.com/media/catalog/product/w/j/image.jpg');

        $this->specificProductAttributesProviderMock
            ->expects($this->once())
            ->method('get')
            ->with($this->modelProductMock, $storeId)
            ->willReturn(['key' => 'some_value']);

        $this->requestMock
            ->expects($this->once())
            ->method('getParam')
            ->with('debug')
            ->willReturn($withDebug);

        $this->optionMock
            ->expects($this->once())
            ->method('getProduct')
            ->willReturn($this->simpleProductMock);

        $itemProductTypeInstanceMock = $this->createMock(AbstractType::class);

        $this->itemProductMock
            ->expects($this->once())
            ->method('getTypeInstance')
            ->with($this->equalTo(true))
            ->willReturn($itemProductTypeInstanceMock);

        $itemProductTypeInstanceMock
            ->expects($this->once())
            ->method('getOrderOptions')
            ->with($this->itemProductMock)
            ->willReturn(['order' => 'options']);

        $this->orderOptionsProviderMock
            ->expects($this->once())
            ->method('get')
            ->with(['order' => 'options'], $this->simpleProductMock)
            ->willReturn([
                'attributes_info' => [
                    [
                        'label' => 'Size',
                        'value' => 'S',
                        'option_id' => '44',
                        'option_value' => '166',
                    ],
                ],
                'simple_name' => 'Product name',
                'simple_sku' => 'SKU',
            ]);

        $this->cartItemInterfaceMock
            ->expects($this->once())
            ->method('getBuyRequest')
            ->willReturn($this->buyRequestInstanceMock);

        $this->buyRequestInstanceMock
            ->expects($this->once())
            ->method('getData')
            ->willReturn([
                'product' => '1396',
                'super_attribute' => [
                    '144' => '166',
                    '93' => '49'
                ],
                'qty' => 1,
                'original_qty' => 1
            ]);
    }
}
