<?php

namespace Glopal\Connector\Test\Unit\Provider\CartTransfer;

use Glopal\Connector\Helper\Configuration;
use Glopal\Connector\Helper\Utils;
use Glopal\Connector\Provider\CartTransfer\CallBackUrlsProvider;
use Magento\Framework\UrlInterface;
use Magento\Quote\Api\Data\CartInterface;
use PHPUnit\Framework\TestCase;

// phpcs:ignoreFile
class CallBackUrlsProviderTest extends TestCase
{
    private $configuration;
    private $urlBuilder;
    private $utils;
    private $quote;

    protected function setUp(): void
    {
        $this->configuration = $this->getMockBuilder(Configuration::class)
            ->disableOriginalConstructor()
            ->setMethods(['getCartValue', 'getGeneralValue'])
            ->getMock();

        $this->urlBuilder = $this->getMockForAbstractClass(
            UrlInterface::class,
            [],
            '',
            false,
            false,
            true,
            ['getById']
        );

        $this->utils = $this->getMockBuilder(Utils::class)
            ->setMethods(['buildHmacKey'])
            ->getMock();

        $this->quote = $this->getMockForAbstractClass(
            CartInterface::class,
            [],
            '',
            false,
            false,
            true,
            ['getId']
        );
    }

    public function testGetInventoryCheckUrl()
    {
        $provider = new CallBackUrlsProvider($this->configuration, $this->urlBuilder, $this->utils);

        $this->configuration->expects($this->once())
            ->method('getCartValue')
            ->with('callback_url/inventory_check', null)
            ->willReturn('https://example.com/inventory_check');

        $this->assertEquals('https://example.com/inventory_check', $provider->getInventoryCheckUrl());
    }

    public function testGetCheckoutPurchaseUrl()
    {
        $provider = new CallBackUrlsProvider($this->configuration, $this->urlBuilder, $this->utils);

        $this->configuration->expects($this->once())
            ->method('getCartValue')
            ->with('callback_url/checkout_purchase', null)
            ->willReturn('https://example.com/checkout_purchase');

        $this->assertEquals(
            'https://example.com/checkout_purchase',
            $provider->getCheckoutPurchaseUrl($this->quote)
        );
    }

    public function testGetDefaultCheckoutPurchaseUrl()
    {
        $provider = new CallBackUrlsProvider($this->configuration, $this->urlBuilder, $this->utils);

        $this->configuration->expects($this->once())
            ->method('getCartValue')
            ->with('callback_url/checkout_purchase', null)
            ->willReturn('');

        $this->urlBuilder->expects($this->once())
            ->method('getUrl')
            ->with('glopal_connector/checkout/callbackPurchase')
            ->willReturn('https://example.com/glopal_connector/checkout/callbackPurchase');

        $this->quote->expects($this->once())
            ->method('getId')
            ->willReturn('123');

        $this->configuration->expects($this->once())
            ->method('getGeneralValue')
            ->with('key', null)
            ->willReturn('123456789');

        $this->utils->expects($this->once())
            ->method('buildHmacKey')
            ->with('123', '123456789')
            ->willReturn('bf0974fceaa1386bb3f086efdfdefd8e27be4b5fc1ee6ea3dcd226afa437cd1c');

        $this->assertEquals(
            'https://example.com/glopal_connector/checkout/callbackPurchase?hmac=bf0974fceaa1386bb3f086efdfdefd8e27be4b5fc1ee6ea3dcd226afa437cd1c',
            $provider->getCheckoutPurchaseUrl($this->quote)
        );
    }

    public function testGetCheckoutPurchaseUrlForPassingQuote()
    {
        $provider = new CallBackUrlsProvider($this->configuration, $this->urlBuilder, $this->utils);

        $this->configuration->expects($this->once())
            ->method('getCartValue')
            ->with('callback_url/checkout_purchase', null)
            ->willReturn('');

        $this->quote->expects($this->once())
            ->method('getId')
            ->willReturn('123');

        $this->configuration->expects($this->once())
            ->method('getGeneralValue')
            ->with('key', null)
            ->willReturn('123456789');

        $this->urlBuilder->expects($this->once())
            ->method('getUrl')
            ->with('glopal_connector/checkout/callbackPurchase')
            ->willReturn('https://example.com/glopal_connector/checkout/callbackPurchase');

        $this->utils->expects($this->once())
            ->method('buildHmacKey')
            ->with('123', '123456789')
            ->willReturn('bf0974fceaa1386bb3f086efdfdefd8e27be4b5fc1ee6ea3dcd226afa437cd1c');

        $this->assertEquals(
            'https://example.com/glopal_connector/checkout/callbackPurchase?hmac=bf0974fceaa1386bb3f086efdfdefd8e27be4b5fc1ee6ea3dcd226afa437cd1c',
            $provider->getCheckoutPurchaseUrl($this->quote)
        );
    }

    public function testGetCustomSuccessPageUrl()
    {
        $provider = new CallBackUrlsProvider($this->configuration, $this->urlBuilder, $this->utils);

        $this->urlBuilder->expects($this->once())
            ->method('getUrl')
            ->with('glopal_connector/checkout/success', ['_query' => ['returnTo' => 'https://example.com/']])
            ->willReturn('https://example.com/glopal_connector/checkout/success/?returnTo=https://example.com/');

        $this->assertEquals(
            'https://example.com/glopal_connector/checkout/success/?returnTo=https://example.com/',
            $provider->getCustomSuccessPageUrl('https://example.com/')
        );
    }
}
