<?php

namespace Glopal\Connector\Test\Unit\Response;

use Glopal\Connector\Helper\ModuleData;
use Glopal\Connector\Processor\CartTransfer;
use Glopal\Connector\Response\CartResponseBuilder;
use Glopal\Connector\Test\Unit\CartTransferTestHelper;
use Magento\Framework\App\RequestInterface;
use Magento\Framework\Serialize\Serializer\Base64Json;
use Magento\Quote\Api\Data\CartInterface;
use PHPUnit\Framework\TestCase;

// phpcs:ignoreFile
class CartResponseBuilderTest extends TestCase
{
    private $magentoVersion;
    private $connectorVersion;
    private $cartTransferProcessor;
    private $cartTransferProcessorMock;
    private $base64JsonSerializer;
    private $dataHelper;
    private $helper;
    private $request;
    private $quote;

    protected function setUp(): void
    {
        $this->magentoVersion = '2.4.3';
        $this->connectorVersion = '1.9.7';
        $this->dataHelper = $this->createMock(ModuleData::class);
        $this->request = $this->getMockForAbstractClass(RequestInterface::class);
        $this->cartTransferProcessorMock = $this->createMock(CartTransfer::class);
        $this->base64JsonSerializer = $this->createMock(Base64Json::class);

        $this->quote = $this->getMockForAbstractClass(
            CartInterface::class,
            [],
            '',
            false,
            false,
            true,
            ['getId', 'getStoreId', 'getAllVisibleItems', 'getData']
        );
        $this->helper = new CartTransferTestHelper();
        $this->cartTransferProcessor = $this->helper->getCartTransferProcessor($this->quote);
    }

    public function testBuildWithoutEncoding(): void
    {
        $this->dataHelper->method('getMagentoVersion')->willReturn($this->magentoVersion);
        $this->dataHelper->method('getExtensionVersion')->willReturn($this->connectorVersion);
        $this->request->method('getParam')->with('debug')->willReturn(false);

        $expectedResponse = [
            'magento_version' => $this->magentoVersion,
            'connector_version' => $this->connectorVersion,
            'quote' => $this->helper->getExpectedResult(),
        ];

        $cartResponseBuilder = new CartResponseBuilder(
            $this->cartTransferProcessor,
            $this->dataHelper,
            $this->base64JsonSerializer,
            $this->request
        );

        $actualResponse = $cartResponseBuilder->build($this->quote, false);

        $this->assertEquals($expectedResponse, $actualResponse);
    }

    public function testBuildWithEncoding(): void
    {
        $encodedQuoteData = 'base64_encoded_data';
        $expectedHash = $this->prepareQuoteWithEncodingDataAndHash($encodedQuoteData);

        $this->request->method('getParam')->with('debug')->willReturn(false);

        $expectedResponse = [
            'magento_version' => $this->magentoVersion,
            'connector_version' => $this->connectorVersion,
            'quote' => $encodedQuoteData,
            'hash' => $expectedHash,
        ];

        $cartResponseBuilder = new CartResponseBuilder(
            $this->cartTransferProcessor,
            $this->dataHelper,
            $this->base64JsonSerializer,
            $this->request
        );

        $actualResponse = $cartResponseBuilder->build($this->quote, true);

        $this->assertEquals($expectedResponse, $actualResponse);
    }

    public function testBuildWithEmptyQuote(): void
    {
        $encodedEmptyQuoteData = 'W10=';

        $quote = $this->getMockForAbstractClass(CartInterface::class);
        $this->dataHelper->method('getMagentoVersion')->willReturn($this->magentoVersion);
        $this->dataHelper->method('getExtensionVersion')->willReturn($this->connectorVersion);
        $this->cartTransferProcessorMock->method('process')->with($quote)->willReturn([]);
        $this->base64JsonSerializer->method('serialize')->with([])->willReturn($encodedEmptyQuoteData);
        $this->dataHelper->method('getGlopalKey')->willReturn('secretKey');
        $expectedHash = hash_hmac('sha256', $encodedEmptyQuoteData, 'secretKey');
        $this->request->method('getParam')->with('debug')->willReturn(false);

        $expectedResponse = [
            'magento_version' => $this->magentoVersion,
            'connector_version' => $this->connectorVersion,
            'quote' => $encodedEmptyQuoteData,
            'hash' => $expectedHash,
        ];

        $cartResponseBuilder = new CartResponseBuilder(
            $this->cartTransferProcessorMock,
            $this->dataHelper,
            $this->base64JsonSerializer,
            $this->request
        );

        $actualResponse = $cartResponseBuilder->build($quote, true);

        $this->assertEquals($expectedResponse, $actualResponse);
    }

    public function testBuildWithEncodingAndDebug(): void
    {
        $encodedQuoteData = 'base64_encoded_data';
        $expectedHash = $this->prepareQuoteWithEncodingDataAndHash($encodedQuoteData);
        $this->request->method('getParam')->with('debug')->willReturn(true);

        $expectedResponse = [
            'magento_version' => $this->magentoVersion,
            'connector_version' => $this->connectorVersion,
            'quote' => $encodedQuoteData,
            'hash' => $expectedHash,
            'quote_debug' => $this->helper->getExpectedResult(),
        ];

        $cartResponseBuilder = new CartResponseBuilder(
            $this->cartTransferProcessor,
            $this->dataHelper,
            $this->base64JsonSerializer,
            $this->request
        );

        $actualResponse = $cartResponseBuilder->build($this->quote, true);

        $this->assertEquals($expectedResponse, $actualResponse);
    }

    private function prepareQuoteWithEncodingDataAndHash(string $encodedQuoteData): string
    {
        $this->dataHelper->method('getMagentoVersion')->willReturn($this->magentoVersion);
        $this->dataHelper->method('getExtensionVersion')->willReturn($this->connectorVersion);
        $this->base64JsonSerializer->method('serialize')
            ->with($this->helper->getExpectedResult())
            ->willReturn($encodedQuoteData);
        $this->dataHelper->method('getGlopalKey')->willReturn('secretKey');

        return hash_hmac('sha256', $encodedQuoteData, 'secretKey');
    }
}
