<?php

namespace Glopal\Connector\Validator\OrderCreate;

use Glopal\Connector\Exception\OrderCreateValidationException;
use Magento\Framework\Phrase;

abstract class BaseOrderCreate implements OrderCreateValidatorInterface
{
    private const REQUIRED_CHARGE_FIELDS = [
        'amount_net',
        'tax_on_amount',
        'total',
    ];
    private const OPTIONAL_CHARGE_FIELDS = [
        'duty_on_amount',
        'tax_on_duty',
    ];

    /**
     * @inheritdoc
     */
    abstract public function validate(array $data): void;

    /**
     * Validate fields with data and zero numbers.
     *
     * @param string $key
     * @param array $data
     * @return void
     * @throws OrderCreateValidationException
     */
    protected function validateCharge(string $key, array $data): void
    {
        foreach (self::REQUIRED_CHARGE_FIELDS as $fieldName) {
            if ($this->isMissing($fieldName, $data)) {
                throw new OrderCreateValidationException(
                    new Phrase(sprintf('Required field %s is missing on %s', $fieldName, $key))
                );
            }

            if ($this->isNotNumber($fieldName, $data)) {
                throw new OrderCreateValidationException(
                    new Phrase(sprintf('Required field %s is empty on %s', $fieldName, $key))
                );
            }
        }

        foreach (self::OPTIONAL_CHARGE_FIELDS as $fieldName) {
            if ($this->isMissing($fieldName, $data) || $this->isNull($fieldName, $data)) {
                continue;
            }

            if ($this->isNotNumber($fieldName, $data)) {
                throw new OrderCreateValidationException(
                    new Phrase(sprintf('Optional field %s is empty on %s', $fieldName, $key))
                );
            }
        }
    }

    /**
     * Validate fields with data.
     *
     * @param array $structure
     * @param array $data
     * @return void
     * @throws OrderCreateValidationException
     */
    protected function validateFieldsWithData(array $structure, array $data): void
    {
        foreach ($structure as $fieldName) {
            if ($this->isMissing($fieldName, $data)) {
                throw new OrderCreateValidationException(
                    new Phrase(sprintf('Required field %s is missing', $fieldName))
                );
            }

            if ($this->isEmpty($fieldName, $data)) {
                throw new OrderCreateValidationException(
                    new Phrase(sprintf('Required field %s is empty', $fieldName))
                );
            }
        }
    }

    /**
     * Validate fields without data.
     *
     * @param array $structure
     * @param array $data
     * @return void
     * @throws OrderCreateValidationException
     */
    protected function validateFieldsWithoutData(array $structure, array $data): void
    {
        foreach ($structure as $fieldName) {
            if ($this->isMissing($fieldName, $data)) {
                throw new OrderCreateValidationException(
                    new Phrase(sprintf('Required field %s is missing', $fieldName))
                );
            }
        }
    }

    /**
     * Validate optional fields.
     *
     * @param array $structure
     * @param array $data
     * @return void
     * @throws OrderCreateValidationException
     */
    protected function validateOptionalFields(array $structure, array $data): void
    {
        foreach ($structure as $fieldName) {
            if ($this->isMissing($fieldName, $data) || null === $data[$fieldName]) {
                continue;
            }

            if ($this->isEmpty($fieldName, $data)) {
                throw new OrderCreateValidationException(
                    new Phrase(sprintf('Required field %s is empty', $fieldName))
                );
            }
        }
    }

    /**
     * Is Not isset?
     *
     * @param string $fieldName
     * @param array $data
     * @return bool
     */
    protected function isMissing(string $fieldName, array $data): bool
    {
        return !array_key_exists($fieldName, $data);
    }

    /**
     * Is empty?
     *
     * @param string $fieldName
     * @param array $data
     * @return bool
     */
    protected function isEmpty(string $fieldName, array $data): bool
    {
        return empty($data[$fieldName]);
    }

    /**
     * Is not number?
     *
     * @param string $fieldName
     * @param array $data
     * @return bool
     */
    protected function isNotNumber(string $fieldName, array $data): bool
    {
        return !is_numeric($data[$fieldName]);
    }

    /**
     * Is null?
     *
     * @param string $fieldName
     * @param array $data
     * @return bool
     */
    protected function isNull(string $fieldName, array $data): bool
    {
        return null === $data[$fieldName];
    }
}
