<?php

namespace Glopal\Connector\ViewModel;

use Glopal\Connector\Api\Data\OrderItemTotalCostInterface;
use Magento\Framework\View\Element\Block\ArgumentInterface;
use Magento\Sales\Api\Data\OrderInterface;
use Magento\Sales\Api\Data\OrderItemInterface;
use Magento\Tax\Model\Config;

class ColumnDutyViewModel implements ArgumentInterface
{
    /** @var Config */
    private $taxConfig;

    /**
     * @param Config $taxConfig
     */
    public function __construct(Config $taxConfig)
    {
        $this->taxConfig = $taxConfig;
    }

    /**
     * Display both prices?
     *
     * @param null|string|int $storeId
     * @return bool
     */
    public function displayBothPrices($storeId = null): bool
    {
        return $this->taxConfig->displaySalesPricesBoth($storeId);
    }

    /**
     * Display price excluded tax?
     *
     * @param null|string|int $storeId
     * @return bool
     */
    public function displayPriceExclTax($storeId = null): bool
    {
        return $this->taxConfig->displaySalesPricesExclTax($storeId);
    }

    /**
     * Display price included tax?
     *
     * @param null|string|int $storeId
     * @return bool
     */
    public function displayPriceInclTax($storeId = null): bool
    {
        return $this->taxConfig->displaySalesPricesInclTax($storeId);
    }

    /**
     * Get duty.
     *
     * @param OrderItemInterface $orderItem
     * @param OrderInterface $order
     * @return string|null
     */
    public function getDuty(OrderItemInterface $orderItem, OrderInterface $order): ?string
    {
        /** @var OrderItemTotalCostInterface[]|null $itemsTotalCost */
        $itemsTotalCost = $this->getItemsTotalCost($order);
        foreach ($itemsTotalCost as $itemTotalCost) {
            if ($itemTotalCost->getOrderItemId() == $orderItem->getId()) {
                return $this->displayPrices(
                    $order,
                    $itemTotalCost->getBaseDutyAmount(),
                    $itemTotalCost->getDutyAmount(),
                    '<br/>'
                );
            }
        }

        return null;
    }

    /**
     * Get duty included tax.
     *
     * @param OrderItemInterface $orderItem
     * @param OrderInterface $order
     * @return string|null
     */
    public function getDutyInclTax(OrderItemInterface $orderItem, OrderInterface $order): ?string
    {
        /** @var OrderItemTotalCostInterface[]|null $itemsTotalCost */
        $itemsTotalCost = $this->getItemsTotalCost($order);
        foreach ($itemsTotalCost as $itemTotalCost) {
            if ($itemTotalCost->getOrderItemId() == $orderItem->getId()) {
                return $this->displayPrices(
                    $order,
                    $itemTotalCost->getBaseDutyAmount() + $itemTotalCost->getBaseTaxOnDutyAmount(),
                    $itemTotalCost->getDutyAmount() + $itemTotalCost->getTaxOnDutyAmount(),
                    '<br/>'
                );
            }
        }

        return null;
    }

    /**
     * Get duty percent.
     *
     * @param OrderItemInterface $orderItem
     * @param OrderInterface $order
     * @return string|null
     */
    public function getDutyPercent(
        OrderItemInterface $orderItem,
        OrderInterface $order
    ): ?string {
        /** @var OrderItemTotalCostInterface[]|null $itemsTotalCost */
        $itemsTotalCost = $this->getItemsTotalCost($order);
        foreach ($itemsTotalCost as $itemTotalCost) {
            if ($itemTotalCost->getOrderItemId() == $orderItem->getId()) {
                return sprintf('%s%%', $itemTotalCost->getDutyPercent());
            }
        }

        return null;
    }

    /**
     * Get items total cost.
     *
     * @param OrderInterface $order
     * @return OrderItemTotalCostInterface[]|null
     */
    private function getItemsTotalCost(OrderInterface $order): ?array
    {
        $orderExtensionAttributes = $order->getExtensionAttributes();
        if (null === $orderExtensionAttributes) {
            return [];
        }

        if (!$orderExtensionAttributes->getGlItemsTcc()) {
            return [];
        }

        return $orderExtensionAttributes->getGlItemsTcc();
    }

    /**
     * Display prices.
     *
     * @param OrderInterface $order
     * @param float $basePrice
     * @param float $price
     * @param string $separator
     * @return string
     */
    private function displayPrices(
        OrderInterface $order,
        float $basePrice,
        float $price,
        string $separator = ' '
    ): string {
        $res = '';

        if ($order->isCurrencyDifferent()) {
            $res .= $order->formatBasePrice($basePrice);
            $res .= $separator;
            $res .= '[' . $order->formatPrice($price) . ']';
        } elseif ($order) {
            $res = $order->formatPrice($price);
        }

        return $res;
    }
}
